#!/itl/links/generic/bin/expect
# WebSubmit 2.0 genFileTranssubmit.cgi @(#)genFileTranssubmit.cgi	1.3 SCCS/s.genFileTranssubmit.cgi  99/05/20 

# Must use expect (expect + tclsh) shell to transfer binary files

# AUTHORS #############################################################

#  Authors: John Koontz, Ryan McCormack, Judith Devaney
#  Original Development Work: Robert Lipman, John Koontz, Katherine
#      Pagoaga

# Copyright Information ###############################################

# This software was developed at the National Institute of Standards
# and Technology by employees of the Federal Government in the course
# of their official duties.  Pursuant to title 17 Section 105 of the
# United States Code this software is not subject to copyright
# protection and is in the public domain.

# CODEINFO BEGIN ######################################################

# info script genFileTranssubmit.cgi> Simple file transfer.

# CODEINFO END ########################################################

# webSubmit setup
source /sasp/WebSubmit/webSubmit2.5/bin/websubmit.conf

cgi_debug -off

cgi_eval {
    # Variable aliases
    alias webSubmit dlHead wsDlDir mimeTypes mimeExtension

    # Process CGI input
    set moduleVarList [list Transfer.x Transfer.y clobber optList\
	hostName moduleName bqs mode title]
    set reqVarList [list Transfer.x Transfer.y clobber optList\
        hostName moduleName title]
    set importVarList [cgiImport $moduleVarList $reqVarList]

    # Set up client filenames (Names for temp local and original)
    set browserFilename [cgi_import_filename -client clientFile]
    set serverFilename [cgi_import_filename -server clientFile]

    # Determine direction of file transfer and perform simple checks
    #  to determine whether necessary filenames specified
    if [info exists Transfer.y] {
	if {${Transfer.y} >= 25} {
	    # Download file from server
            set dir download
	    if {$remFile == ""} {
		cgiBomb genFileTranssubmit.cgi "No remote file specified for\
                    download"
		exit
	    } 
	    if {![file isdirectory $wsDlDir]} {
		set messg "Internal error:  the WebSubmit\
                    Server's directory for downloads, $wsDlDir, is not set up\
                    properly."
		cgiAdminMail genFileTranssubmit.cgi $messg
		cgiBomb genFileTranssubmit.cgi "$messg\
                    The WebSubmit Administrator has been notified."
	    }
	} else {
	    # Upload client file to server
	    set dir upload
	    if {$browserFilename == ""} {
		cgiBomb genFileTranssubmit.cgi "No client file specified for upload"
		exit
	    }
	}
    } else {
	cgiBomb genFileTranssubmit.cgi "Cannot establish direction of transfer"
	exit
    }

    # Set up arguments to scp
    set scpArgs ""
    set optList [split [string trim $optList \{\}]]
    foreach var $optList {
	if {($var != "clobber") && ([set $var] != "") && ![string match *pad* $var]} {
	    lappend scpArgs [set $var]
	}
    }

    # Get information about remote host
    set remUser [webSubmit::auth::authTrans $hostName]
    set remHome [webSubmit::network::getRemEnv HOME $remUser $hostName]

    # Remote filename resolution
    if {$remFile == ""} {
	# Home directory if none specified
	set remFile $remHome
    } else {
	# Check for wildcards and resolve path wrt home directory
	if [regexp {[\*\?]} $remFile] {
	    cgiBomb genFileTranssubmit.cgi "Remote filename contains wildcard characters.  Multiple file transfers\
		not currently supported"
	} else {
	    # Resolve remote filename wrt home directory
	    set remFile [mkPathAbs $remFile $remHome]
	}
    }


    #############################################
    # Perform File Transfer Actions
    #############################################
    set remExist [webSubmit::network::remFileExist $remUser $hostName $remFile]
    if {$remExist} {
	set remType [webSubmit::network::remFileType $remUser $hostName $remFile]
    }
    if {$dir == "download"} {
	# File downloaded from remote system to client
	if {$remExist} {
	    if {$remType != "directory"} {
		# SCP File from remote to server and save in tmpFile
		#  - Always overwrite temporary file
		set fName [file tail $remFile]
		set tmpFile [file join $wsDlDir $fName]
		if [file exists $tmpFile] {
		    set idx 1
		    set tmpFile [file join $wsDlDir $fName.$idx]
		    while [file exists $tmpFile] {
			incr idx
			set tmpFile [file join $wsDlDir $fName.$idx]
		    }
		}

		# Add extension if download MIME types are configured by extension
		if {$mimeTypes != "directory"} {
		    set tmpFile "$tmpFile.$mimeExtension"
		}
		if [catch {webSubmit::network::scpFile $tmpFile get 1 $remUser $hostName $remFile $scpArgs} result] {
		    cgiBomb genFileTranssubmit.cgi "An error occurred during file transfer ($result)"
		} else {
		    tclChmod 600 $tmpFile
		}
	    } else {
		cgiBomb genFileTranssubmit.cgi "Transfer of entire remote directories is not currently supported"
	    }
	} else {
	    cgiBomb genFileTranssubmit.cgi "Download specified but remote file \"$remFile\" does not exist"
	}
    } else {
	# File uploaded from client system to remote
	if {$remExist} {
	    if {$remType == "directory"} {
		# Store client file in specified directory
		# Set remote filename to match the name on the browser system
		set remFile [file join $remFile [file tail $browserFilename]]

		# Test for the existence of this file and compare with overwrite
		set remExist [webSubmit::network::remFileExist $remUser $hostName $remFile]
		if {$remExist && !($clobber)} {
		    cgiBomb genFileTranssubmit.cgi "Target file \"$remFile\" for upload exists, but overwrite not specified"
		}
	    } else {
		if {!($clobber)} {
		    # Remote file exists, but no overwrite specified
		    cgiBomb genFileTranssubmit.cgi "Target file \"$remFile\" for upload exists, but overwrite not specified"
		}
	    }
	} else {
	    # Specified target file does not exist
	    #  If the directory corresponding to this file exists, then write the client
	    #  file into this directory.  Otherwise, raise an error
	    set remDir [file dirname $remFile]
	    if ![webSubmit::network::remFileExist $remUser $hostName $remDir] {
		cgiBomb genFileTranssubmit.cgi "Target file does not exist and does not contain a valid\
		    directory path.  Transfer aborted"
	    }
	}
    }

    set workingTitle "$title Report"

    # Generate HTML via CGI

    if {$dir == "download"} {
	# HTTP Header to redirect and invoke save dialog
	cgi_http_head {
	    cgi_redirect "$dlHead/[file tail $tmpFile]"
	}
    } else {
	# Standard HTTP header
	cgi_http_head
	
	cgi_html {
	    cgi_head {
		cgi_title "$workingTitle"
	    }
	    
	    cgi_body link=antiquewhite bgcolor=$masterBgcolor {
		# Basic HTML form elements
		cgi_h4 "[cgi_font color=blue [cgi_bold "Remote Host:"]] $hostName"
		cgi_h4 "[cgi_font color=blue [cgi_bold "Remote User:"]] $remUser" 
		cgi_h4 "[cgi_font color=blue [cgi_bold "Client File:"]] $browserFilename"
		cgi_h4 "[cgi_font color=blue [cgi_bold "Remote File:"]] $remFile"
		cgi_h4 "[cgi_font color=blue [cgi_bold "Transfer Direction:"]] $dir"
		cgi_br

		# Transfer File
		cgi_h3 "File Transfer in progress...Please wait"
		if [catch {webSubmit::network::scpFile $serverFilename put $clobber $remUser $hostName $remFile $scpArgs} result] {
		    cgiWarn genFileTranssubmit.cgi "An error occurred during file transfer ($result)"
		}
		cgi_br 

		cgi_h3 "$workingTitle"
		puts "[cgi_sample $result]"
	    }
	}
    }
 }

# Cleanup
catch {file delete -force $serverFilename}









