#!/itl/links/generic/bin/tclsh8.0
# WebSubmit 2.0 llGaussiansubmit.cgi @(#)llGaussiansubmit.cgi	1.2 SCCS/s.llGaussiansubmit.cgi  99/05/20 

# AUTHORS #############################################################

#  Authors: John Koontz, Ryan McCormack, Judith Devaney
#  Original Development Work: Robert Lipman, John Koontz, Katherine
#      Pagoaga

# Copyright Information ###############################################

# This software was developed at the National Institute of Standards
# and Technology by employees of the Federal Government in the course
# of their official duties.  Pursuant to title 17 Section 105 of the
# United States Code this software is not subject to copyright
# protection and is in the public domain.

# CODEINFO BEGIN ######################################################

# info script llGaussiansubmit.cgi> LL Gaussian job submission processor.

# CODEINFO END ########################################################

# Load websubmit information
source /sasp/WebSubmit/webSubmit2.5/bin/websubmit.conf

catch {namespace import webSubmit::loadleveler::*}
catch {namespace import webSubmit::forms::*}

alias webSubmit GaussianVersions

cgi_debug -off

cgi_eval {
    # Read Load Leveler Keyword Database
    createKeywordDb

    # Variable aliasing
    eval alias webSubmit::loadleveler classNameList Gaussian $webSubmit::loadleveler::dbKeyList

    # Initialize module variables
    set moduleVarList [list remGaussianFile locGaussianFile clobberGaussian\
        Gaussian(OutputType) Gaussian(Nproc) Gaussian(JobNameTxt)\
        Gaussian(Output) Gaussian(NProcType) loadedSessName initialdir class\
        job_name notify_user gvers.option]
    set reqVarList [list hostName title initialdir class notify_user job_name\
	Gaussian(OutputType) Gaussian(NProcType) gvers.option]
    set varList [cgiImport $moduleVarList $reqVarList]
    
    # Get Load Leveler Class Information
    getClassInfo $hostName

    # Get remote information
    set remUser [webSubmit::auth::authTrans $hostName]
    set remHome [webSubmit::network::getRemEnv HOME $remUser $hostName]

    # Verify CGI Input data
    set initialdir [mkPathAbs $initialdir $remHome]
    set remGaussianFile [mkPathAbs $remGaussianFile $initialdir]
    set INPUT(varList) [list initialdir remGaussianFile Gaussian(Nproc)]
    set INPUT(initialdir) [list noWhite isDirectory]
    set INPUT(initialdir,isDirectory,hostName) $hostName
    set INPUT(initialdir,isDirectory,remUser) $remUser
    set INPUT(remGaussianFile) [list noWhite remFileExist]
    set INPUT(remGaussianFile,remFileExist,hostName) $hostName
    set INPUT(remGaussianFile,remFileExist,remUser) $remUser
    set INPUT(Gaussian(Nproc)) isPositiveInt
    inputVerify INPUT

    ###############################################
    # Process form data
    ###############################################

    #  Change "/tmp/CGI[pid]-[verno] {}" returned by cgi_file_button for
    #  empty file into null string as expected in code
    if [regexp { \{\}$} $locGaussianFile] {
	set locGaussianFile ""
    }

    ##################################################################################
    # Analyze Gaussian directives file specification
    #
    #  Users must specify either a remote directives file or a file to upload from the
    #  browser system.  If a remote file is specified, it is copied to the WebSubmit
    #  server so that it can be processed as needed.  If a local file is specified to
    #  upload, then the new directives file on the remote system is given the same
    #  name as the uploaded file.  If two directives files are specified, an error
    #  is generated.
    #
    #  tmpGaussianFile = Name of temporary Gaussian directives file on WebSubmit server.  File
    #                is obtained either via upload or by copying it from the remote system
    # 
    #  remGaussianFile = Name of Gaussian file on remote system
    #
    #  locGaussianFile = If non-null, list containing name of uploaded file on WebSubmit server,
    #                  and original name of file on browser system
    ##################################################################################
    set tmpGaussianFile ""
    if {$locGaussianFile == "" && $remGaussianFile == ""} {
	# No directives file
	cgiBomb "llGaussiansubmit.cgi for $hostName"\
                "No Gaussian directives file specified"
    } elseif {$locGaussianFile != "" && $remGaussianFile != ""} {
	# Two directives files specified - inconsistent input
	cgiBomb "llGaussiansubmit.cgi for $hostName"\
                "Inconsistent input - two Gaussian directives files specified"
    } else {
	if {$remGaussianFile != ""} {
	    # Attempt to copy remote Gaussian directives file for processing
	    # Input file path is relative to working directory (initialdir)
	    set remGaussianFile [mkPathAbs $remGaussianFile $initialdir]
	    set tmpGaussianFile "/tmp/tmpGaussian.[pid]"
	    set clobberGaussian 1
	    if [catch {webSubmit::network::scpFile $tmpGaussianFile get 1 $remUser $hostName $remGaussianFile} result] {
		cgiBomb  "llGaussiansubmit.cgi for $hostName" "$result"
	    }
	} else {
	    # Directives file has been uploaded from user browser system
	    set tmpGaussianFile [lindex $locGaussianFile 0]

	    # Name of remote Gaussian file set to match name of file on browser system
	    set tmpName [file tail [lindex $locGaussianFile 1]]
	    set remGaussianFile [mkPathAbs $tmpName $initialdir]
	}
    }
    
    # Set up job_name based on input values
    if {$job_name == "IF"} {
	# Job name based on directives file name
	set job_name "[file tail $remGaussianFile].job"
    } elseif {$job_name == "UD"} {
	# User-defined job name
	set job_name "$Gaussian(JobNameTxt).job"
    } else {
	set job_name job
    }

    # Set number of processors if user-defined and not provided
    if {$Gaussian(NProcType) == "UD" && $Gaussian(Nproc) == ""} {
	set Gaussian(Nproc) 1
    }

    ######################
    # Generate HTML
    ######################
    cgi_http_head
    
    set workingTitle "$bqs $title Report"

    cgi_html {
	cgi_head {cgi_title "$workingTitle"}
	cgi_body bgcolor=$masterBgcolor {
	    cgi_center {
		cgi_h2 "$workingTitle[cgi_nl]\
		    System [cgi_font color=blue "$hostName"]"
	    }

	    # Submit Gaussian Job
	    submitGaussianJob $hostName $remUser ${gvers.option}\
               $clobberGaussian $tmpGaussianFile $remGaussianFile
	}
    }
}







































