#!/itl/links/generic/bin/tclsh8.0
# WebSubmit 2.0 llJobsubmit.cgi @(#)llJobsubmit.cgi	1.4 SCCS/s.llJobsubmit.cgi  99/07/12 

# AUTHORS #############################################################

#  Authors: John Koontz, Ryan McCormack, Judith Devaney
#  Original Development Work: Robert Lipman, John Koontz, Katherine
#      Pagoaga

# Copyright Information ###############################################

# This software was developed at the National Institute of Standards
# and Technology by employees of the Federal Government in the course
# of their official duties.  Pursuant to title 17 Section 105 of the
# United States Code this software is not subject to copyright
# protection and is in the public domain.

# CODEINFO BEGIN ######################################################

# info script llJobsubmit.cgi> LL job submission processor.

# CODEINFO END ########################################################

# Load websubmit information
source /sasp/WebSubmit/webSubmit2.5/bin/websubmit.conf

cgi_debug -off

cgi_eval {
    # HTTP Header
    cgi_http_head

    catch {namespace import webSubmit::loadleveler::*}
    catch {namespace import webSubmit::forms::*}

    # Variable aliasing
    alias webSubmit::loadleveler jobScript LLPREF LLREQ

    # Read Load Leveler Keyword Database
    createKeywordDb

    # Variable aliasing
    eval alias webSubmit::loadleveler $webSubmit::loadleveler::dbKeyList

    # Module Variable Initialization
    set moduleVarList [list initialdir executable arguments input output\
       error class min_processors max_processors job_name job_type useReqs\
       LLREQ(Memory) LLREQ(Adapter) LLREQ(Feature) env_Copy env_CopyString\
       env_ExcludeString env_CopyValueString usePrefs LLPREF(Memory)\
       LLPREF(Adapter) LLPREF(Feature) jobScript shell startdate hold\
       parallel_path loadedSessName hostName title]
    set reqVarList [list class job_type hostName title ]
    set varList [cgiImport $moduleVarList $reqVarList]

    # Get Load Leveler Class Information
    getClassInfo $hostName

    # Trim white space from selected variables
    set whiteVarList [list initialdir input output error min_processors\
        max_processors job_name executable arguments LLREQ(Memory)\
        env_CopyString env_ExcludeString env_CopyValueString LLPREF(Memory)]
    trimWhite $whiteVarList

    # Get remote information
    set remUser [webSubmit::auth::authTrans $hostName]
    set remHome [webSubmit::network::getRemEnv HOME $remUser $hostName]

    # Get user notification email address from auth.db
    webSubmit::auth::createAuthDb
    set notify_user [webSubmit::database::getDbVal webSubmit::auth $webSubmit::auth::wsID email]

    # Path resolution and Initialization
    set initialdir [mkPathAbs $initialdir $remHome]
    foreach file [list executable input output error] {
	set value [set $file]
	if {$value != ""} {
	    set $file [mkPathAbs $value $initialdir]
	}
    }

    # CGI Input Verification
    set INPUT(varList) $moduleVarList
    set INPUT(min_processors) isPositiveInt
    set INPUT(max_processors) isPositiveInt
    set INPUT(executable) [list noWhite isExecutable]
    set INPUT(executable,isExecutable,hostName) $hostName
    set INPUT(executable,isExecutable,remUser) $remUser
    set INPUT(initialdir) [list noWhite isDirectory]
    set INPUT(initialdir,isDirectory,hostName) $hostName
    set INPUT(initialdir,isDirectory,remUser) $remUser
    set INPUT(input) [list noWhite remFileExist]
    set INPUT(input,remFileExist,hostName) $hostName
    set INPUT(input,remFileExist,remUser) $remUser
    set INPUT(output) noWhite
    set INPUT(error) noWhite
    set INPUT(LLPREF(Memory)) isPositiveInt
    set INPUT(LLREQ(Memory)) isPositiveInt
    set INPUT(job_name) noWhite
    set INPUT(parallel_path) regexpMatch
    set INPUT(parallel_path,regexpMatch,pattern) {[^:]+ *: *[^:]+}
    inputVerify INPUT


    ###############################################
    # Process form data
    ###############################################

    # Job Script / Executable Consistency checking
    set warnExec 0
    if [regexp {[a-zA-Z0-9]+} $jobScript] {
	if {$executable != ""} {
	    set executable ""
	    set arguments ""
	    set warnExec 1
	}
    } else {
	set jobScript ""
	set shell ""
    }

    # Build requirements and preferences strings
    if {$usePrefs != ""} {
	set preferences [buildLLBoolean LLPREF]
    }

    if {$useReqs != ""} {
	set requirements [buildLLBoolean LLREQ]
    }

    # Build any user-defined environment variables
    set envString "LL_JOB=TRUE;"
    if {$env_Copy != ""} {
	append envString "COPY_ALL;"
    }

    if {$env_CopyString != ""} {
	regsub -all " " $env_CopyString "" env_CopyString
	foreach elt [split $env_CopyString ,] {
	    append envString "\$$elt;"
	}
    }

    if {$env_ExcludeString != ""} {
	regsub -all " " $env_ExcludeString "" env_ExcludeString
	foreach elt [split $env_ExcludeString ,] {
	    append envString "!$elt;"
	}
    }

    if {$env_CopyValueString != ""} {
	regsub -all " " $env_CopyValueString "" env_CopyValueString
	foreach elt [split $env_CopyValueString ,] {
	    set elt [string trim $elt \{\}]
	    append envString "$elt;"
	}
    }
    set environment [string trimright $envString \;]  

    set workingTitle "$bqs $title Report"

    cgi_html {
	cgi_head {cgi_title "$workingTitle"}
	cgi_body bgcolor=$masterBgcolor {
	    cgi_center {
		cgi_h2 "$workingTitle[cgi_nl]\
		    System [cgi_font color=blue "$hostName"]"
	    }

	    if {$warnExec} {
		cgiWarn llJobsubmit.cgi "Executable shell script will\
                    override specified executable code"
	    }

	    # Verify consistency of class and number of processors
	    if [getValidNumProc $class] {
		cgiWarn llJobsubmit.cgi "Class and number of processors\
                        specified are inconsistent.  Number\
			of processors revised to match class"
	    }

	    if {$job_type != "pvm3" && $parallel_path != ""} {
		cgiWarn llJobsubmit.cgi "Parallel path information is only\
                    required for pvm3 jobs"
	    }

	    # Submit Job
	    submitLLJob $hostName $remHome $remUser
	}
    }
}
















































































































































































