#!/itl/links/generic/bin/tclsh8.0
# WebSubmit 2.0 llMPIsubmit.cgi @(#)llMPIsubmit.cgi	1.2 SCCS/s.llMPIsubmit.cgi  99/05/20 

# AUTHORS #############################################################

#  Authors: John Koontz, Ryan McCormack, Judith Devaney
#  Original Development Work: Robert Lipman, John Koontz, Katherine
#      Pagoaga

# Copyright Information ###############################################

# This software was developed at the National Institute of Standards
# and Technology by employees of the Federal Government in the course
# of their official duties.  Pursuant to title 17 Section 105 of the
# United States Code this software is not subject to copyright
# protection and is in the public domain.

# CODEINFO BEGIN ######################################################

# info script nqsGaussiansubmit.cgi> LL MPI job submission processor.

# CODEINFO END ########################################################

# Load websubmit information
source /sasp/WebSubmit/webSubmit2.5/bin/websubmit.conf

cgi_debug -off

cgi_eval {
    # HTTP Header
    cgi_http_head

    catch {namespace import webSubmit::loadleveler::*}
    catch {namespace import webSubmit::forms::*}

    # Variable aliasing
    alias webSubmit::loadleveler jobScript LLPREF LLREQ inputFileList outputFileList 

    # Read Load Leveler Keyword Database
    createKeywordDb

    # Variable aliasing
    eval alias webSubmit::loadleveler $webSubmit::loadleveler::dbKeyList

    # Module Variable initialization
    set moduleVarList [list initialdir executable input output error\
        class min_processors max_processors job_name userExec userArgs\
        job_type useReqs LLREQ(Memory) LLREQ(Adapter) LLREQ(Feature)\
	env_Copy env_CopyString env_ExcludeString env_CopyValueString\
        usePrefs LLPREF(Memory) LLPREF(Adapter) LLPREF(Feature) files2nodes\
        inputFileString outputFileString returnHost returnDir cpOut\
        jobDescription loadedSessName hostName title]
    set reqVarList [list class job_type hostName title ]
    set varList [cgiImport $moduleVarList $reqVarList]

    # Get Load Leveler Class Information
    getClassInfo $hostName

    # Trim white space from selected variables
    set whiteVarList [list initialdir input output error min_processors\
        max_processors job_name userExec userArgs LLREQ(Memory)\
        env_CopyString env_ExcludeString env_CopyValueString LLPREF(Memory)\
        inputFileString outputFileString returnHost returnDir]
    trimWhite $whiteVarList

    # Get remote information
    set remUser [webSubmit::auth::authTrans $hostName]
    set remHome [webSubmit::network::getRemEnv HOME $remUser $hostName]

    # Get user notification email address from auth.db
    webSubmit::auth::createAuthDb
    set notify_user [webSubmit::database::getDbVal webSubmit::auth $webSubmit::auth::wsID email]

    # Path resolution and Initialization
    set initialdir [mkPathAbs $initialdir $remHome]
    foreach file [list userExec input output error] {
	set value [set $file]
	if {$value != ""} {
	    set $file [mkPathAbs $value $initialdir]
	}
    }

    # CGI Input Verification
    set INPUT(varList) $moduleVarList
    set INPUT(min_processors) isPositiveInt
    set INPUT(max_processors) isPositiveInt
    set INPUT(userExec) [list noWhite isExecutable]
    set INPUT(userExec,isExecutable,hostName) $hostName
    set INPUT(userExec,isExecutable,remUser) $remUser
    set INPUT(executable) [list noWhite isExecutable]
    set INPUT(executable,isExecutable,hostName) $hostName
    set INPUT(initialdir) [list noWhite isDirectory]
    set INPUT(initialdir,isDirectory,hostName) $hostName
    set INPUT(initialdir,isDirectory,remUser) $remUser
    set INPUT(input) [list noWhite remFileExist]
    set INPUT(input,remFileExist,hostName) $hostName
    set INPUT(input,remFileExist,remUser) $remUser
    set INPUT(output) noWhite
    set INPUT(error) noWhite
    set INPUT(returnHost) isValidHost
    set INPUT(returnDir) [list noWhite isDirectory]
    set INPUT(returnDir,isDirectory,hostName) $returnHost
    set INPUT(returnDir,isDirectory,remUser) $remUser
    set INPUT(LLPREF(Memory)) isPositiveInt
    set INPUT(LLREQ(Memory)) isPositiveInt
    set INPUT(job_name) noWhite
    inputVerify INPUT


    ###############################################
    # Process form data
    ###############################################

    # POE and Job Script Evaluation
    set warnPoe 0
    set warnExec 0
    if {$files2nodes != ""} {
	# Set up input and output file lists
	if {$inputFileString != ""} {
	    regsub -all " " $inputFileString "" inputFileString
	    set inputFileList [split $inputFileString ,]
	}

	if {$outputFileString != ""} {
	    regsub -all " " $outputFileString "" outputFileString
	    set outputFileList [split $outputFileString ,]
	}

	if [regexp {[a-zA-Z0-9]+} $jobScript] {
	    set warnPoe 1
	}
	
	if {$userExec != ""} {
	    if {$cpOut != ""} {
		if {$returnHost == ""} {
		    cgiBomb sp2MPIsubmit.cgi "POE Execution with returned output\
			    specified, but no return host provided"
		}
	    } else {
		set cpOut 0
	    }

	    # Build POE Script
	    if {$input == "/dev/null"} {
		set input ""
	    }
	    buildPoeScript $remUser $hostName $userExec $userArgs $input $cpOut $returnDir
	    set shell /bin/ksh
	    set executable ""
	    set arguments ""
	} else {
	    cgiBomb sp2MPIsubmit.cgi "POE Execution specified, but no executable code name given"
	}
    } else {
	if [regexp {[a-zA-Z0-9]+} $jobScript] {
	    if {$executable != ""} {
		set executable ""
		set arguments ""
		set warnExec 1
	    }
	} else {
	    set arguments [string trimright "$userExec $userArgs"]
	    set jobScript ""
	    set shell ""
	}
    }

    # Build requirements and preferences strings
    if {$usePrefs != ""} {
	set preferences [buildLLBoolean LLPREF]
    }

    if {$useReqs != ""} {
	set requirements [buildLLBoolean LLREQ]
    }

    # Set MP_EUILIB environment variabel is using 'Adapter == hps_user'
    if {$LLREQ(Adapter) == "hps_user"} {
	set envString "MP_EUILIB=us;"
    } else {
	set envString ""
    }

    # Build any user-defined environment variables
    if {$env_Copy != ""} {
	append envString "COPY_ALL;"
    }

    if {$env_CopyString != ""} {
	regsub -all " " $env_CopyString "" env_CopyString
	foreach elt [split $env_CopyString ,] {
	    append envString "\$$elt;"
	}
    }

    if {$env_ExcludeString != ""} {
	regsub -all " " $env_ExcludeString "" env_ExcludeString
	foreach elt [split $env_ExcludeString ,] {
	    append envString "!$elt;"
	}
    }

    if {$env_CopyValueString != ""} {
	regsub -all " " $env_CopyValueString "" env_CopyValueString
	foreach elt [split $env_CopyValueString ,] {
	    set elt [string trim $elt \{\}]
	    append envString "$elt;"
	}
    }

    # Additional environment variables based on job description
    # if {$jobDescription == "MPI"} {
    #     append envString "MP_EUILIB=us;"
    # }

    set environment [string trimright $envString \;] 
 
    set workingTitle "$bqs $title Report"

    cgi_html {
	cgi_head {cgi_title "$workingTitle"}
	cgi_body bgcolor=$masterBgcolor {
	    cgi_center {
		cgi_h2 "$workingTitle[cgi_nl]\
		    System [cgi_font color=blue "$hostName"]"
	    }

	    if {$warnPoe} {
		cgiWarn sp2MPIsubmit.cgi "POE Execution specified; executable\
                    shell script ignored"
	    }

	    if {$warnExec} {
		cgiWarn sp2MPIsubmit.cgi "Executable shell script will\
                    override specified executable code"
	    }

	    # Verify consistency of class and number of processors
	    if [getValidNumProc $class] {
		cgiWarn sp2MPIsubmit.cgi "Class and number of processors\                         specified inconsistent.  Number of processors revised\
                    to match class"
	    }

	    # Submit Parallel Job
	    submitLLJob $hostName $remHome $remUser
	}
    }
}




































































































































































