#!/itl/links/generic/bin/tclsh8.0
# WebSubmit 2.0 lsfJobsubmit.cg @(#)lsfJobsubmit.cgi	1.3 SCCS/s.lsfJobsubmit.cgi  99/05/20 

# AUTHORS #############################################################

#  Authors: John Koontz, Ryan McCormack, Judith Devaney
#  Original Development Work: Robert Lipman, John Koontz, Katherine
#      Pagoaga

# Copyright Information ###############################################

# This software was developed at the National Institute of Standards
# and Technology by employees of the Federal Government in the course
# of their official duties.  Pursuant to title 17 Section 105 of the
# United States Code this software is not subject to copyright
# protection and is in the public domain.

# CODEINFO BEGIN ######################################################

# info script lsfJobsubmit.cgi> LSF job submission processor.

# CODEINFO END ########################################################

# Load websubmit information
source /sasp/WebSubmit/webSubmit2.5/bin/websubmit.conf

cgi_debug -off

cgi_eval {
    # HTTP Header
    cgi_http_head

    catch {namespace import webSubmit::lsf::*}

    # Variable aliasing
    alias webSubmit::lsf BSUBOPTS BSUBFLAGS jobFileName jobScript workDir inputFileList outputFileList workDir shell

    # Initialize module variables
    set moduleVarList [list workDir jobScript BSUBOPTS(-o) BSUBOPTS(-i)\
        BSUBOPTS(-e) BSUBOPTS(-J) BSUBOPTS(-c) BSUBOPTS(-M)\
	BSUBOPTS(-b) BSUBOPTS(-q)\
        minProc maxProc jobFileName cpIn cpOut inputFileString\
        outputFileString hostName moduleName shell loadedSessName]
    set reqVarList [list hostName title bqs BSUBOPTS(-q) jobScript]
    set varList [cgiImport $moduleVarList $reqVarList]

    set whiteVarList [list workDir BSUBOPTS(-i) BSUBOPTS(-e) BSUBOPTS(-o)\
        BSUBOPTS(-J) minProc maxProc jobFileName BSUBOPTS(-M)]
    trimWhite $whiteVarList

    # Get remote information
    set remUser [webSubmit::auth::authTrans $hostName]
    set remHome [webSubmit::network::getRemEnv HOME $remUser $hostName]

    # Set up input and output file lists
    if {$inputFileString != ""} {
	regsub -all " " $inputFileString "" inputFileString
	set inputFileList [split $inputFileString ,]
    }
    
    if {$outputFileString != ""} {
	regsub -all " " $outputFileString "" outputFileString
	set outputFileList [split $outputFileString ,]
    }

    # Path resolution
    set workDir [mkPathAbs $workDir $remHome]
    foreach flag [list -o -i -e] {
	if {$BSUBOPTS($flag) != ""} {
	    set BSUBOPTS($flag) [mkPathAbs $BSUBOPTS($flag) $workDir]
	}
    }

    # Verify CGI input information
    set INPUT(varList) [list workDir BSUBOPTS(-M) BSUBOPTS(-o) BSUBOPTS(-i) BSUBOPTS(-e) BSUBOPTS(-J) minProc maxProc jobFileName]
    set INPUT(workDir) [list noWhite isDirectory]
    set INPUT(workDir,isDirectory,hostName) $hostName
    set INPUT(workDir,isDirectory,remUser) $remUser
    set INPUT(BSUBOPTS(-M)) isPositiveInt
    set INPUT(BSUBOPTS(-o)) noWhite
    set INPUT(BSUBOPTS(-i)) [list noWhite remFileExist]
    set INPUT(BSUBOPTS(-i),remFileExist,hostName) $hostName
    set INPUT(BSUBOPTS(-i),remFileExist,remUser) $remUser
    set INPUT(BSUBOPTS(-e)) noWhite
    set INPUT(BSUBOPTS(-J)) noWhite
    set INPUT(minProc) isPositiveInt
    set INPUT(maxProc) isPositiveInt
    set INPUT(jobFileName) noWhite
    set i 0
    foreach file $inputFileList {
	set dummyVar$i $file
	set INPUT(dummyVar$i) remFileExist
	set INPUT(dummyVar$i,remFileExist,hostName) $hostName
	set INPUT(dummyVar$i,remFileExist,remUser) $remUser
	incr i
    }
    inputVerify INPUT 

    ###############################################
    # Process form data
    ###############################################

    # Set up job name for LSF Queue
    if {$BSUBOPTS(-J) == ""} {
       if {$jobFileName == ""} {
          set BSUBOPTS(-J) job.[pid]
          } else {
          set BSUBOPTS(-J) $jobFileName
          }
      }

    # Set up jobFileName on remote system
    if {$jobFileName == ""} {
	set jobFileName [file join $workDir job.[pid]]
    } else {
	set jobFileName [file join $workDir $jobFileName]
    }

    # Set up number of processors
    if {$minProc == ""} {
        set minProc 1
    }

    if {$maxProc == ""} {
        set maxProc $minProc
    }

    # Set up number of processors; put at 1,1 if not set at all
    if {$minProc != ""} {
        if {$maxProc != ""} {
            if {$minProc > $maxProc} {
              set minProc $maxProc }
            set BSUBOPTS(-n) "$minProc,$maxProc"
        } else {
            set BSUBOPTS(-n) "$minProc,$minProc"
        }
    } else {
        if {$maxProc != ""} {
            set BSUBOPTS(-n) $maxProc,$maxProc
        } else {
            set BSUBOPTS(-n) "1,1"
        }
    }



    # Set up file lists for copying to and from local nodes (if any)
    set slash "/"
    set slashtmp "/tmp/"
    set slashwrk "/wrk/"
    set CopyOver " > "
    set CopyBack " < "
    if {$cpIn != ""} {
	foreach file $inputFileList {
        #lfile is local file; rfile is remote file 
        set lfile  [list $workDir $slash $file]
        set lfile  [join $lfile  "" ]
        set rfile  [list $slashtmp $remUser $slash $file]
        set rfile  [join $rfile  "" ]
        set movov1 [list \" $lfile $CopyOver $rfile \"]
        set movov  [join $movov1 ""]
	    lappend BSUBOPTS(-f) $movov
	}
    }

    if {$cpOut != ""} {
	foreach file $outputFileList {
        #lfile is local file; rfile is remote file 
        set lfile  [list $slashwrk  $remUser $slash $file]
        set lfile  [join $lfile  "" ]
        set rfile  [list $slashtmp $remUser $slash $file]
        set rfile  [join $rfile  "" ]
        set movbk1 [list \" $lfile $CopyBack $rfile \"]
        set movbk [join $movbk1 ""]
	    lappend BSUBOPTS(-f) $movbk
	}
    }

    set workingTitle "$bqs $title Report"

    cgi_html {
	cgi_head {cgi_title "$workingTitle"}
	cgi_body bgcolor=$masterBgcolor {
	    cgi_center {
		cgi_h2 "$workingTitle[cgi_nl]\
		    System [cgi_font color=blue "$hostName"]"
	    }

	    # Submit LSF Job
	    submitLSFjob $hostName $remHome $remUser
	}
    }
}








































