#!/itl/links/generic/bin/tclsh8.0
# WebSubmit 2.0 lsfMPI.cgi@(#)lsfMPI.cgi	1.3 SCCS/s.lsfMPI.cgi  99/05/20  

# AUTHORS #############################################################

#  Authors: John Koontz, Ryan McCormack, Judith Devaney
#  Original Development Work: Robert Lipman, John Koontz, Katherine
#      Pagoaga

# Copyright Information ###############################################

# This software was developed at the National Institute of Standards
# and Technology by employees of the Federal Government in the course
# of their official duties.  Pursuant to title 17 Section 105 of the
# United States Code this software is not subject to copyright
# protection and is in the public domain.

# CODEINFO BEGIN ######################################################

# info script lsfMPI.cgi> LSF job form generator.

# CODEINFO END ########################################################

# Load WebSubmit package
source /sasp/WebSubmit/webSubmit2.5/bin/websubmit.conf

cgi_debug -off

cgi_eval {
    # HTTP Header
    cgi_http_head

    catch {namespace import webSubmit::lsf::*}
    catch {namespace import webSubmit::forms::*}
    catch {namespace import webSubmit::forms::session::*}

    # CGI configuration
    alias webSubmit cgiHead
    alias webSubmit::lsf BSUBOPTS BSUBFLAGS BQUEUES queueList queueHeaderList\
        shell jobFileName workDir inputFileList outputFileList SchemaName\
        SchemaContents
    cgi_root $cgiHead
    cgi_link config "configuration module" "$cgiHead/config.cgi"

    # Module-specific variables and general setup 
    set moduleVarList [list workDir SchemaName SchemaContents ExecutableName\
        executableFileString inputFileString outputFileString\
        BSUBOPTS(-o) BSUBOPTS(-e) jobFileName\
        BSUBOPTS(-q) BSUBOPTS(-J) BSUBOPTS(-c) BSUBOPTS(-M)\
        minProc maxProc BSUBOPTS(-u) \
        BSUBOPTS(-b) BSUBOPTS(-n)\
        shell loadedSessName\
	hostName moduleName bqs mode title]
    set reqVarList [list hostName moduleName bqs mode title]
    set importVarList [cgiImport $moduleVarList $reqVarList]

    # Load the current session
    if {[info exists loadedSessName] && "$loadedSessName" != ""} {
	session load -name $loadedSessName $moduleName $hostName
    } else {
	session load $moduleName $hostName
    }	
    getSessionData

    # Get form mode (either from user configuration or master.db)
    webSubmit::master::createMasterDb

    # CGI configuration
    set formProcessor ${moduleName}submit

    # Get Remote Site information
    set remUser [webSubmit::auth::authTrans $hostName]
    set remHome [webSubmit::network::getRemEnv HOME $remUser $hostName]

    # Get queue information
    getQueueInfo $hostName

    if {$workDir == ""} {
	set workDir $remHome
    }

    if {$BSUBOPTS(-o) == ""} {
	set BSUBOPTS(-o) "%J.out"
    }

    if {$BSUBOPTS(-e) == ""} {
	set BSUBOPTS(-e) "%J.err"
    }

    if {$BSUBOPTS(-q) == ""} {
	set BSUBOPTS(-q) [lindex $queueList 0]
    }

    if {$minProc == ""} {
	set minProc 1
    }
   
    # Set up help information
    set HELP(elementList) [list workDir SchemaName SchemaContents executableFileString ExecutableName inputFileString\
            outputFileString BSUBOPTS(-o) BSUBOPTS(-e) BSUBOPTS(-J)\
	    BSUBOPTS(-c) BSUBOPTS(-M) BSUBOPTS(-b) BSUBOPTS(-q) BSUBOPTS(-u) processors jobFileName files2nodes]
    set HELP(workDir,header) "Working Directory"
    set HELP(workDir,text) "Directory to be used as the working directory.  If a relative pathname is provided,\
	    it is interpreted relative to <FONT color=yellow><SAMP>$remHome</SAMP></FONT>.  If no directory\
	    is specified, your home directory is assumed."
    set HELP(SchemaName,header) "Schema File"
    set HELP(SchemaName,text) "The filename entered in this area contains the commands to run your Executable\
	    on the rapid nodes.  WebSubmit writes this file for you. In the simplest case, this would just\
            contain the one line '/tmp/Username/ExecutableName N'.  This would execute your program 'ExecutableName'\
            on each node of your virtual machine. Programs are executed in the /tmp/YourUsername directory on the local nodes.\
            The schema file, executable, and input files are copied over for you automatically. The output files are\
            automatically copied back, but these are renamed. See 'Output Files' for the details on this."
    set HELP(SchemaContents,header) "Schema File Contents"
    set HELP(SchemaContents,text) "This contains directions as to which nodes of your virtual machine your executables are to run on.\
	    This allows you to run different executables on different nodes. It also allows you to have\
            command line arguments."
    set HELP(ExecutableName,header) "Executable"
    set HELP(ExecutableName,text) "The filename entered in this area contains the name of your program."
    set HELP(executableFileString,header) "Executables"
    set HELP(executableFileString,text) "The filenames entered in this area contains the names of your programs."
    set HELP(inputFileString,header) "Input Files (Form: file1,...,fileN)"
    set HELP(inputFileString,text) "These are your input files. Do not use wildcards (eg *). It is not necessary to\
             specify which nodes need which inputs."
    set HELP(outputFileString,header) "Output Files (Form: file1,...,fileN)"
    set HELP(outputFileString,text) "These are the output files. Do not use wildcards (eg *). It is not necessary to\
             specify which nodes produce which outputs. Each output file will have three things appended to its name:\
             its mPI rank, the name of the node it was produced on, and its LSF Job ID. So if LSF Job ID: 7041 produces file 'out'\
             on node 0 on rapid 3 and the MPI rank on node 0 is 2, and on node 1 on rapid 7 with rank 5, the returned files will\
              be out.2.rapid3.7041 and out.5.rapid7.7041.\
             Be sure that the /tmp/username directory is clean before you submit your job to avoid confusion. Use\
             CleanAll (to delete all files in /tmp/username on every node), or 'CleanHost rapidx' (to delete all files\
             in /tmp/username on node rapidx) in the MPI Queue or Command Modules. Files are retuned to /wrk/$remUser."
    set HELP(BSUBOPTS(-o),header) "Standard Output File:"
    set HELP(BSUBOPTS(-o),text) "Standard output will be redirected to this file.  If a relative pathname is provided,\
	    it is interpreted relative to the specified working directory.  If no file is specified, the output file is\
            named J.out, where J is the LSF Job ID (The LSF job ID is returned to the user upon job submission)."
    set HELP(BSUBOPTS(-e),header) "Standard Error File:"
    set HELP(BSUBOPTS(-e),text) "Standard error will be redirected to this file.  If a relative pathname is provided,\
	    it is interpreted relative to the specified working directory.   If no file is specified, the error file is\
            named J.err, where J is the LSF Job ID (The LSF job ID is returned to the user upon job submission)."
    set HELP(jobFileName,header) "LSF Job Script File Name:"
    set HELP(jobFileName,text) "This is the name that will be used for the file containing your LSF job script.\
	    This script file contains LSF directives specifying job parameters, and the actual executable\
	    script that constitutes your job.  If unspecified, WebSubmit will automatically select a filename.\
	    When your job is submitted, this is done using <FONT color=yellow><SAMP>bsub jobFileName</SAMP></FONT>."
    set HELP(BSUBOPTS(-J),header) "Job Name"
    set HELP(BSUBOPTS(-J),text) "This assigns the specified name to your job request; it is merely an\
	    identifier used by LSF.  If you omit the request name, the default is the name of the job script file\
	    with the leading path name removed."
    set HELP(BSUBOPTS(-c),header) "CPU Time:"
    set HELP(BSUBOPTS(-c),text) "This specifies the maximum CPU time that your entire request will utilize.\
	    In an LSF job context, a request corresponds to all of the commands contained in your job script.\
	    If the time used by your request exceeds this limit, the entire batch request is terminated."
    set HELP(BSUBOPTS(-M),header) "Memory:"
    set HELP(BSUBOPTS(-M),text) "The maximum amount of memory used by your LSF request. In an LSF job context,\
	    a request corresponds to all of the commands contained in your job script.  If the total memory used\
	    by your request exceeds that specified, processes in the request are not allowed to allocate more\
	    memory."
    set HELP(BSUBOPTS(-u),header) "Email Notification"
    set HELP(BSUBOPTS(-u),text) "Specifies the address to which email notification should be sent on job completion.\
	    The address should be of the form <FONT color=yellow><SAMP>user\[@host\]</SAMP></FONT>; if the host is\
	    specified, it must be a valid Internet hostname."
    set HELP(BSUBOPTS(-b),header) "Submission Time"
    set HELP(BSUBOPTS(-b),text) "This controls the time at which your job is submitted for processing.  If you specify\
	    a time in the future, your job will enter the queue and will be help until that time, at which point it\
	    will be allowed to execute.  Allowed formats for specification of time can be found in the <FONT color=yellow>\
	    <SAMP>bsub</SAMP></FONT> manual page."
    set HELP(BSUBOPTS(-q),header) "Job Queue"
    set HELP(BSUBOPTS(-q),text) "The job queue determines the resource limits and\
	    priority for your job.  You should select the queue that best meets the requirements for your job."
    set HELP(files2nodes,header) "Transfer Files to and from Local Nodes"
    set HELP(files2nodes,text) "In order to get performance, it is necessary to copy your Schema file, your executable\
            and your input files to the local nodes for execution.  Please provide a comma-separated\
            list of your executable, your schema file name, and all input files to copy to the local nodes.\
            Output files will be generated on the local nodes. In order for them to be returned from the local\
            nodes to <FONT color=yellow><SAMP>$hostName</SAMP></FONT> once execution is complete, please also\
            provide a comma separated list of the files to be returned."
    set HELP(processors,header) "Multiprocessor Specification"
    set HELP(processors,text) "Select the minimum and maximum number of processors to be used in your parallel job.\
	    The default for the minimum is 1; the default for the maximum is the current minimum"

    # Define help links
    helpSetup HELP

    set workingTitle "$bqs $title"

    cgi_html {
	cgi_head {
	    cgi_title "$workingTitle"
	}
	cgi_body bgcolor=$masterBgcolor {
	    # Form
	    cgi_form sessionManager {
		# Basic HTML form elements
		cgi_center {
		    cgi_h2 "$workingTitle[cgi_nl]\
		    System [cgi_font color=blue "$hostName"]"
		}

		# Instructions
		cgi_h3 "[cgi_font color=darkmagenta Instructions:]"
		cgi_p "Fill out the necessary information related to your job and then simply click\
			[cgi_bold "Submit Job"]. The form has two modes: [cgi_bold BASIC] and \
                        [cgi_bold ADVANCED]. In [cgi_bold BASIC] mode, the only required fields are\
                        the [cgi_font color=darkmagenta [cgi_bold "Executable and Schema Name (and Input and Output files, if applicable)"]].\
                        Default values will be supplied for other fields\
			in the form if they are left blank. In [cgi_bold ADVANCED] mode,\
                        you must enter the contents of your own schema file in [cgi_font color=darkmagenta [cgi_bold "Schema File Contents"]].\
                        Modes can be switched using the [cgi_link config]."
		cgi_br

		cgi_table cellpadding=3 border=1  {
		    ###########################
		    # Path Information
		    ###########################


                    cgi_table_row {
                        cgi_td bgcolor=darkslateblue colspan=2 "[cgi_font size=+1 color=white\
                                "[cgi_bold "Job Specification Information"]"][cgi_nbspace][cgi_nbspace][cgi_font color=white\
                                "(Current Form Mode: "][cgi_font color=yellow $mode][cgi_font color=white )]"
                    }

		    
		    cgi_table_row {
			cgi_td colspan=2 "[cgi_font size=+1 color=darkmagenta [cgi_bold "File and Directory Information"]]"
		    }

		    cgi_table_row {
			cgi_td align=center "[cgi_bold [cgi_link workDir]] on [cgi_font color=darkmagenta "[cgi_sample "$hostName"]"]"
			cgi_table_data nowrap {
			    puts "[cgi_font color=darkmagenta [cgi_bold "Directory Name:"]][cgi_nbspace]"
			    cgi_text workDir=$workDir size=35 maxlength=200
			}
		    }

                    cgi_table_row {
                        cgi_td align=center "[cgi_bold [cgi_link SchemaName]]"
                        cgi_table_data nowrap {
                            puts "[cgi_font color=darkmagenta [cgi_bold "Schema File Name:"]][cgi_nbspace]"
                            cgi_text SchemaName=$SchemaName size=35 maxlength=200
                        }
                    }

                    if {$mode == "BASIC"} {

                    cgi_table_row {
                        cgi_td align=center "[cgi_bold [cgi_link ExecutableName]]"
                        cgi_table_data nowrap {
                            puts "[cgi_font color=darkmagenta [cgi_bold "Executable Name:"]][cgi_nbspace]"
                            cgi_text ExecutableName=$ExecutableName size=35 maxlength=200
                        }
                    }


}


                    if {$mode == "ADVANCED"} {

                    cgi_table_row {
                        cgi_td align=center "[cgi_bold [cgi_link SchemaContents
]]"
                        cgi_table_data nowrap {
                            puts "[cgi_nl]"
                            cgi_textarea SchemaContents=$SchemaContents rows=5 cols=45
                        }
                    }


                    cgi_table_row {
                        cgi_td align=center "[cgi_bold [cgi_link executableFileString]]"
                        cgi_table_data nowrap {
                            puts "[cgi_font color=darkmagenta [cgi_bold "Form: Executable1,...,ExecutableN"]][cgi_nbspace]"
                            cgi_text executableFileString=$executableFileString size=40 maxlength=450
                        }
                    }



                 }


                    cgi_table_row {
                        cgi_td align=center "[cgi_bold "Data File Names"]"
                        cgi_table_data nowrap {
                            cgi_table {



                                   cgi_table_row {
                                        cgi_td nowrap "[cgi_bold [cgi_link inputFileString]][cgi_nl]"
                                        cgi_table_data nowrap {
                                            cgi_text inputFileString=$inputFileString size=40 maxlength=450
                                        }
                                    }



                                    cgi_table_row {
                                        cgi_td nowrap "[cgi_bold [cgi_link outputFileString]][cgi_nl]"
                                        cgi_table_data nowrap {
                                            cgi_text outputFileString=$outputFileString size=40 maxlength=450
                                        }
                                    }





                                cgi_table_row {
                                    cgi_td "[cgi_font color=darkmagenta [cgi_bold [cgi_link BSUBOPTS(-o)]]][cgi_nbspace]"
                                    cgi_table_data nowrap {
                                        cgi_text BSUBOPTS(-o)=$BSUBOPTS(-o) size=35 maxlength=200
                                    }
                                }
                                cgi_table_row {
                                    cgi_td "[cgi_font color=darkmagenta [cgi_bold [cgi_link BSUBOPTS(-e)]]][cgi_nbspace]"
                                    cgi_table_data nowrap {
                                        cgi_text BSUBOPTS(-e)=$BSUBOPTS(-e) size=35 maxlength=200
                                    }
                                }
                                cgi_table_row {
                                    cgi_td "[cgi_font color=darkmagenta [cgi_bold [cgi_link jobFileName]]][cgi_nbspace]"
                                    cgi_table_data nowrap {
                                        cgi_text jobFileName=$jobFileName size=35 maxlength=200
                                    }
                                }
                            }
                        }
                    }






                    cgi_table_row {
                        cgi_td colspan=2 "[cgi_font size=+1 color=darkmagenta [cgi_bold "LSF Job Information"]]"
                    }



                    cgi_table_row {
                        cgi_td align=center "[cgi_bold [cgi_link BSUBOPTS(-q)]]"
                        cgi_table_data nowrap {
                            # Data about queues
                            cgi_table width=100% {
                                cgi_table_row {
                                    foreach header $queueHeaderList {
                                        if {$BQUEUES($header,show)} {
                                            cgi_td align=center "[cgi_font size=-1 [cgi_bold "$BQUEUES($header,text)"]]"
                                        }
                                    }
                                    cgi_td align=center "[cgi_font size=-1 color=darkmagenta [cgi_bold "SELECT"]]"
                                }

                                foreach queue $queueList {
                                    cgi_table_row {
                                        foreach header $queueHeaderList {
                                            if {$BQUEUES($header,show)} {
                                                cgi_td align=center "[cgi_font size=-1 $BQUEUES($queue,$header)]"
                                            }
                                        }
                                        cgi_table_data nowrap align=center {
                                            cgi_radio_button BSUBOPTS(-q)=$queue checked_if_equal=$BSUBOPTS(-q)
                                        }
                                    }
                                }
                            }
                        }
                    }


                    cgi_table_row {
                        cgi_td align=center "[cgi_bold [cgi_link BSUBOPTS(-J)]]"
                        cgi_table_data nowrap {
                            puts "[cgi_font color=darkmagenta [cgi_bold "Name:"]][cgi_nbspace]"
                            cgi_text BSUBOPTS(-J)=$BSUBOPTS(-J) size=35 maxlength=45
                        }
                    }



 if {$mode == "ADVANCED"} {
                    cgi_table_row {
                        cgi_td align=center "[cgi_bold "Resource Limits"]"
                        cgi_table_data nowrap {
                            puts "[cgi_font color=darkmagenta [cgi_bold [cgi_link BSUBOPTS(-c)]]][cgi_nbspace]"
                            cgi_text BSUBOPTS(-c)=$BSUBOPTS(-c) size=15 maxlength=200
                            puts "[cgi_nbspace][cgi_nbspace][cgi_bold "Valid format:"] \[hours:\]minutes"
                            puts "[cgi_nl][cgi_font color=darkmagenta [cgi_bold [cgi_link BSUBOPTS(-M)]]][cgi_nbspace]"
                            cgi_text BSUBOPTS(-M)=$BSUBOPTS(-M) size=10 maxlength=200
                            puts "[cgi_nbspace]Mb"
                        }
                    }
  }

                    cgi_table_row {
                        cgi_td align=center "[cgi_bold [cgi_link processors]]"
                        cgi_table_data nowrap {
                            puts "[cgi_font color=darkmagenta [cgi_bold "Number of processors:"]][cgi_nbspace]Min"
                            cgi_text minProc=$minProc size=2
                            puts "[cgi_nbspace][cgi_nbspace]Max"
                            cgi_text maxProc=$maxProc size=2
                        }
                    }



                   cgi_table_row {
                        cgi_td align=center "[cgi_bold [cgi_link BSUBOPTS(-u)]]"
                        cgi_table_data nowrap {
                            puts "[cgi_font color=darkmagenta [cgi_bold "Notification Address:"]][cgi_nbspace]"
                            cgi_text BSUBOPTS(-u)=$BSUBOPTS(-u) size=35 maxlength=450
                        }
                    }

 if {$mode == "ADVANCED"} {
                        cgi_table_row {
                            cgi_td align=center "[cgi_bold [cgi_link BSUBOPTS(-b)]]"
                            cgi_table_data nowrap {
                                puts "[cgi_font color=darkmagenta [cgi_bold "Time:"]][cgi_nbspace]"
                                cgi_text BSUBOPTS(-b)=$BSUBOPTS(-b) size=20 maxlength=450
                                puts "[cgi_nbspace][cgi_nbspace][cgi_bold "Valid format:"] \[\[month:\]day:\]hour:minute"
                            }
                        }
  }



                   cgi_table_row {
                        cgi_td bgcolor=darkslateblue colspan=2 "[cgi_font size=+1 color=white\
                                "[cgi_bold "Submit LSF Job"]"]"
                    }

                    cgi_table_row {
                        cgi_table_data nowrap colspan=2 {
                            puts "[cgi_nbspace][cgi_nbspace][cgi_nbspace][cgi_nbspace]"
                            cgi_submit_button "action=Submit Job"
                            puts "[cgi_nbspace][cgi_nbspace][cgi_nbspace][cgi_nbspace]"
                            cgi_reset_button
                        }
                    }

                    cgi_table_row {
                        cgi_table_data nowrap align=center colspan=2 {
			    cgi_export mode=$mode
			    cgi_export moduleName=$moduleName
			    cgi_export hostName=$hostName
			    cgi_export bqs=$bqs
			    cgi_export title=$title
                            cgi_export formProcessor=$cgiHead/$formProcessor.cgi

                            # Session Manager
                            sessionManager $moduleName $hostName
                        }
                    }
		}
	    }
	}
    }
}
















































