#!/itl/links/generic/bin/tclsh8.0
# WebSubmit 2.0 lsfMPIsubmit.cgi @(#)lsfMPIsubmit.cgi	1.3 SCCS/s.lsfMPIsubmit.cgi  99/05/20 

# AUTHORS #############################################################

#  Authors: John Koontz, Ryan McCormack, Judith Devaney
#  Original Development Work: Robert Lipman, John Koontz, Katherine
#      Pagoaga

# Copyright Information ###############################################

# This software was developed at the National Institute of Standards
# and Technology by employees of the Federal Government in the course
# of their official duties.  Pursuant to title 17 Section 105 of the
# United States Code this software is not subject to copyright
# protection and is in the public domain.

# CODEINFO BEGIN ######################################################

# info script lsfMPIsubmit.cgi> LSF job submission processor.

# CODEINFO END ########################################################

# Load websubmit information
source /sasp/WebSubmit/webSubmit2.5/bin/websubmit.conf

cgi_debug -off

cgi_eval {
    # HTTP Header
    cgi_http_head

    catch {namespace import webSubmit::lsf::*}

    # Variable aliasing
    alias webSubmit::lsf BSUBOPTS BSUBFLAGS jobFileName workDir inputFileList\
        outputFileList shell SchemaName  SchemaContents

    # Initialize module variables
    set moduleVarList [list workDir SchemaName SchemaContents ExecutableName\
	executableFileString inputFileString outputFileString\
	BSUBOPTS(-o) BSUBOPTS(-e) jobFileName\
	BSUBOPTS(-q) BSUBOPTS(-J) BSUBOPTS(-c) BSUBOPTS(-M)\
	minProc maxProc BSUBOPTS(-u) \
	BSUBOPTS(-b) BSUBOPTS(-n)\
	shell loadedSessName]
    initVars $moduleVarList
    set reqVarList [list hostName title BSUBOPTS(-q) SchemaName]
    initVars $moduleVarList
    set varList [cgiImport $moduleVarList $reqVarList]

    set whiteVarList [list workDir BSUBOPTS(-i) BSUBOPTS(-e) BSUBOPTS(-o) BSUBOPTS(-J) minProc maxProc jobFileName BSUBOPTS(-M)]
    trimWhite $whiteVarList

    # Get remote information
    set remUser [webSubmit::auth::authTrans $hostName]
    set remHome [webSubmit::network::getRemEnv HOME $remUser $hostName]

    
    #ExecutableName or ExecutableFileString must be set
    if {$ExecutableName == ""} {

	if {$executableFileString == ""} {
	    cgiBomb lsfMPIsubmit.cgi "No executable has been provided!" 
	}
    }

    if {$ExecutableName != ""} {
	set SchemaContents "/tmp/$remUser/$ExecutableName N"

	# Put all files to be copied over into inputFileString
	set executableFileList "$ExecutableName $SchemaName"


    } else {
	#in ADVANCED mode: 

	regsub -all " " $executableFileString "" executableFileString
	set executableFileList [split $executableFileString ,]
	set executableFileList [concat $executableFileList $SchemaName]

    }


    # Set up input and output file lists
    if {$inputFileString != ""} {
	regsub -all " " $inputFileString "" inputFileString
	set inputFileList [split $inputFileString ,]
    }


    if {$outputFileString != ""} {
	regsub -all " " $outputFileString "" outputFileString
	set outputFileList [split $outputFileString ,]
    }


    # Put all files to be copied over into inputFileList
    set inputFileList [concat $inputFileList $executableFileList]

    # Path resolution
    set workDir [mkPathAbs $workDir $remHome]
    foreach flag [list -o -i -e] {
	if {$BSUBOPTS($flag) != ""} {
	    set BSUBOPTS($flag) [mkPathAbs $BSUBOPTS($flag) $workDir]
	}
    }


    if {$BSUBOPTS(-o) == ""} {
	set BSUBOPTS(-o) "%J.out"
    }

    if {$BSUBOPTS(-e) == ""} {
	set BSUBOPTS(-e) "%J.err"
    }

    if {$BSUBOPTS(-q) == ""} {
	set BSUBOPTS(-q) [lindex $queueList 0]
    }

    if {$minProc == ""} {
	set minProc 1
    }

    if {$maxProc == ""} {
	set maxProc $minProc
    }

    # Verify CGI input information; to be used variable must be in the varList!!!
    set INPUT(varList) [list workDir BSUBOPTS(-M) BSUBOPTS(-o) BSUBOPTS(-i) BSUBOPTS(-e) BSUBOPTS(-J) minProc maxProc jobFileName SchemaContents]
    set INPUT(workDir) [list noWhite isDirectory]
    set INPUT(workDir,isDirectory,hostName) $hostName
    set INPUT(workDir,isDirectory,remUser) $remUser
    set INPUT(BSUBOPTS(-M)) isPositiveInt
    set INPUT(BSUBOPTS(-o)) noWhite
    set INPUT(BSUBOPTS(-i)) [list noWhite remFileExist]
    set INPUT(BSUBOPTS(-i),remFileExist,hostName) $hostName
    set INPUT(BSUBOPTS(-i),remFileExist,remUser) $remUser
    set INPUT(BSUBOPTS(-e)) noWhite
    set INPUT(BSUBOPTS(-J)) noWhite
    set INPUT(minProc) isPositiveInt
    set INPUT(maxProc) isPositiveInt
    set INPUT(jobFileName) noWhite
    set INPUT(SchemaContents) hasChars
    set i 0
    foreach file $inputFileList {
	set dummyVar$i $file
	set INPUT(dummyVar$i) remFileExist
	set INPUT(dummyVar$i,remFileExist,hostName) $hostName
	set INPUT(dummyVar$i,remFileExist,remUser) $remUser
	incr i
    }
    inputVerify INPUT 

    ###############################################
    # Process form data
    ###############################################

    # Set up job name for LSF Queue
    if {$BSUBOPTS(-J) == ""} {
	if {$jobFileName == ""} {
	    set BSUBOPTS(-J) job.[pid]
	} else {
	    set BSUBOPTS(-J) $jobFileName
	}
    }

    # Set up jobFileName on remote system
    if {$jobFileName == ""} {
	set jobFileName [file join $workDir job.[pid]]
    } else {
	set jobFileName [file join $workDir $jobFileName]
    }


    # Set up number of processors; put at 1,1 if not set at all
    if {$minProc != ""} {
	if {$maxProc != ""} {
	    if {$minProc > $maxProc} {
		set minProc $maxProc }
	    set BSUBOPTS(-n) "$minProc,$maxProc"
	} else {
	    set BSUBOPTS(-n) "$minProc,$minProc"
	}
    } else {
	if {$maxProc != ""} {
	    set BSUBOPTS(-n) $maxProc,$maxProc
	} else {
	    set BSUBOPTS(-n) "1,1"
	}
    }

    set workingTitle "$bqs $title Report"

    cgi_html {
	cgi_head {cgi_title "$workingTitle"}
	cgi_body bgcolor=$masterBgcolor {
	    cgi_center {
		cgi_h2 "$workingTitle[cgi_nl]\
		    System [cgi_font color=blue "$hostName"]"
	    }

	    # Submit LSF MPI (LAM) Job
	    submitLSFMPIjob $hostName $remHome $remUser
	}
    }
}





















