#!/itl/links/generic/bin/tclsh8.0
# WebSubmit 2.0 lsfQstat.cgi @(#)lsfQstat.cgi	1.3 SCCS/s.lsfQstat.cgi  99/05/20 %Q

# AUTHORS #############################################################

#  Authors: John Koontz, Ryan McCormack, Judith Devaney
#  Original Development Work: Robert Lipman, John Koontz, Katherine
#      Pagoaga

# Copyright Information ###############################################

# This software was developed at the National Institute of Standards
# and Technology by employees of the Federal Government in the course
# of their official duties.  Pursuant to title 17 Section 105 of the
# United States Code this software is not subject to copyright
# protection and is in the public domain.

# CODEINFO BEGIN ######################################################

# info script lsfQstat.cgi>  SP2 queuing system interface.

# CODEINFO END ########################################################


# webSubmit setup
source /sasp/WebSubmit/webSubmit2.5/bin/websubmit.conf

cgi_debug -off

cgi_eval {
    # HTTP Header
    cgi_http_head

    # Aliases
    alias webSubmit cgiHead
    alias webSubmit::lsf BJOBS BJOBSINFO PATHS jobHeaderList queueList 

    catch {namespace import webSubmit::lsf::*}
    catch {namespace import webSubmit::forms::*}

    # Process CGI input
    initVars [list jobInfoList jobList action jobSignal nodeString title]
    set moduleVarList [list hostName moduleName mode title bqs]
    set reqVarList [list title bqs hostName moduleName]
    cgiImport $moduleVarList $reqVarList

    # CGI definitions
    cgi_root $cgiHead
    set formProcessor $moduleName

    set headColor darkmagenta
    set tag 0

    # Get remote site information
    set remUser [webSubmit::auth::authTrans $hostName]

    # Process any pending actions by the user
    foreach cmd [list bqueues bjobs bpeek bkill bstop bresume jmsum cleanall cleansome showall mkall] {
	set ${cmd}Cmd $PATHS($hostName,$cmd)
    }

    if {$action == "CleanAll"} {
        # delete all files in /tmp/user on all rapid nodes
	lappend jobInfoList [webSubmit::network::sshCmdExec $remUser $hostName $cleanallCmd]
    }

    if {$action == "ShowAll"} {
        # Show all processes and all files in /tmp/user on all rapid nodes
	lappend jobInfoList [webSubmit::network::sshCmdExec $remUser $hostName $showallCmd]
    }

    if {$action == "MkUsrDirAll"} {
        # create directory /tmp/user on all rapid nodes; /wrk/usr on fron end node
	lappend jobInfoList [webSubmit::network::sshCmdExec $remUser $hostName $mkallCmd]
    }

    # General Commands
    if {$action == "Show queue"} {
	lappend jobInfoList [webSubmit::network::sshCmdExec $remUser $hostName "$bqueuesCmd -l $queue"]
    }

    if {$action == "CleanSome"} {

        # delete contents of /tmp/user on indicated nodes

        if {$nodeString != ""} {
	    set numberList [split $nodeString ,]
	    set nodeList ""
	    set n1 1
	    set n2 16
	    set fail 0
	    foreach number $numberList {
                if {$number < $n1 || $number > $n2} {
		    set fail 1
		    break
		}
	    }

            if {!$fail } {
		foreach number $numberList {
		    set number [join [list "rapid" $number ] ""]
		    lappend nodeList $number
		}
                lappend jobInfoList [webSubmit::network::sshCmdExec $remUser $hostName "$cleansomeCmd $nodeList"]
            } else { 
		lappend jobInfoList "bad node list; $number is invalid; valid nodes are 1..16" 
            }
	}
	
    }

    # Commands related to specific jobs
    if {$action == "Kill Jobs"} {
        # Kill selected LSF jobs
        if {$jobList != ""} {
            lappend jobInfoList [webSubmit::network::sshCmdExec $remUser $hostName "$bkillCmd $jobList"]
        }
    } elseif {$action == "Generate Listing"} {
        # Query for information about jobs
        if {$jobList != ""} {
            foreach job $jobList {
                lappend jobInfoList [webSubmit::network::sshCmdExec $remUser $hostName "$bjobsCmd -l $job"]
            }
        }
    } elseif {$action == "Peek"} {
        # Look at standard output from jobs
        if {$jobList != ""} {
            foreach job $jobList {
                lappend jobInfoList [webSubmit::network::sshCmdExec $remUser $hostName "$bpeekCmd $job"]
            }
        }
    } elseif {$action == "Signal Jobs"} {
        # Send a signal to jobs
        if {$jobList != ""} {
            if {$jobSignal != ""} {
                lappend jobInfoList [webSubmit::network::sshCmdExec $remUser $hostName "$bkillCmd -s $jobSignal $jobList"]
            }
        }
    } elseif {$action == "Stop"} {
        # Stop selected LSF jobs
        if {$jobList != ""} {
            lappend jobInfoList [webSubmit::network::sshCmdExec $remUser $hostName "$bstopCmd $jobList"]
        }
    } elseif {$action == "Resume"} {
        # Stop selected LSF jobs
        if {$jobList != ""} {
            lappend jobInfoList [webSubmit::network::sshCmdExec $remUser $hostName "$bresumeCmd $jobList"]
	}

    }

    #######################
    # Get Queue information
    #######################

    if [catch {webSubmit::network::sshCmdExec $remUser $hostName $bqueuesCmd} result] {
        cgiBomb rapidQstat.cgi $result
    } else {
        set bqueuesInfo [split $result \n]
    }

    catch {webSubmit::network::sshCmdExec $remUser $hostName "$bjobsCmd -u all"} result
    set allInfo [split $result \n]

    catch {webSubmit::network::sshCmdExec $remUser $hostName "$jmsumCmd"} result
    set nodeInfo [split $result \n]

    # Get job information
    getBjobsInfo $remUser $hostName
    set jobList $BJOBS(jobIdList)

    # get list of queues
    getQueueInfo $hostName

    set workingTitle "$bqs $title"    

    cgi_html {
	cgi_head {
	    cgi_title "$workingTitle"
	}
	
	cgi_body bgcolor=$masterBgcolor {
	    # Basic HTML form elements
	    cgi_center {
		cgi_h2 "$workingTitle[cgi_nl]\
		    System [cgi_font color=blue "$hostName"]"
	    }
	    cgi_br
	    cgi_form $formProcessor {
                if {"$mode" != ""} {
		    cgi_export mode=$mode
		}
		cgi_export moduleName=$moduleName
		cgi_export hostName=$hostName
		cgi_export title=$title
		cgi_export bqs=$bqs

		# Output generated from commands
		if {$jobInfoList != ""} {
		    cgi_br
		    cgi_h3 "[cgi_font color=darkmagenta "Output:"]"
		    cgi_table width=100% border=1 {
			cgi_table_row {
			    cgi_table_data nowrap bgcolor=darkslateblue {
				if {$jobList != ""} {
				    foreach job $jobList {
					cgi_h4 "[cgi_font color=yellow "Job Information for job $job"]"
					set i [lsearch -exact $jobList $job]
					cgi_preformatted {
					    foreach line [split [lindex $jobInfoList $i] \n] {
						puts "[cgi_font color=white "[cgi_sample $line]"]"
					    }
					}
				    }
				} else {
				    cgi_preformatted {
					foreach line [split $jobInfoList \n] {
					    puts "[cgi_font color=white "[cgi_sample $line]"]"
					}
				    }
				}
			    }
			}
		    }
		    cgi_br;cgi_br;cgi_hr
		}

		###################################################
		# Information about user jobs
		cgi_h3 "[cgi_font color=darkmagenta "Jobs Currently Queued for:"][cgi_nbspace][cgi_nbspace][cgi_font color=blue "$remUser"]"
		if {$jobList != ""} {
		    cgi_table align=center width=90% cellpadding=3 border=1 {
			cgi_table_row {
			    foreach header $jobHeaderList {
				if {$BJOBSINFO($header,show)} {
				    cgi_td align=center "[cgi_bold $BJOBSINFO($header,text)]"
				}
			    }
			    cgi_td align=center "[cgi_bold Select]"
			}

			foreach job $jobList {
			    cgi_table_row {
				foreach header $jobHeaderList {
				    set value $BJOBS($job,$header)
				    if {$value == ""} {
					set value "[cgi_nbspace]"
				    }

				    if {$header == "JOBID"} {
					cgi_td align=center "[cgi_font color=blue [cgi_bold $job]]"
				    } else {
					if {$BJOBSINFO($header,show)} {
					    if [info exists BJOBSINFO($header,$value)] {
						cgi_td align=center "$BJOBSINFO($header,$value)"
					    } else {
						cgi_td align=center "$value"
					    }
					}
				    }
				}
				cgi_table_data nowrap align=center {
				    cgi_checkbox "jobList=$job"
				}
			    }
			}
		    }
		} else {
		    puts "[cgi_nbspace][cgi_nbspace][cgi_bold None]"
		}
		cgi_br;cgi_br;cgi_br


		# Node Information
		cgi_h3 "[cgi_font color=darkmagenta "Node Summary: "]"
		cgi_preformatted {
		    foreach line $nodeInfo {
			if ![string match "*No unfinished jobs*" $line] {
			    puts [cgi_sample $line]
			}
		    }
		}
		cgi_br

		# Complete Queue Information
		cgi_h3 "[cgi_font color=darkmagenta "Queue Summary: "]\
			[cgi_nbspace][cgi_nbspace][cgi_font color=blue "[cgi_sample [webSubmit::util::tclDate]]"]"
		cgi_preformatted {
		    foreach line $allInfo {
			puts [cgi_sample $line]
		    }
		}
		cgi_br

		cgi_preformatted {
		    foreach line $bqueuesInfo {
			puts [cgi_sample $line]
		    }
		}
		cgi_br
		cgi_submit_button "action=Update Queue Information"
		cgi_hr;cgi_br;cgi_br

		# LSF Actions
		cgi_h3 "[cgi_font color=darkmagenta "LSF Actions:"]"
		cgi_table align=center width=90% cellpadding=3 border=1 {
		    cgi_table_row {
			cgi_td align=center "[cgi_bold "Description"]"
			cgi_td align=center "[cgi_bold "Execute"]"
		    }

		    cgi_table_row {
			cgi_td "[cgi_font color=darkmagenta [cgi_bold "CleanAll"]]: Delete all files in /tmp/user on [cgi_font color=blue [cgi_bold "EVERY"]] remote rapid node"
			cgi_table_data nowrap align=center {
			    cgi_submit_button "action=CleanAll" "onClick=if (!confirm('Delete all files in /tmp/user on *every* rapid node?')) return false;"
			}
		    }


		    cgi_table_row {
			cgi_table_data {
			    puts "[cgi_font color=darkmagenta [cgi_bold "CleanSome"]]: Delete all files in /tmp/user on the following remote rapid nodes ([cgi_font color=blue [cgi_bold "The format is a comma separated list of node numbers e.g. 5,2,11"]]):[cgi_nbspace][cgi_nbspace]"
			    cgi_text nodeString=$nodeString length=40 maxlength=200
			    cgi_table_data nowrap align=center {
				cgi_submit_button "action=CleanSome"
			    }
			}
		    }


		    cgi_table_row {
			cgi_td "[cgi_font color=darkmagenta [cgi_bold "ShowAll"]]: Show all processes and show all files in /tmp/user on [cgi_font color=blue [cgi_bold "EVERY"]] remote rapid node"
			cgi_table_data nowrap align=center {
			    cgi_submit_button "action=ShowAll"
			}
		    }

		    cgi_table_row {
			cgi_td "[cgi_font color=darkmagenta [cgi_bold "MkUsrDirAll"]]: Create directory /tmp/user on [cgi_font color=blue [cgi_bold "EVERY"]] remote rapid node; create /wrk/user on the front end node"
			cgi_table_data nowrap align=center {
			    cgi_submit_button "action=MkUsrDirAll" 
			}
		    }


		    cgi_table_row {
			cgi_table_data nowrap {
			    puts "[cgi_font color=darkmagenta [cgi_bold "bjobs -l"]]: Get detailed information about queue"
			    cgi_select queue {
				foreach q $queueList {
				    cgi_option $q
				}
			    }
			}
			cgi_table_data nowrap align=center {
			    cgi_submit_button "action=Show queue"
			}
		    }	

		    if {$jobList != ""} {
			cgi_table_row {
			    cgi_td "[cgi_font color=darkmagenta [cgi_bold "bkill"]]: Kill all of the selected jobs"
			    cgi_table_data nowrap align=center {
				cgi_submit_button "action=Kill Jobs" "onClick=if (!confirm('Kill jobs?')) return false;"
			    }
			}

			cgi_table_row {
			    cgi_td "[cgi_font color=darkmagenta [cgi_bold "bjobs -l"]]: Get detailed information about the selected jobs"
			    cgi_table_data nowrap align=center {
				cgi_submit_button "action=Generate Listing"
			    }
			}

			cgi_table_row {
			    cgi_td "[cgi_font color=darkmagenta [cgi_bold "bpeek"]]: Peek at output for the selected jobs"
			    cgi_table_data nowrap align=center {
				cgi_submit_button "action=Peek"
			    }
			}

			cgi_table_row {
			    cgi_table_data {
				puts "[cgi_font color=darkmagenta [cgi_bold "bkill -s"]]: Send signal[cgi_nbspace]"
				cgi_text jobSignal=$jobSignal length=8 maxlength=15
				puts "[cgi_nbspace]to the selected jobs"
				cgi_table_data nowrap align=center {
				    cgi_submit_button "action=Signal Jobs"
				}
			    }

			    cgi_table_row {
				cgi_td "[cgi_font color=darkmagenta [cgi_bold "bstop"]]: Stop the selected jobs"
				cgi_table_data nowrap align=center {
				    cgi_submit_button "action=Stop"
				}
			    }

			    cgi_table_row {
				cgi_td "[cgi_font color=darkmagenta [cgi_bold "bresume"]]: Resume the selected jobs"
				cgi_table_data nowrap align=center {
				    cgi_submit_button "action=Resume"
				}
			    }
			}
		    }
		}
	    }
	}
    }
}









































