#!/itl/links/generic/bin/tclsh8.0
# WebSubmit 2.0 nqsGaussiansubmit.cgi @(#)nqsGaussiansubmit.cgi	1.4 SCCS/s.nqsGaussiansubmit.cgi  99/05/20 

# AUTHORS #############################################################

#  Authors: John Koontz, Ryan McCormack, Judith Devaney
#  Original Development Work: Robert Lipman, John Koontz, Katherine
#      Pagoaga

# Copyright Information ###############################################

# This software was developed at the National Institute of Standards
# and Technology by employees of the Federal Government in the course
# of their official duties.  Pursuant to title 17 Section 105 of the
# United States Code this software is not subject to copyright
# protection and is in the public domain.

# CODEINFO BEGIN ######################################################

# info script nqsGaussiansubmit.cgi> NQS Gaussian job submission processor.

# CODEINFO END ########################################################

# Load websubmit information
source /sasp/WebSubmit/webSubmit2.5/bin/websubmit.conf
catch {namespace import webSubmit::nqs::*}

alias webSubmit GaussianVersions

cgi_debug -off

cgi_eval {
    # Variable aliasing
    alias webSubmit::nqs Gaussian QSUBOPTS QSUBFLAGS
    
    # Initialize module variables
    set moduleVarList [list hostName title remDir remGaussianFile\
        locGaussianFile clobberGaussian Gaussian(OutputType) Gaussian(Nproc)\
	Gaussian(JobNameTxt) Gaussian(NProcType) loadedSessName QSUBOPTS(-r)\
        QSUBFLAGS(-me) QSUBOPTS(-l,mpp_p) QSUBOPTS(-lT) QSUBOPTS(-lM)\
        memUnits QSUBOPTS(-o) jobName gvers.option]
    set reqVarList [list hostName title Gaussian(OutputType)\
        Gaussian(NProcType) jobName QSUBOPTS(-lT) QSUBOPTS(-lM) gvers.option]
    cgiImport $moduleVarList $reqVarList

    # Get remote information
    set remUser [webSubmit::auth::authTrans $hostName]
    set remHome [webSubmit::network::getRemEnv HOME $remUser $hostName]
    
    # Verify CGI Input data
    set remDir [mkPathAbs $remDir $remHome]
    set remGaussianFile [mkPathAbs $remGaussianFile $remDir]
    set INPUT(varList) [list remDir QSUBOPTS(-lM) QSUBOPTS(-l,mpp_p) remGaussianFile]
    set INPUT(remDir) [list noWhite isDirectory]
    set INPUT(remDir,isDirectory,hostName) $hostName
    set INPUT(remDir,isDirectory,remUser) $remUser
    set INPUT(remGaussianFile) [list noWhite remFileExist]
    set INPUT(remGaussianFile,remFileExist,hostName) $hostName
    set INPUT(remGaussianFile,remFileExist,remUser) $remUser
    set INPUT(QSUBOPTS(-lM)) isPositiveInt
    set INPUT(QSUBOPTS(-l,mpp_p)) isPositiveInt
    set INPUT(QSUBOPTS(-lT)) noWhite
    inputVerify INPUT

    ###############################################
    # Process form data
    ###############################################
    
    #  Change "/tmp/CGI[pid]-[verno] {}" returned by cgi_file_button for
    #  empty file into null string as expected in code
    if [regexp { \{\}$} $locGaussianFile] {
	set locGaussianFile ""
    }
    
    ##################################################################################
    # Analyze Gaussian directives file specification
    #
    #  Users must specify either a remote directives file or a file to upload from the
    #  browser system.  If a remote file is specified, it is copied to the WebSubmit
    #  server so that it can be processed as needed.  If a local file is specified to
    #  upload, then the new directives file on the remote system is given the same
    #  name as the uploaded file.  If two directives files are specified, an error
    #  is generated.
    #
    #  tmpGaussianFile = Name of temporary Gaussian directives file on WebSubmit server.  File
    #                is obtained either via upload or by copying it from the remote system
    # 
    #  remGaussianFile = Name of Gaussian file on remote system
    #
    #  locGaussianFile = If non-null, list containing name of uploaded file on WebSubmit server,
    #                  and original name of file on browser system
    ##################################################################################
    set tmpGaussianFile ""
    if {$locGaussianFile == "" && $remGaussianFile == ""} {
	# No directives file
	cgiBomb "nqsGaussiansubmit.cgi for $hostName"\
            "No Gaussian directives file specified"
    } elseif {$locGaussianFile != "" && $remGaussianFile != ""} {
	# Two directives files specified - inconsistent input
	cgiBomb "nqsGaussiansubmit.cgi for $hostName"\
           "Inconsistent input - two Gaussian directives files specified"
    } else {
	if {$remGaussianFile != ""} {
	    # Attempt to copy remote Gaussian directives file for processing
	    # Input file path is relative to working directory (remDir)
	    set tmpGaussianFile "/tmp/tmpGaussian.[pid]"
	    set clobberGaussian 1
	    if [catch {webSubmit::network::scpFile $tmpGaussianFile get 1 $remUser $hostName $remGaussianFile} result] {
		cgiBomb "nqsGaussiansubmit.cgi for $hostName" "$result"
	    }
	} else {
	    # Directives file has been uploaded from user browser system
	    set tmpGaussianFile [lindex $locGaussianFile 0]
	    
	    # Name of remote Gaussian file set to match name of file on browser system
	    set tmpName [file tail [lindex $locGaussianFile 1]]
	    set remGaussianFile [mkPathAbs $tmpName $remDir]
	}
    }
    
    # Set up job_name based on input values
    if {$jobName == "IF"} {
	# Job name based on directives file name
	set QSUBOPTS(-r) "[file tail $remGaussianFile].job"
    }
    
    # Set number of processors if user-defined and not provided
    if {$Gaussian(NProcType) == "UD" && $QSUBOPTS(-l,mpp_p) == ""} {
	set QSUBOPTS(-l,mpp_p) 1
    }

    ########################
    # Generate HTML
    ########################
    cgi_http_head
    
    set workingTitle "$bqs $title Report"

    cgi_html {
	cgi_head {cgi_title "$workingTitle"}
	cgi_body bgcolor=$masterBgcolor {
	    cgi_center {
		cgi_h2 "$workingTitle[cgi_nl]\
		    System [cgi_font color=blue "$hostName"]"
	    }

	    # Check resource limits
	    checkLimits $hostName maxCPUTime $QSUBOPTS(-lT)
	    checkLimits $hostName maxMemory $QSUBOPTS(-lM) $memUnits
	    append QSUBOPTS(-lM) $memUnits

	    # Submit Gaussian Job
	    submitGaussianJob $remDir $hostName $remUser ${gvers.option}\
                $clobberGaussian $tmpGaussianFile $remGaussianFile
	}
    }
}





















