/*************************************************************************
 *                         CLASS: netDirBrowser
 *  R.P. McCormack
 *  Last Revised: 01.12.99
 *
 *   Used to browse through directories on a collection of 
 *   remote filesystems.  Directory querying is done using a CGI
 *   proxy contained within the WebSubmit package.  The Applet is
 *   meant to be spawned by a 'Browse...' button in an HTML form.
 *
 *   Applet Buttons:
 *   ---------------
 *    Browse: Browse the specified directory on the chosen host
 *    Done: Close the Applet and place the selected file in the HTML form
 *    Cancel: Close the Applet and perform no action
 *
 *   Parameters passed to Applet:
 *   ----------------------------
 *    hostList: Comma-separated list of valid hosts for browsing
 *    form: Name of the form containing the file text field to be filled
 *    filefield: Name of the file text field
 *
 *   Event Handling:
 *   ---------------
 *    File List Box:
 *      Click: Select file (no result if file = directory)
 *      Dbl-Click: Change into the specified directory (no action if file)
 *    Directory Entry Field:
 *      <Return>: Equivalent to pressing 'Browse'
 *    Host Name Choice:
 *      Change Host: Clears all fields of the Applet
 *
 *   Remote Error Handling:
 *   ----------------------
 *    The command submission script in WebSubmit has been configured to
 *    return a line of the form 'ERROR: ...' when an error has occurred
 *    (e.g., the directory could not be read by the current user).  The
 *    parser in the Applet looks for this string and breaks further
 *    processing if it is seen.
 *
 *   Portability:
 *   ------------
 *    The Applet will not work properly with MSIE browsers, since LiveConnect
 *    (the interface between Java and Javascript) is used to allow the 
 *    Applet to alter contents of the HTML form.
 **************************************************************************/

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;
import java.util.*;
import netscape.javascript.JSObject;

public class netDirBrowser extends Applet implements ItemListener {
  // Properties in the applet
  TextField dirName;
  TextField fileName;
  Choice hostName;
  List fileList;
  TextArea output;
  Button browse;
  Button done;
  Button cancel;
  URL proxyURL;
  String form;
  String filefield;
  Hashtable directories = new Hashtable();

  // Colors
  Color darkSlateGrey = new Color(47,79,79);
  Color slateGrey = new Color(112,128,144);
  Color antiqueWhite = new Color(250,235,215);
  Color paleYellow = new Color(255,255,224);
  Color rosyBrown3 = new Color(205,155,155);

  // Fonts
  Font monoFont = new Font("Monospaced",Font.PLAIN,10);
  Font timesBold = new Font("Serif",Font.BOLD,12);
  Font times = new Font("Serif",Font.PLAIN,12);


  // INIT method for applet
  public void init() {
    setLayout(new BorderLayout());

    // Top level panels
    Panel p1 = new Panel();
    Panel p2 = new Panel();
    Panel p21 = new Panel();
    Panel p22 = new Panel();
    Panel p23 = new Panel();
    p1.setLayout(new BorderLayout());
    p2.setLayout(new FlowLayout(FlowLayout.LEFT));
    p2.setBackground(slateGrey);
    p21.setLayout(new GridLayout(3,1));
    p22.setLayout(new GridLayout(3,1));
    p23.setLayout(new GridLayout(3,1));

    // List for files
    fileList = new List(10,false);
    fileList.addItemListener(this);
    fileList.addActionListener(new ListActionListener());
    fileList.setFont(monoFont);
    fileList.add("");
    fileList.setBackground(rosyBrown3);

    // Text area for output and messages
    output = new TextArea(5,60);
    output.setEditable(false);
    output.setFont(monoFont);
    output.setBackground(slateGrey);

    // Panels containing host and directory fields
    Label dirLabel = new Label("Directory: ");
    Label hostLabel = new Label("Host: ");
    Label fileLabel = new Label("FileName: ");
    dirLabel.setFont(timesBold);
    hostLabel.setFont(timesBold);
    fileLabel.setFont(timesBold);
    dirName = new TextField(40);
    dirName.setBackground(slateGrey);
    dirName.setFont(monoFont);
    dirName.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
	getListing();
      }
    });
    hostName = new Choice();
    hostName.setBackground(darkSlateGrey);
    hostName.setForeground(paleYellow);
    hostName.setFont(monoFont);
    fileName = new TextField(40);
    fileName.setFont(monoFont);
    fileName.setBackground(slateGrey);

    // Retrieve list of hosts from parameter list and parse
    String hostList = this.getParameter("hostList");
    StringTokenizer stHosts = new StringTokenizer(hostList,",");
    while (stHosts.hasMoreTokens()) {
      hostName.add(stHosts.nextToken());
    }
    hostName.addItemListener(new ItemListener() {
        public void itemStateChanged(ItemEvent event) {
	  setPwd("");
	  setFileName("");
	  fileList.removeAll();
	}
    });

    // Retrieve selected host if possible
    setSelectedHost(this.getParameter("remHost"));
    
    // Retrieve proxy URL for performing directory queries
    String prURL = this.getParameter("proxyURL");
    try {
      proxyURL = new URL(prURL);
    }
    catch (MalformedURLException e) {
      showText("Invalid proxy URL " + prURL);
    }

    // Button to retrieve directory
    browse = new Button("Browse");
    browse.setFont(timesBold);
    browse.setBackground(darkSlateGrey);
    browse.setForeground(paleYellow);
    browse.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
	// Clear hash table of all entries
	directories.clear();
	
	// Retrieve listing
	getListing();
      }
    });

    // Button to pass current file to form and exit
    form=this.getParameter("form");
    filefield=this.getParameter("filefield");
    done = new Button("Done");
    done.setFont(timesBold);
    done.setBackground(darkSlateGrey);
    done.setForeground(paleYellow);
    done.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
	try {
	  JSObject jsroot = JSObject.getWindow(netDirBrowser.this);
	  String pwd = getPwd();
	  String fName = getFileName();
	  String fullFilePath;
	  if (!fName.equals("")) {
	    String javaScriptCmd = new String("self.opener.document." + form + "."
               + filefield + ".value = '" + getFilePath() + "';");
	    jsroot.eval(javaScriptCmd);
	  }
	  jsroot.eval("self.close()");
	}
	catch (Exception e) {
	  showText(e.toString());
	}
      }
    });

    // Button to cancel
    cancel = new Button("Cancel");
    cancel.setFont(timesBold);
    cancel.setBackground(darkSlateGrey);
    cancel.setForeground(paleYellow);
    cancel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
	try {
	  JSObject jsroot = JSObject.getWindow(netDirBrowser.this);
	  jsroot.eval("self.close()");
	}
	catch (Exception e) {
	  showText(e.toString());
	}
      }
    });    

    // Add main panels to layout
    add("North",p1);
    add("Center",fileList);
    add("South",output);

    // Add subpanels to layout
    p21.add(hostLabel);
    p21.add(dirLabel);
    p21.add(fileLabel);
    p22.add(hostName);
    p22.add(dirName);
    p22.add(fileName);
    p23.add(browse);
    p23.add(done);
    p23.add(cancel);
    p2.add(p21);
    p2.add(p22);
    p2.add(p23);
    p1.add("Center",p2);
  }


  // *************************************************
  //                 EVENT HANDLERS
  // *************************************************

  // Event Handler for single Clicks in File List
  // Triggered event sets the filename field to the selected list element
  public void itemStateChanged(ItemEvent event) {
    String file = fileList.getSelectedItem();
    if (!file.equals("./")) {
      if (!file.endsWith("/")) {
	setFileName(file);
      } else {
	setFileName("");
      }
    }
  }

  // Listener inner class for double clicks in file list
  class ListActionListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      // Retrieve currently selected item and determine
      //  Whether it is a directory or not
      String eltName = fileList.getSelectedItem();
      if (eltName.endsWith("/")) {
	if (!eltName.equals("./")) {
	  if (eltName.equals("../")) {
	    // Parent directory
	    File dir = new File(getPwd());
	    setPwd(dir.getParent());
	    setFileName("");
	    getListing();
	  } else {
	    String sel = fileList.getSelectedItem();
	    setPwd(getPwd() + "/" + sel.substring(0,sel.length()-1));
	    setFileName("");
	    getListing();
	  }
	}
      }
    }
  }


  // *************************************************
  //              REGULAR CLASS METHODS
  // *************************************************

  // Method to construct a safe query string for a URL
  public String queryString(Object name, Object value) {
    String query = URLEncoder.encode(name.toString()) + "=" +
      URLEncoder.encode(value.toString());
    return query;
  }


  // Method to build an entire query and construct the URL
  public String getQueryUrl(String host, String command) {
    // Construct the required query string
    String query = "";
    query += queryString("hostName",host);
    query += "&" + queryString("moduleName","genExecCmd");
    query += "&" + queryString("title","Command+Execution");
    query += "&" + queryString("workDir","");
    query += "&" + queryString("noHTML","1");
    query += "&" + queryString("oneCmd",command);

    // Construct CGI request URL
    String cgiUrl = proxyURL.toString() + "?" + query;
    return cgiUrl;
  }


  // Method to retrieve a URL
  public Object fetchUrl(String s) {
    Object o = new Object();
    // Retrieve the URL
    try {
      URL u = new URL(s);
      try {
	o = u.getContent();
      }
      catch (IOException e) {
	showText("Could not connect to " + u.getHost() + "\n" + e.toString());
      }
      catch (NullPointerException e) {
	showText("There was a problem with the content\n" + e.toString());
      }
    }
    catch (MalformedURLException e) {
      showText(s + " is not a valid URL\n" + e.toString());
    }
    return o;
  }


  // Method to retrieve a directory listing
  // Listing is either downloaded from a URL or pulled from a cache
  public void getListing() {
    String pwd = getPwd();
    String host = getSelectedHost();
    String key = new String(host + ":" + pwd);
    Object o = new Object();
    if (directories.containsKey(key)) {
      // Retrieve listing from hashtable and parse
      showText("Loading cached copy of " + key);
      parseFileNames((String)directories.get(key));
    } else {
      // Retrieve listing from URL
      showText("Retrieving directory information...please wait");
      String command = "cd " + pwd + "; pwd; /bin/ls -ap";

      try {
	o = fetchUrl(getQueryUrl(host,command));
      }
      catch (Exception e) {
	showText("Error during URL query:\n" + e.toString());
      }
      
      // Retrieve list of files
      if (o instanceof InputStream) {
	parseFileNames((InputStream) o);
      } else if (o instanceof String) {
	parseFileNames((String) o);
      } else {
	showText("Unknown object returned from URL query...");
	return;
      }
      
      // Current Working Directory
      //  Present working directory is contained in first returned entry
      setPwd(fileList.getItem(0));
      fileList.delItem(0);

      // Add this directory's contents to the hashtable
      if (fileList.getItemCount() > 0) {
	String[] listing = fileList.getItems();
	StringBuffer tmp = new StringBuffer(listing[0]);
	for (int i = 1; i < listing.length; i++) {
	  tmp.append("\n"+listing[i]);
	}

	String newKey = new String(host + ":" + getPwd());
	directories.put(newKey,tmp.toString());
      }
    }

    // Unset current filename
    setFileName("");
  }


  // Method to break apart the string representing a file list
  void parseFileNames(String s) {
    fileList.removeAll();
    StringTokenizer st = new StringTokenizer(s,"\n");
    while (st.hasMoreTokens()) {
      String nt = st.nextToken();
      if (nt.startsWith("ERROR:")) {
	showText(nt);
	break;
      } else {
	fileList.add(nt);
      }
    }
  }

  // Method to parse file list input stream
  void parseFileNames(InputStream is) {
    String nextline;
    
    fileList.removeAll();
    try {
      BufferedReader in = new BufferedReader(new InputStreamReader(is));
      while ((nextline = in.readLine()) != null) {
	if (nextline.startsWith("ERROR:")) {
	  showText(nextline);
	  break;
	} else {
	  fileList.add(nextline);
	}
      }
    }
    catch (IOException e) {
      showText(e.toString());
    }
  }


  // Method to display messages in output window
  void showText(String s) {
    output.append(s + "\n");
  }


  // Return the current working directory
  public String getPwd() {
    return dirName.getText();
  }


  // Set the current working directory
  public void setPwd(String s) {
    dirName.setText(s);
  }


  // Return the currently selected file
  public String getFileName() {
    String fn;
    String tmpName = fileName.getText();
    if (tmpName.endsWith("/")) {
      fn = new String(tmpName.substring(0,tmpName.length() - 1));
    } else {
      fn = new String(tmpName);
    }
    return fn;
  }


  // Set the current fileName
  public void setFileName(String s) {
    fileName.setText(s);
  }

  // Get the currently selected host
  public String getSelectedHost() {
    return hostName.getSelectedItem();
  }

  // Set the currently selected host
  public void setSelectedHost(String s) {
    hostName.select(s);
  }

  // Construct the full file path based on pwd and fileName
  public String getFilePath() {
    String fName = getFileName();
    String fullFilePath;
    String pwd = getPwd();
    if (!pwd.equals("")) {
      fullFilePath = new String(pwd + "/" + fName);
    } else {
      fullFilePath = new String(fName);
    }
    return fullFilePath;
  }
}




