#!/bin/sh
# The following line will be ignored by Tcl after the exec\
exec tclsh "$0" "$@"

# Determine the install directory
set installDir [pwd]

# Determine the WebSubmit directory structure
set wsRootDir "/sasp/WebSubmit/webSubmit2.5"
set wsLibDir "/sasp/WebSubmit/webSubmit2.5/lib"

# Determine the full path to the current Tcl interpreter
set tclInterpPath [info nameofexecutable]

# Attempt to locate expect
if [catch {exec which expect} expectPath] {
    set expectPath [file join $wsRootDir extensions bin expect]
    if ![file executable $expectPath] {
	set expectPath "Unable to locate"
    }
}

# Set up the path to the widget package for wsManager
set widgetPath [file join $wsRootDir extensions widget]

# Set up default paths for cgi.tcl and Tclbin
set cgiPath "/sasp/cgi.tcl-0.7"
set tclBinPath [file join $wsRootDir extensions lib]

# Write necessary directory information in wsManager.tcl
set managerFile [file join $installDir wsManager.tcl]
if [catch {open "$managerFile" r+} fd] {
    puts "Error opening $managerFile...exiting"
    exit
} else {
    set data [split [read $fd] \n]
    set idx [lsearch -exact $data "# DIRECTORY SETUP"]
    if {$idx == -1} {
	puts "Error: Could not update directory information in $managerFile"
	exit
    } else {
	seek $fd 0 start
	set l1 "set installDir \"$installDir\""
set wsRootDir "/sasp/WebSubmit/webSubmit2.5"
set wsLibDir "/sasp/WebSubmit/webSubmit2.5/lib"
	set insert [expr $idx + 1]
	if ![string match set* [lindex $data $insert]] {
	    set data [linsert $data $insert $l1 $l2 $l3]
	} else {
	    set data [lreplace $data $insert [expr $insert + 2] $l1 $l2 $l3]
	}
    }
    foreach line $data {
	puts $fd $line
    }
    close $fd
}

# Create file containing relevant path information
set pathFile [file join $installDir websubmit.paths]
set varList [list installDir wsLibDir tclInterpPath expectPath widgetPath cgiPath tclBinPath]
if [catch {open "$pathFile" w} fd] {
    puts "Error opening $pathFile file...exiting"
    exit
} else {
    # Append data for relevant variables
    set date [exec date]
    puts "\n\nWebSubmit Configuration Information ($pathFile)"
    puts $fd "########################################################################"
    puts $fd "# Configuration information (created $date)"
    puts $fd "#   This information was automatically generated by ./configure.tcl"
    puts $fd "########################################################################"
    foreach var $varList {
	set value [set $var]
	puts " $var:  \"$value\""
	puts $fd "set $var \"$value\""
    }
    puts $fd "lappend auto_path \$widgetPath \$installDir \$wsLibDir \$cgiPath \$tclBinPath"
    puts $fd "\nreturn 0"
    close $fd
}






