#############################################################################
#  WebSubmit Configuration and Management package
#############################################################################

package provide wsConfigure 1.0

###################################
# Labeled entry box
###################################
proc labelEntry {parent name label width args} {
    set w $parent.$name
    frame $w
    label $w.label -text $label -width $width -anchor w
    eval {entry $w.entry} $args
    pack $w.entry -side right -padx 1m -pady 1m
    pack $w.label -side right -fill x -padx 4m -pady 1m
    return $parent.$name
}

###########################################
# Checkbutton for tab text widgets
###########################################
proc tabCheckButton {parent name text variable args} {
    set w $parent.$name
    eval {checkbutton $w  -text "$text" -variable $variable} $args
    return $w
}

##########################
# A simple error dialog
##########################
proc errorDialog {msg} {
    tk_messageBox -icon error -message $msg -title "Error" -type ok
    return
}

##########################
# A simple yes/no dialog
##########################
proc yesnoDialog {msg} {
    set answer [tk_messageBox -icon question -message $msg -title "Action Verification" -type yesno]
    return $answer
}

##########################
# A simple ok dialog
##########################
proc okDialog {msg} {
    tk_messageBox -icon info -message $msg -title "Message" -type ok
    return
}

#########################################
# Dialog creation and destruction macros
#  See Welch, Ed.2, pp. 437
#########################################
proc Dialog_Create {top title args} {
    global dialog
    if [winfo exists $top] {
	switch -- [wm state $top] {
	    normal {
		# Raise a buried window
		raise $top
	    }
	    withdrawn -
	    iconified {
		# Open and restore geometry
		wm deiconify $top
		catch {wm geometry $top $dialog(geo,$top)}
	    }
	}
	return 0
    } else {
	eval {toplevel $top} $args
	wm title $top $title
	return 1
    }
}

proc Dialog_Wait {top varName {focus {}}} {
    upvar $varName var
    
    # Poke the variable if the user nukes the window
    bind $top <Destroy> [list set $varName $var]

    # Grab focus for the dialog
    if {[string length $focus] == 0} {
	set focus $top
    }
    set old [focus -displayof $top]
    focus $focus
    catch {tkwait visibility $top}
    catch {grab $top}

    # Wait for the dialog to complete
    tkwait variable $varName
    catch {grab release $top}
    focus $old
}

proc Dialog_Dismiss {top} {
    global dialog
    # Save the current dize and position
    catch {
	# Window may have been deleted
	set dialog(geo,$top) [wm geometry $top]
	wm withdraw $top
    }
}

########################################
# A simple prompter dialog
#  See Welch, Ed. 2, pp. 439
########################################
proc Dialog_Prompt {string {state ""}} {
    global prompt
    set f .prompt
    if [Dialog_Create $f "Prompt Dialog" -borderwidth 10 -class Dialog] {
	message $f.msg -text $string -aspect 1000
	entry $f.entry -textvariable prompt(result)
	set b [frame $f.buttons]
	pack $f.msg $f.entry $f.buttons -side top -fill x
	pack $f.entry -pady 5

	button $b.ok -text Ok -default active -command {set prompt(ok) 1}
	button $b.cancel -text Cancel -command {set prompt(ok) 0}
	pack $b.ok -side left -fill x -expand 1 -padx 1m -pady 1m
	pack $b.cancel -side right -fill x -expand 1 -padx 4m
	bind $f.entry <Return> {set prompt(ok) 1 ; break}
	bind $f.entry <Control-c> {set prompt(ok) 0 ; break}
    }
    set prompt(ok) 0
    Dialog_Wait $f prompt(ok) $f.entry
    if {$prompt(ok)} {
	set rVal $prompt(result)
    } else {
	set rVal {}
    }
    destroy $f
    if {$state == ""} {
	catch {unset prompt(result)}
    }
    return $rVal
}


########################################
# A simple select dialog
#  See Welch, Ed. 2, pp. 439
########################################
proc Dialog_Select {string list} {
    global select
    set f .select
    if [Dialog_Create $f "Select Dialog" -borderwidth 10 -class Dialog] {
	set s [frame $f.selection]
	set b [frame $f.buttons]
	pack $s $b -side top -fill x
	message $s.msg -text $string -aspect 1000
	eval tk_optionMenu $s.opt select(result) $list
	pack $s.msg $s.opt -side left -fill x -expand 1 -padx 2m -pady 2m

	set o [frame $b.default -relief sunken -bd 1]
	button $o.ok -text Ok -command {set select(ok) 1}
	bind $o.ok <Return> {set select(ok) 1 ; break}
	button $b.cancel -text Cancel -command {set select(ok) 0}
	pack $o -side left  -fill x -expand 1 -padx 4m
	pack $o.ok -side left -fill both -expand 1 -padx 1m -pady 1m
	pack $b.cancel -side right
    }
    set select(ok) 0
    Dialog_Wait $f select(ok)
    if {$select(ok)} {
	set rVal $select(result)
    } else {
	set rVal {}
    }
    destroy $f
    catch {unset select(result)}
    return $rVal
}

####################################################
# Megawidget for database records
####################################################
proc dbRecordFrame {parent key} {
    global debug labelWidth sectionHdColor DBMAP ATTR
    foreach var [list attrList DB dbKeyList] {
	upvar #0 $DBMAP($parent,$var) $var
    }
    set keyText $DBMAP($parent,keyText)

    # Avoid problems with names and Tk widget-nameing conventions
    set name [string tolower $key]

    # Header
    set w $parent
    set bgColor "ivory1"
    set hdr [frame $w.${name}_header -bg $bgColor -relief ridge -bd 3]
    set hdrTxt [label $hdr.txt -font {times 12 bold} -justify left -text "$keyText:" -bg $bgColor -fg $sectionHdColor]
    set hdrVal [label $hdr.key -font {times 12 normal} -justify left -bg $bgColor -text "$key"]
    set delete [button $hdr.delete -text "Delete" -command "dbRecord delete $w $key"]
    pack $hdrTxt $hdrVal -side left -pady 1m
    pack $delete -side right -pady 1m -padx 1m

    # Insert header frame into text widget
    $parent window create end -window $hdr -padx 10
    $parent insert end "\n"
    
    # Record Attributes
    set dbArrName $DBMAP($parent,DB)
    set numAttr [llength $attrList]
    for {set i 0} {$i < $numAttr} {incr i} {
	set var [lindex $attrList $i]
	if {$var != "auto"} {
	    set ATTR($key,$var) [labelEntry $w ${name}_attr$i $var $labelWidth -textvariable ${dbArrName}($key,$var)]

	    # Insert attribute frame into text widget
	    $parent window create end -window $ATTR($key,$var)

	    # Tag this key-attribute pair
	    $parent tag add ${key}_$var [$parent index $ATTR($key,$var)] insert
	    $parent insert end "\n"
	}
    }
    $parent insert end "\n\n"

    # Terminate tag after record for future deletion
    set tagEnd [$parent index insert]

    $parent tag add $key [$parent index $hdr] $tagEnd
    return
}

####################################################
#  Constructor for a tab notebook
####################################################
proc notebook {name args} {
    global tabCoverColor
    frame $name -relief sunken
    set tab [eval tabnotebook $name.tab $args]
    pack $tab -fill both -expand 1
    $tab.hold configure -background $tabCoverColor
    return $name
}

####################################################
#  Setup procedure for a tab textbox containing a
#   series of database records.  This is a generic
#   database procedure.  Specialized procedures 
#   must be implemented on a case-by-case basis.
####################################################
proc tabDb {buttonParent parent} {
    global DBMAP

    # Build Database
    if {!($DBMAP($parent,made))} {
	eval $DBMAP($parent,dbMake)
	dbBackup create $parent
    }

    # Database action commands
    if {!($DBMAP($parent,made))} {
	set b $buttonParent
	set b0 [button $b.search -text "Find" -command "searchDbTab $parent"]
	set b1 [button $b.addRec -text "Add Record" -command "dbRecord add $parent {}"]
	set b2 [button $b.addAttr -text "Add Field" -command \
		"dbAttribute add $parent ; drawTab $parent"]
	set b3 [button $b.delAttr -text "Delete Field" -command \
		"dbAttribute delete $parent ; drawTab $parent"]
	set b4 [button $b.save -text "Save" -command "saveDb $parent ; dbBackup create $parent"]
	set b5 [button $b.saveas -text "Save As..." -command "saveDb $parent getName ; dbBackup create $parent"]
	set b6 [button $b.restore -text "Restore" -command "dbBackup restore $parent"] 
	pack $b6 $b5 $b4 $b3 $b2 $b1 $b0 -side right -padx 2m -pady 2m
	set DBMAP($parent,made) 1
    }

    foreach key [set $DBMAP($parent,dbKeyList)] {
	dbRecordFrame $parent $key
    }
    $parent configure -state disabled
}


####################################################
#  Setup procedure for a tab textbox containing a
#   series of database records.  This is a generic
#   database procedure.  Specialized procedures 
#   must be implemented on a case-by-case basis.
####################################################
proc authTabDb {buttonParent parent} {
    global DBMAP
    set dbNS $DBMAP($parent,NS)

    # Build Database
    if {!($DBMAP($parent,made))} {
	eval $DBMAP($parent,dbMake)
	dbBackup create $parent
    }

    # Database action commands
    if {!($DBMAP($parent,made))} {
	set b $buttonParent
	set b0 [button $b.search -text "Find" -command\
		"searchDbTab $parent"]
	set b1 [button $b.addUser -text "Add User" -command "dbRecord add $parent {}"]
	set b2 [button $b.addAttr -text "Add Host" -command\
		"dbAttribute add $parent ; ${dbNS}::getHosts ; drawTab $parent"]
	set b3 [button $b.delAttr -text "Delete Host" -command\
		"dbAttribute delete $parent ; ${dbNS}::getHosts ; drawTab $parent"]
	set b4 [button $b.save -text "Save" -command "saveDb $parent ; dbBackup create $parent"]
	set b5 [button $b.saveas -text "Save As..." -command\
		"saveDb $parent getName ; dbBackup create $parent"]
	set b6 [button $b.restore -text "Restore" -command "dbBackup restore $parent; ${dbNS}::getHosts"]
	pack $b6 $b5 $b4 $b3 $b2 $b1 $b0 -side right -padx 1m -pady 2m
	set DBMAP($parent,made) 1
    }

    foreach key [set $DBMAP($parent,dbKeyList)] {
	dbRecordFrame $parent $key
    }
    $parent configure -state disabled
}

####################################################
#  Setup procedure for a tab textbox to act as a 
#   simple file editor for WS packages
####################################################
proc tabEditor {name {fileInfoList ""} {fileDirName ""}} {
    upvar #0 $fileDirName fileDir
    global debug labelWidth FILE sectionHdColor PACKAGE SELPACK

    # Store build command
    set buildCmd [info level 0]

    # File loading and saving buttons
    set h [frame $name -relief sunken -bd 1]
    set b [frame $h.buttons]

    # Set up either vanilla editor or specialized package editor
    if {$fileInfoList != ""} {
	set lab [label $b.lab -text "Package Name:  "]
	set max 0
	foreach {packName packFile} $fileInfoList {
	    set PACKAGE($packName) [file join $fileDir $packFile]
	    lappend packageList $packName
	    set len [string length $packName]
	    if {$len > $max} {
		set max $len
	    }
	}
	
	eval tk_optionMenu $b.opt SELPACK($h) $packageList
	$b.opt configure -width $max
	set b1 [button $b.load -text "Load Package" -command "loadText $h.editor \$PACKAGE(\$SELPACK($h))"]
	pack $lab $b.opt $b1 -side left -padx 1m -pady 1m
    } else {
	set b1 [button $b.load -text "Load File" -command "loadText $h.editor"]
	pack $b1 -side left -padx 1m -pady 1m
    }
    set bsave [button $b.save -text "Save" -command "saveText $h.editor \$PACKAGE(\$SELPACK($h))"]
    set bsaveas [button $b.saveas -text "Save As..." -command "saveText $h.editor"]
    pack $bsaveas $bsave -side right -padx 1m -pady 1m
    pack $b -side top -fill x -padx 2m -pady 2m

    # Additional file details and actions
    set a [frame $h.file]
    set fname [labelEntry $a filename "Current File: " 10 -textvariable FILE($h.editor) -fg $sectionHdColor -width 55]
    set fsrch [button $a.search -text "Search text..." -command "searchText $h.editor"]
    pack $fname -side left -padx 2m
    pack $fsrch -side right -padx 2m
    pack $a -side top -fill x -padx 1m

    # Insert a scrolled text frame to hold the file being edited
    set t [scrolledtext $h.editor -height 20 -bg gray -fg black -relief sunken -borderwidth 2 -autoscrollbar 0]
    $h.editor tag configure highlight -background linen
    pack $t -side top -pady 5 -padx 5 -fill both -expand 1

    return $name
}

#######################################################
# Procedure to load a text file into a text widget
#######################################################
proc loadText {parent {file ""}} {
    global FILE wsLibDir
    # Prompt for filename if none given
    if {$file == ""} {
	set file [tk_getOpenFile -initialdir $wsLibDir -title "File Locator"]
	if {$file == ""} {
	    writeOutput "File load aborted..."
	    return
	}
    }

    # Check for file existence
    if ![file exists $file] {
	writeOutput "File $file does not exist..." ERROR
	return
    }

    # Open file and load into text widget
    if [catch {open $file r} fd] {
	writeOutput "File could not be opened: $fd" ERROR
	return 
    } else {
	# Erase widget
	$parent delete 1.0 end
	foreach line [split [read $fd] \n] {
	    if [regexp "\[ \t\]*;;\[a-zA-Z\]+" $line match] {
		$parent insert end "$line\n" highlight
	    } else {
		$parent insert end "$line\n"
	    }
	}
	writeOutput "Loaded file $file..."
	set FILE($parent) $file
	close $fd
    }
    return
}

#######################################################
# Procedure to save the contents of a text widget
#  to a file
#######################################################
proc saveText {parent {file ""}} {
    global FILE wsLibDir
    # Prompt for filename if none given
    if {$file == ""} {
	set file [tk_getSaveFile -initialdir $wsLibDir -title "Save File"]
	if {$file == ""} {
	    writeOutput "Save aborted..."
	    return
	}
    }

    # Check for file existence
    if [file exists $file] {
	set ans [yesnoDialog "File exists. Overwrite?"]
	if {$ans == "no"} {
	    writeOutput "Save aborted..."
	    return
	}
    }

    # Open file and dump text widget data
    if [catch {open $file w+} fd] {
	writeOutput "File could not be opened for writing: $fd" ERROR
	return 
    } else {
	# Retrieve all text
	set fileText [$parent get 1.0 end]
	foreach line [split $fileText \n] {
	    puts $fd $line
	}
	writeOutput "Saved text data in $file..."
	close $fd
    }
    return
}


###########################################################
# Procedure to search a text widget for text
###########################################################
proc searchText {parent {string ""}} {
    # Prompt for search string if none given
    if {$string == ""} {
	set string [Dialog_Prompt "Enter search string:" save]
	if {$string == ""} {
	    return
	}
    }

    # Clear the current selection
    clearSelect $parent

    # Search text widget for string
    set cnt 0
    set idx [$parent search -count cnt -forward -regexp -- $string insert]
    if {$cnt != 0} {
	# Select the found text
	$parent tag add sel $idx "$idx +$cnt chars"
	$parent see $idx
	$parent mark set insert "$idx +$cnt chars"
    } else {
	okDialog "String \"$string\" not found in file"
	$parent mark set insert 1.0
    }
    return
}

##################################################
# Clear the selection in a text widget
##################################################
proc clearSelect {parent} {
    catch {eval $parent tag remove sel [$parent tag ranges sel]}
    return
}


#######################################################
# Procedure for creating or restoring a copy of a 
#  database
#######################################################
proc dbBackup {command parent} {
    global DBMAP BACKUP
    foreach var [list DB dbKeyList attrList] {
	upvar #0 $DBMAP($parent,$var) $var
    }

    # Determine action based on command
    if {$command == "create"} {
	# Store database data in BACKUP array
	set BACKUP($DBMAP($parent,DB),dbKeyList) $dbKeyList
	set BACKUP($DBMAP($parent,DB),attrList) $attrList
	foreach key $dbKeyList {
	    foreach attr $attrList {
		if [info exists DB($key,$attr)] {
		    set BACKUP($DBMAP($parent,DB),$key,$attr) $DB($key,$attr)
		} else {
		    set BACKUP($DBMAP($parent,DB),$key,$attr) ""
		}
	    }
	}
    } elseif {$command == "restore"} {
	# Restore database, but get ok from user first
	set ans [yesnoDialog "This will restore the last saved\
		 version of the database.  Continue?"]
	if {$ans == "yes"} {
	    if [info exists BACKUP($DBMAP($parent,DB),dbKeyList)] {
		set dbKeyList $BACKUP($DBMAP($parent,DB),dbKeyList)
		set attrList $BACKUP($DBMAP($parent,DB),attrList)
		foreach key $dbKeyList {
		    foreach attr $attrList {
			if [info exists BACKUP($DBMAP($parent,DB),$key,$attr)] {
			    set DB($key,$attr) $BACKUP($DBMAP($parent,DB),$key,$attr)
			} else {
			    set DB($key,$attr) ""
			}
		    }
		}

		# Redraw the tab containing the db
		drawTab $parent
		writeOutput "Database restored..."
	    } else {
		errorDialog "No backup available...sorry!"
	    }
	} else {
	    writeOutput "Database restore aborted..."
	}
    }
    return
}


########################################################
# Procedure to write database
########################################################
proc saveDb {parent {saveAs ""}} {
    global DBMAP
    set dbArrName $DBMAP($parent,DB)

    # Determine namespace for database
    set dbNS [namespace qualifiers $dbArrName]

    # Write database, invoking dialog for filename if desired
    if {$saveAs != ""} {
	set fileName [tk_getSaveFile]
	if {$fileName != ""} {
	    if [catch {::webSubmit::database::writeDb $dbNS : $fileName} err] {
		errorDialog $err
		return -code return
	    }
	    writeOutput "Database $dbArrName saved as $fileName"
	} else {
	    writeOutput "Database save aborted..."
	    return
	}
    } else {
	if [catch {::webSubmit::database::writeDb $dbNS :} err] {
	    errorDialog $err
	    return -code return
	}
	writeOutput "Database $dbArrName saved"
    }
    return
}


########################################################
#  Procedure to search a DB tab for a certain string
########################################################
proc searchDbTab {parent} {
    global DBMAP
    foreach var [list DB dbKeyList attrList] {
	upvar #0 $DBMAP($parent,$var) $var
    }

    # Retrieve string to search for
    set str [Dialog_Prompt "Enter search string:"]
    if {$str != ""} {
	# Traverse data in the database
	#   Only allow moves forward in the text widget
	#   (Wrapping not currently supported)
	foreach key $dbKeyList {
	    regexp {([0-9]+)\.[0-9]+ +([0-9]+)\.[0-9]} [$parent tag range $key] match begin end
	    foreach attr $attrList {
		if [string match *$str* $DB($key,$attr)] {
		    if {$begin > [$parent index current]} {
			# Move to the position of this tag
			set idx [lindex [$parent tag range ${key}_$attr] 0]
			$parent see $idx
			return
		    }
		}
	    }
	}
    }
    return
}

#######################################################
# Procedure to destroy and redraw a tab if it is
#  updated
#######################################################
proc drawTab {parent} {
    global BUILDTABS DBMAP
    set db $DBMAP($parent,DB)

    set dbParent $BUILDTABS($db,parent)
    # Destroy all children of parent window
    if [winfo exists $dbParent] {
	# Activate text
	$dbParent configure -state normal

	# Delete all text within text widget
	$dbParent delete 1.0 end
    }

    # Rebuild notebook
    eval $BUILDTABS($db,build)
    return
}


#######################################################
# Procedure to create a text widget embedded in a tab
#  Constructor is the proc to build the contents of the
#  tab, and always takes its Tk pathname as the first
#  argument.  In other words, addTab constructs
#  a widget with Tk path $parent.$name using the
#  constructor proc.
#######################################################
proc addTab {parent name tabText constructor args} {
    global TABS
    set w $parent.$name

    # Check to see if window exists, and if it does, destroy it
    if [winfo exists $w] {
	destroy $w
    }

    set t [eval $constructor $w $args]
    set TABS($parent,$t) [$parent add " $tabText " -window $t]
    return $t
}


####################################################
# Procedure to add or delete DB records
####################################################
proc dbRecord {command parent key} {
    global DBMAP ATTR
    foreach var [list dbKeyList DB attrList] {
	upvar #0 $DBMAP($parent,$var) $var
    }

    set commandList [list add delete]
    if {[lsearch -exact $commandList $command] == -1} {
	errorDialog "Invalid DB record command...ignoring"
	return
    }
    
    if {$command == "delete"} {
	set keyIdx [lsearch -exact $dbKeyList $key]
	if {$keyIdx == -1} {
	    # Key not found in database
	    errorDialog "Key $key not found in Database"
	} else {
	    set dbKeyList [lreplace $dbKeyList $keyIdx $keyIdx]
	    foreach elt [array names DB $key*] {
		# Undo textvariable associations for this record
		if [info exists ATTR($elt)] {
		    if [winfo exists $ATTR($elt).entry] {
			catch {$ATTR($elt).entry configure -textvariable {}}
		    }
		}
		unset DB($elt)
	    }
	    
	    # Erase record from text widget
	    $parent configure -state normal
	    eval $parent delete [$parent tag range $key]
	    $parent configure -state disabled

	    writeOutput "Record $key deleted"
	}
    } elseif {$command == "add"} {
	if {$key == ""} {
	    # Retrieve key from entry dialog
	    set key [Dialog_Prompt "Enter a record key name:"]
	}
	set key [string trim $key]
	if {$key != ""} {
	    if {[lsearch -exact $dbKeyList $key] == -1} {
		lappend dbKeyList $key
		foreach attr $attrList {
		    set DB($key,$attr) {}
		}
		$parent configure -state normal
		dbRecordFrame $parent $key
		eval $parent see [lindex [$parent tag range $key] 0]
		$parent configure -state disabled
		
		writeOutput "Record $key added"
	    } else {
		errorDialog "Record \"$key\" already exists"
		return -code return
	    }
	} else {
	    writeOutput "Record addition aborted...nothing changed"
	}
    }
}


####################################################
# Procedure to add or delete DB attributes
####################################################
proc dbAttribute {command parent} {
    global DBMAP ATTR
    foreach var [list DB dbKeyList attrList fieldList] {
	upvar #0 $DBMAP($parent,$var) $var
    }

    set commandList [list add delete]
    if {[lsearch -exact $commandList $command] == -1} {
	errorDialog "Invalid DB attribute command...ignoring"
	return
    }
    
    if {$command == "delete"} {
	# Retrieve attribute from selection dialog
	if {$DBMAP($parent,displayList) != ""} {
	    set displayList [set $DBMAP($parent,displayList)]
	    set attr [Dialog_Select "Select a field:" $displayList]
	} else {
	    set attr [Dialog_Select "Select a field:"  $attrList]
	}
	if {$attr != ""} {
	    set attrIdx [lsearch -exact $attrList $attr]
	    if {$attrIdx == -1} {
		# Attribute not found in database
		errorDialog "Attribute $attr not found in Database"
		return -code return
	    } else {
		set attrList [lreplace $attrList $attrIdx $attrIdx]
		set fieldIdx [lsearch -exact $fieldList $attr]
		set fieldList [lreplace $fieldList $fieldIdx $fieldIdx]
		foreach elt [array names DB *$attr] {
		    # Disassociate variable in entry widget (if it exists)
		    if [info exists ATTR($elt)] {
			if [winfo exists $ATTR($elt).entry] {
			    catch {$ATTR($elt).entry configure -textvariable {}}
			}
		    }
		    unset DB($elt)
		}
		writeOutput "Field $attr deleted"
	    }
	} else {
	    writeOutput "Fields unmodified"
	    return -code return
	}
    } elseif {$command == "add"} {
	# Retrieve attribute from entry dialog
	set attr [Dialog_Prompt "Enter a field name:"]
	set attr [string trim $attr]
	if {$attr != ""} {
	    # Disallow duplicate fields
	    if {[lsearch -exact $attrList $attr] == -1} {
		lappend attrList $attr
		lappend fieldList $attr
		foreach key $dbKeyList {
		    set DB($key,$attr) ""
		}
		writeOutput "Field $attr added"
	    } else {
		errorDialog "Field \"$attr\" already exists"
		return -code return
	    }
	} else {
	    writeOutput "Field addition aborted...nothing changed"
	    return -code return
	}
    }
}


#############################################################
# Constructor procedure to create textbox containing widgets.
#  Data concerning the widgets is contained in the
#  array passed as argument
#############################################################
proc textMegawidget {name arrName args} {
    upvar #0 $arrName DATA
    global debug labelWidth sectionHdColor WIDGET

    # Create scrolledtext container
    set parent [eval scrolledtext $name $args]
    $parent tag configure header -foreground $sectionHdColor -lmargin1 5 -spacing1 5 -spacing3 5 -font {times 14 normal}
    set widPadX 10

    if ![info exists DATA(sectionList)] {
	set DATA(sectionList) main
	set DATA(main,header) {}
	set DATA(main,varList) $DATA(varList)
    }

    foreach section $DATA(sectionList) {
	# Generate Header if necessary
	if {[info exists DATA($section,header)] && $DATA($section,header) != ""} {
	    $parent insert end "$DATA($section,header)\n" header
	}

	foreach var $DATA($section,varList) {
	    if ![info exists DATA($var,type)] {
		set DATA($var,type) "entry"
	    }
	    if ![info exists DATA($var,text)] {
		set DATA($var,text) "$var"
	    }
	    if ![info exists DATA($var,help)] {
		set DATA($var,help) "None"
	    }
	    if {![info exists DATA($var,args)] || ($DATA($var,args) == "{}")} {
		set DATA($var,args) ""
	    }
	    set type $DATA($var,type)
	    set text $DATA($var,text)
	    set help $DATA($var,help)
	    set args $DATA($var,args)
	    set widName [string tolower $var]
	    if {$type == "entry"} {
		set WIDGET($var) [eval labelEntry $parent $widName $text $labelWidth -textvariable $var $args]
		balloonhelp $WIDGET($var).label $help
		$parent window create end -window $WIDGET($var) -padx $widPadX
		$parent insert end "\n"
	    } elseif {$type == "checkButton"} {
		set WIDGET($var) [eval tabCheckButton $parent $widName $text $var $args]
		balloonhelp $WIDGET($var) $help
		$parent window create end -window $WIDGET($var) -padx $widPadX
		$parent insert end "\n"
	    } else {
		set WIDGET($var) [eval $type $parent.$widName $args]
		balloonhelp $WIDGET($var) $help
		$parent window create end -window $WIDGET($var) -padx $widPadX
		$parent insert end "\n"
	    }
	}
	$parent insert end "\n\n"
    }
    $parent configure -state disabled
    return $parent
}
    
##################################################
# Procedure to write output to output text widget
##################################################
proc writeOutput {msgList {header ""}} {
    global outputText
    $outputText configure -state normal
    if {$header != ""} {
	# Write text and flag with color to indicate error
	$outputText insert end "\n"
	$outputText insert end "----------- $header -----------\n" error
	foreach line [split $msgList \n] {
	    $outputText insert end "$line\n" error
	    $outputText see end
	    update idletasks
	}
	$outputText insert end "-----------------------------\n\n" error
    } else {
	foreach line [split $msgList \n] {
	    $outputText insert end "$line\n"
	    $outputText see end
	    update idletasks
	}
    }
    $outputText configure -state disabled
    return
}



# CODEINFO BEGIN
# info > Procedure to obtain the list of files in the WebSubmit distribution
# arg distFile> Name of distribution file (absolute pathname)
# return> List of distribution files or ERROR
# loc-sclr distFile> Name of distribution file (absolute pathname)
# loc-sclr fileId> Filehandle for distribution file
proc getDist {distFile} {
    global DIST distList installDir wsRootDir updateClean
    set distFile [file join $installDir $distFile]

    # Open file containing distribution
    if [catch {open $distFile r} fileId] {
	# Error opening distribution file
	writeOutput "Problem opening distribution file:\n$fileId" ERROR
	set updateClean 1
    }

    # Read input file
    set distList ""
    while {[gets $fileId line] >= 0} {
	if {[regexp {^#.*} $line match] || ($line == "")} {
	    # Comment or blank line - ignore
	} else {
	    if [regexp {([^ ]+) +([^ ]+)} $line match mode file] {
		# Resolve directory and name components of file
		set file [file join $wsRootDir $file]
		set workDir [file dirname $file]
		set fileName [file tail $file]
		set DIST($file,mode) $mode
		set DIST($file,dir) $workDir
		lappend distList $file
	    }
	}
    }
    close $fileId
    return 0
}


# CODEINFO BEGIN
# info> Procedure to scan a file and replace specified variable
#  definitions with new values.  Not done in the most efficient
#  manner, but it works...

# arg file> Name of file to update
# return> Error status
# loc-sclr file> Name of file to update
# loc-sclr fileId> Filehandle for File   
proc updateVars {file} {
    global UPDATEVARS

    # Certain files are marked for no update
    set fileDir [file dirname $file]
    if [file exists [file join $fileDir noupdate]] {
	# Don't update files in this directory!!!
	return
    }

    # Open file, scan for entry, and replace as necessary
    set fileName [file tail $file]
    if [catch {open $file r+} fileId] {
	writeOutput "Could not read $file for updating:\n$fileId" ERROR
	return -code error "Could not read $file for updating:\n$fileId"
    }

    # Extract lines and store
    set lines [split [read $fileId] \n]
    close $fileId

    # Reopen file and truncate
    if [catch {open $file w} fileId] {
	writeOutput "Could not update $file:\n$fileId" ERROR

	# Restore backup copy
	file copy -force $file.bak $file
	return -code error "Could not update $file:\n$fileId"
    }

    # Variable definitions
    foreach var $UPDATEVARS($file) {
	if ![regexp {([^\(]+)\((.*)\)} $var match arrName key] {
	    # Variable is string or list
	    global $var
	    set TYPE($var) "scalar"
	} else {
	    # Variable is array
	    global $arrName
	    set TYPE($var) "array"
	    set TYPE($var,name) $arrName
	    set TYPE($var,key) $key
	}
    }

    foreach line $lines {
	set outputLine $line
	foreach var $UPDATEVARS($file) {
	    if {$TYPE($var) == "scalar"} {
		set value [set $var]
		if [regexp "(\[\#\; \t\]*)(set|variable|readOnly) +$var +(\[^ \]+)" $line match white varName tail] {
		    # Found set expression for variable
		    writeOutput "    $fileName: $var ---> $value"
		    set outputLine "$white$varName $var \"$value\""
		    break
		}
	    } else {
		# Variable is array
		set value [set $var]
		set arrName $TYPE($var,name)
		set key $TYPE($var,key)
		if [regexp "(\[\#\; \t\]*)readOnlyArray +$arrName +$key +\[^ \]+ (.*)" $line match white tail] {
		    # Found readOnlyArray expression for variable
		    writeOutput "    $fileName: $var ---> $value"
		    set outputLine "${white}readOnlyArray $arrName $key \"$value\" $tail"
		    break
		} elseif [regexp "(\[\#\; \t\]*)set +$var +(\[^ \]+)" $line match white] {
		    # Found readOnlyArray expression for variable
		    writeOutput "    $fileName: $var ---> $value"
		    set outputLine "${white}set $var \"$value\""
		    break
		}
	    }
	}
	puts $fileId $outputLine
    }
    close $fileId
    return 0
}

# CODEINFO BEGIN
# info> Procedure to update interpreter path
# arg fileList> Filenames to update
# arg interp> Path to interpreter
# return> None
proc updateInterp {fileList interp} {
    foreach file $fileList {
	# Open file, scan for entry, and replace as necessary
	if [catch {open $file r+} fileId] {
	    return -code error "$fileId"
	}

	# Extract lines and store
	set lines [split [read $fileId] \n]
	close $fileId

	# Find line that matches interpreter string
	set interpId [lsearch -regexp $lines {^#!.*}]
	if {$interpId != -1} {
	    # Replace element with proper interpreter value
	    set lines [lreplace $lines $interpId $interpId "#!$interp"]
	    writeOutput "   $file"
	}

	# Reopen file and truncate
	if [catch {open $file w} fileId] {
	    return -code error $fileId
	}    

	# Write data
	foreach line $lines {
	    puts $fileId $line
	}
	close $fileId
    }
    return 0
}

# CODEINFO BEGIN
# info> Procedure to update Tcl source commands so that the sourced files
#     contain the proper full pathnames
# arg srcFile> File to scan for 'source' commands
# arg fileName> Filename used in source command
# arg dirName> Path to filename
# return> None

proc updateTclSource {srcFile fileName dirName} {
    # Open file, scan for entry, and replace as necessary
    if [catch {open $srcFile r+} fileId] {
	writeOutput "Could not update source commands for $srcFile" ERROR
	return -code return
    }

    # Extract lines and store
    set lines [split [read $fileId] \n]
    close $fileId

    # Reopen file and truncate
    if [catch {open $srcFile w} fileId] {
	return -code error $fileId
    }    
    
    # Find line that matches source command
    set srcId [lsearch -regexp $lines " *source .*$fileName"]
    set fullName [file join $dirName $fileName]
    set white ""
    if {$srcId != -1} {
	writeOutput "   [file tail $srcFile]"
	regexp "(\[\# \t\]*)source .*$fileName\$" $srcId match white 
	set lines [lreplace $lines $srcId $srcId "${white}source $fullName"]
    }

    # Write data
    foreach line $lines {
	puts $fileId $line
    }
    close $fileId
    return 0
}


#######################################################
# Verify Tcl interpreter is the correct version (8.x)
#######################################################
proc verifyInterp {tclInterp} {
    if [file exists $tclInterp] {
	# Check version of interpreter
	if [catch {exec echo puts \[info tclversion\] | $tclInterp} version] {
	    writeOutput "Could not verify interpreter version ($version)" WARNING
	} else {
	    if ![string match 8.* $version] {
		writeOutput "tclsh v8.x required, v$version found" ERROR
		return -code error "tclsh v8.x required, v$version found"
	    }
	}
    } else {
	writeOutput "Specified Tcl interpreter \"$tclInterp\" does not exist" ERROR
	return -code error "Specified Tcl interpreter \"$tclInterp\" does not exist"
    }
    return
}


########################################################
# Verify the existence of a package
########################################################
proc verifyPackage {packageName {packagePath ""}} {
    global auto_path
    if {$packagePath != ""} {
	if {[lsearch -exact $auto_path $packagePath] == -1} {
	    # Add package to auto_path
	    lappend auto_path $packagePath
	}
    }

    if [catch {uplevel "package require $packageName"} error] {
	return -code error "$error"
    } else {
	return
    }
}

#########################################################
# Verify distribution: Existence, Permissions, Ownership
#########################################################
proc verifyDist {} {
    global distList
    set verified 1
    foreach file $distList {
	set fileName [file tail $file]
	if ![file exists $file] {
	    writeOutput "   File \"$fileName\" appears to be missing..."
	    set verified 0
	} else {
	    writeOutput "    $file"
	}
    }
    return $verified
}


########################################################################
# Procedure to fix file permissions and ownership in the distribution
########################################################################
proc fixPerms {} {
    global DIST distList serverUser
    set currentUser [exec whoami]
    foreach file $distList {
	if [file exists $file] {
	    set fileName [file tail $file]
	    set mode [file attributes $file -permissions]
	    set owner [file attributes $file -owner]
	    set reqMode $DIST($file,mode)
	    if {"00$reqMode" != $mode} {
		writeOutput "    Changing permissions for $fileName to $reqMode"
		if [catch {exec /bin/chmod $reqMode $file} err] {
		    writeOutput "   WARNING: $err"
		}
	    }
	    
	    # Change file ownership to match HTTP serverUser
	    if {$owner != "$serverUser"} {
		if [catch {exec /bin/chown $serverUser $file} error] {
		    writeOutput "   WARNING:  Could not change ownership for $fileName ($error)"
		} else {
		    writeOutput "   Changed ownership for $fileName to $serverUser"
		}
	    }
	} else {
	    writeOutput "File \"$fileName\" appears to be missing..." WARNING
	}
    }
}


##############################################################
# Update variables in a group of files
##############################################################
proc fileUpdate {fileList varList} {
    global UPDATEVARS
    foreach var $varList {
	if ![catch {eval exec /sbin/grep -l $var $fileList} foundList] {
	    foreach file $foundList {
		lappend UPDATEVARS($file) $var
	    }
	}
    }
    
    # Update necessary files
    foreach file [array names UPDATEVARS] {
	# Create a backup copy of file
	file copy -force $file $file.bak

	# Update variables
	if [catch {updateVars $file} error] {
	    file copy -force $file.bak $file
	    writeOutput "Problem updating $file - changes discarded\nError Message: $error" ERROR
	    return -code return
	}
    }
}


##################################################
# Cleanup procedure to be called during updating
##################################################
proc cleanup {msg} {
    global siteFile
    if {[yesnoDialog "Error during update\nDelete site configuration file:\n$siteFile?"] == "yes"} {
	writeOutput "\n  Deleting $siteFile"
	file delete $siteFile
    }
    writeOutput "$msg" ERROR
    return -code return
}


######################################################################
# Procedures to control updates for various sections of code
#  updateTcl:  Interpreter + extensions
#  updateWs:   WebSubmit variables
#  updateUnix: UNIX command paths
#  updateHttp: HTTP server configuration variables
######################################################################
proc updateConfig {} {
    global wsRootDir distList TCLSETUP WSSETUP UNIXSETUP HTTPSETUP DIST configVarList siteFile
    eval global $configVarList
    set wsBinDir [file join $wsRootDir bin]
    set wsLibDir "/sasp/WebSubmit/webSubmit2.5/lib"
    set wsNbinDir [file join $wsRootDir nbin]
    set wsModuleDir [file join $wsRootDir modules]

    writeOutput "Updating configuration..." UPDATE
	
    # Get names, locations, and permissions for all distribution files
    getDist websubmit.dist

    # Verify Distribution
    if {$verify} {
	writeOutput "\n  Verifying distribution files..."
	set verified [verifyDist]
	if {$verified} {
	    writeOutput "\n  Distribution VERIFIED"
	} else {
	    writeOutput "Distribution verification failed" WARNING
	}
    }

    # Open file for site-specific configuration parameters
    if {$updateAll || $updateTcl || $updateWs || $updateUnix || $updateHttp} {
	# Open site configuration file
	writeOutput "\n  Updating site configuration file ($siteFile)"
	if [catch {open $siteFile a} fd] {
	    cleanup $fd
	}
	puts $fd "##########################################"
	puts $fd "# Site-specific configuration"
	puts $fd "#  Created [exec date]"
	puts $fd "##########################################"
    }

    # Get a list of all files in bin, nbin, and lib
    set binLibList [glob $wsBinDir/*.cgi $wsBinDir/*.conf $wsNbinDir/*.cgi $wsNbinDir/*.conf $wsLibDir/*.tcl]

    # Tcl interpreter and extensions
    if {$updateTcl} {
	# Verify version 8.x Tcl interpreter
	writeOutput "\n  Verifying 8.x Tcl interpreter..."
	if [catch {verifyInterp $tclInterpPath} err] {
	    cleanup $err
	}

	# Verify existence of required packages
	writeOutput "\n  Verifying required packages..."
	foreach package $TCLSETUP(packageList) {
	    if [catch {verifyPackage $package $TCLSETUP($package,path)} err] {
		cleanup $err
	    }
	}
	
	# Update all tclsh interpreter paths
	writeOutput "\n  Updating tclsh interpreter paths..."
	set fileList [glob -nocomplain $wsBinDir/*.cgi $wsNbinDir/*.cgi $wsModuleDir/*/*/*.cgi $wsModuleDir/*/*/*/*.cgi]
	if [catch {updateInterp $fileList $tclInterpPath} err] {
	    cleanup $err
	}

	# Update all expect interpreter paths
	writeOutput "\n  Updating expect interpreter paths..."
	set exFileList [glob -nocomplain $wsModuleDir/generic/fileTrans/*.cgi]
	if [catch {updateInterp $exFileList $expectPath} err] {
	    cleanup $err
	}

	# Update cgiPath and tclBinPath
	writeOutput "\n  Updating Tcl extension paths..."
	set extVarList [list cgiPath tclBinPath]
	if [catch {fileUpdate $binLibList $extVarList} err] {
	    cleanup $err
	}

	# Write to site file
	foreach var $TCLSETUP(varList) {
	    puts $fd "set $var \"[set $var]\""
	}
    }
    
    # WebSubmit variable updates
    if {$updateWs} {
	writeOutput "\n  Updating WebSubmit variables..."
	if [catch {fileUpdate $binLibList $WSSETUP(varList)} err] {
	    cleanup $err
	}

	# Write to site file
	foreach var $WSSETUP(varList) {
	    puts $fd "set $var \"[set $var]\""
	}

	# Fix source commands for websubmit.conf
	writeOutput "\n  Updating source commands..."
	foreach file [glob -nocomplain $wsBinDir/*.cgi $wsNbinDir/*.cgi $wsModuleDir/*/*.cgi $wsModuleDir/*/*/*.cgi] {
	    updateTclSource $file websubmit.conf $wsBinDir
	}
    }
    
    # UNIX command path updates
    if {$updateUnix} {
	writeOutput "\n  Updating UNIX command paths..."
	if [catch {fileUpdate $binLibList $UNIXSETUP(varList)} err] {
	    cleanup $err
	}

	# Write to site file
	foreach var $UNIXSETUP(varList) {
	    puts $fd "set $var \"[set $var]\""
	}
    }

    # HTTP configuration updates
    if {$updateHttp} {
	writeOutput "\n  Updating Web server variables..."
	if [catch {fileUpdate $binLibList $HTTPSETUP(varList)} err] {
	    cleanup $err
	}

	# Write to site file
	foreach var $HTTPSETUP(varList) {
	    puts $fd "set $var \"[set $var]\""
	}
    }
    catch {close $fd}

    # Modify file permissions and ownership if desired
    if {$fixPerms} {
	writeOutput "\n  Modifying file permissions and ownership..."
	fixPerms
    }

    # Activate Database tabs
    activateDbTabs

    writeOutput "\n\nDONE"
    return
}

##########################################################
# Procedure to activate tabs in database manager
##########################################################
proc activateDbTabs {} {
    global TABS dbTabList dbMan
    foreach {dbTab dbName} $dbTabList {
	$dbMan itemconfigure $TABS($dbMan,$dbTab) -state normal
	drawTab $dbTab.db.text
    }
    return
}




#############################################################
# Constructor for a database widget that contains a toolbar
#   with buttons along the top and a scrolled text widget
#   for the database entries.  The database builder fills
#   in the buttons for flexibility.
#############################################################
proc dbManager {name} {
    global debug 

    # Frame for buttons and scrolled text (Database)
    set h [frame $name -relief sunken -bd 1]

    # Insert a scrolled text frame to hold the file being edited
    set t [scrolledtext $h.db -height 20 -relief sunken -borderwidth 1 -autoscrollbar 0]
    pack $t -side bottom -pady 5 -padx 5 -fill both -expand 1

    return $name
}



