# WebSubmit 2.0 webSubmit.tcl @(#)webSubmit.tcl	1.74 SCCS/s.webSubmit.tcl  99/07/12 

# AUTHORS #############################################################

#  Authors: John Koontz, Ryan McCormack, Judith Devaney
#  Original Development Work: Robert Lipman, John Koontz, Katherine
#      Pagoaga

# Copyright Information ###############################################

# This software was developed at the National Institute of Standards
# and Technology by employees of the Federal Government in the course
# of their official duties.  Pursuant to title 17 Section 105 of the
# United States Code this software is not subject to copyright
# protection and is in the public domain.

# CODEINFO BEGIN ######################################################

# info package webSubmit> This package contains the primary routines 
#     that comprise the webSubmit package.  These routines are used to
#     build a user-customizable interface to high-performance computing
#     facilities.

# webSubmit-sclr WebSubmit> Name of the project
# webSubmit-sclr httpHead > HTTP root URL for webSubmit CGI scripts
# webSubmit-sclr localHost > Name of the WebSubmit server machine
# webSubmit-sclr programmerEmailAddrs > Email address of the programmer
# webSubmit-sclr programmerName > Name of the programmer
# webSubmit-sclr debug > Debug variable (ON = 1, OFF = 0)
# webSubmit-sclr execError > String containing CGI-related exec error
# webSubmit-sclr cgiHead > Location of CGI scripts
# webSubmit-sclr platform> Platform under which code executes
# webSubmit-sclr urlProtocol> Specifies secure or insecure links (http vs. https)
# webSubmit-sclr imgHead> URL header for image files

# For each database thisDb in {...} ???
# This might be as informative and more succint than repeating everything
# n times.  JEK

# webSubmit::$thisDb-arr DB > Entries for $thisDb database
# webSubmit::$thisDb-arr DBTXT > Auxiliary text for $thisDb database
# webSubmit::$thisDb-list attrList > List of all attributes in $thisDb, 
#     extracted from the *.db save file.
# webSubmit::$thisDb-sclr dbFile > File containing main database.
# webSubmit::$thisDb-sclr txtFile > File containing database text.
#     Set to "" if there is none.
# webSubmit::$thisDb-sclr stateFile > File containing database initial
#     state information.  Set to "" if there is none.
# webSubmit::$thisDb-list reqAttrList > List of required attributes in 
#     $thisDb, set up in webSubmit.tcl and used to validate the database.
# webSubmit::$thisDb-list dbKeyList > List of key attributes of $thisDb

# CODEINFO END ########################################################

# Package Declarations ################################################
package provide webSubmit 1.0
package require wsUtil

# Namespace Definition and Variable Initialization ####################
namespace eval webSubmit {
    # Procedures to be exported

    # General variable definitions
    variable thisNS [namespace current]
    variable upNS [namespace parent]
    variable wsNS [namespace current]

    # Procedures to be imported
    catch {namespace import ${wsNS}::util::*}

    # System-independent variables
    variable debug 0
    variable execError
    variable WebSubmit "WebSubmit"
    variable wsEnv "WS"
    variable wsID ""
    variable userName ""
    variable userPath ""
    variable userType ""
    variable userEmail ""
    variable VARSAFE() {}
    set VARSAFE(ArrayList) ""
    trace variable VARSAFE wu checkChangeScope
    webSubmit::util::readOnly securityOn 1
    readOnly requireHttps 1

    # PATH VARIABLES
    readOnly wsRootDir "/sasp/WebSubmit/webSubmit2.5"
    readOnly wsLibDir "/sasp/WebSubmit/webSubmit2.5/lib"
    readOnly wsBinDir "$wsRootDir/bin"
    readOnly wsNbinDir "$wsRootDir/nbin"
    readOnly wsModuleDir "$wsRootDir/modules"
    readOnly wsWrapDir "$wsRootDir/wrap"
    readOnly wsUserDir "$wsRootDir/user"
    readOnly wsLogDir "$wsRootDir/logs"
    readOnly wsCertDir "$wsRootDir/cert"
    readOnly wsDocDir "$wsRootDir/doc"
    readOnly wsDlDir "$wsRootDir/doc/download"
    readOnly cgiPath "/sasp/cgi.tcl-0.7"
    readOnly tclBinPath /sasp/tcl/itl/lib

    # Various System-dependent variables
    readOnly adminMailAddr "websubmit@ogma.nist.gov"
    readOnly cgiName "$WebSubmit Master"
    variable platform $tcl_platform(platform)
    readOnly GaussianVersions [list Gaussian94 Gaussian98]

    # Data for standard HTTP server (port 80 - insecure)
    readOnly nCgiRoot "$wsNbinDir"
    readOnly nUrlProtocol "http"
    readOnly nServerName "ogma.nist.gov"
    readOnly nServerBinAlias "websubmit"
    readOnly nServerDocAlias "doc"
    readOnly nServerImgAlias "$nServerDocAlias"
    readOnly nHttpHead "$nUrlProtocol://$nServerName"
    readOnly nCgiHead "$nHttpHead/$nServerBinAlias"
    readOnly nDocHead "$nHttpHead/$nServerDocAlias"
    readOnly nImgHead "$nHttpHead/$nServerImgAlias"
    readOnly certManager "http://webservices.nist.gov/pki"
    readOnly registerCaURL "$nCgiHead/registerCA.cgi"
    readOnly certRequestURL "$nCgiHead/reqClientCert.cgi"
    readOnly nsLoadCert "$nCgiHead/nsLoadClientCert.cgi" 
  
    # Data for Secure HTTP server (port 443)
    readOnly mimeTypes "directory"
    readOnly mimeExtension "ws"
    readOnly cgiRoot "$wsBinDir"
    readOnly urlProtocol "https"
    readOnly serverName "ogma.nist.gov"
    readOnly serverHost "ogma.nist.gov"
    readOnly serverUser "nobody"
    readOnly serverBinAlias "websubmit"
    readOnly serverDocAlias "$nServerDocAlias"
    readOnly serverImgAlias "$nServerImgAlias"
    readOnly serverJsAlias "$serverDocAlias"
    readOnly serverDlAlias "$serverDocAlias/download"
    readOnly httpHead "$urlProtocol://$serverName"
    readOnly cgiHead "$httpHead/$serverBinAlias"
    readOnly moduleHead "$cgiHead/modules"
    readOnly imgHead "$httpHead/$serverImgAlias"
    readOnly docHead "$httpHead/$serverDocAlias"
    readOnly dlHead "$httpHead/$serverDlAlias"
    readOnly javaScriptHead "$httpHead/$serverJsAlias"

    # Safe interpreter specifications
    readOnly safeInterp [interp create -safe]
    $safeInterp alias safeSet safeSet
    $safeInterp alias safeArrSet safeArrSet
    interp invokehidden $safeInterp source [file join $wsLibDir safeInterp.tcl]

    # Programmer-dependent variables
    variable programmerName "$WebSubmit Programming Team"
    variable programmerEmailAddr $adminMailAddr

    # Paths to WebSubmit Server UNIX Commands
    ;;readOnly unixScp "/usr/bin/scp"
    ;;readOnly unixSsh "/usr/bin/ssh"
    ;;readOnly unixFile "/usr/bin/file"
    ;;readOnly unixChmod "/bin/chmod"
    ;;readOnly unixNslookup "/usr/sbin/nslookup"
}

# Modify auto_path to get CGI and TclBin packages
lappend auto_path $::webSubmit::cgiPath $::webSubmit::tclBinPath

##### Tcl Packages ######
package require cgi
package require Bin
package require Mdfive

##### WebSubmit packages #####
package require wsDatabase
package require wsAuth
package require wsNetwork
package require wsMaster
package require wsForms
package require wsFormElements
package require wsCgi
package require wsLoadLeveler
package require wsNQS
package require wsLSF


################
# Initialization

# HTML configuration
set masterBgcolor "antiquewhite"

# Import several namespaces into global
set wsNS webSubmit
namespace import ${wsNS}::util::*
namespace import ${wsNS}::cgi::*

# DEBUGGING: Load test setup script to set various environment and
#  debugging options.  This is to allow execution from a non-CGI
#  environment.
if ![isUser nobody] {
    # Script being run from command-line
    if [catch {source [file join $::webSubmit::wsBinDir testSetup.conf]} err] {
	puts "Error sourcing debug script...$err"
	exit
    }
}

# Make the global environment read-only
foreach var [array names env] {
    readOnlyArray env $var $env($var)
}

























