# WebSubmit 2.0 wsCgi.tcl name @(#)wsCgi.tcl	1.28 SCCS/s.wsCgi.tcl  99/06/17 


# CODEINFO END ########################################################
 

# AUTHORS #############################################################

#  Authors: John Koontz, Ryan McCormack, Judith Devaney
#  Original Development Work: Robert Lipman, John Koontz, Katherine
#      Pagoaga

# Copyright Information ###############################################
 
# This software was developed at the National Institute of Standards
# a555nd Technology by employees of the Federal Government in the course
# of their official duties.  Pursuant to title 17 Section 105 of the
# United States Code this software is not subject to copyright
# protection and is in the public domain.

# CODEINFO BEGIN ######################################################

# info package wsCgi> This package contains a collection of CGI routines
#     that are specific to WebSubmit.  In some cases, these routines
#     extend the functionality of cgi.tcl; in other cases, they are
#     completely new routines

# CODEINFO END ########################################################

package provide wsCgi 1.0

namespace eval webSubmit::cgi {
    # Namespace variables
    variable thisNS [namespace current]
    variable wsNS [namespace parent]
    variable upNS [namespace parent]

    # Procedures to be exported
    namespace export cgiAdminMail
    namespace export cgiBomb
    namespace export cgiWarn
    namespace export safeImport
    namespace export authHeader
    namespace export sessionManager
    namespace export cgiSetup
    namespace export cgiImport
    namespace export wsHelpButton
    namespace export inputVerify
    namespace export helpSetup
    namespace export alert
    namespace export queryString

    # Procedures to be imported
    catch {namespace import ${wsNS}::util::*}
    catch {namespace import ${wsNS}::auth::*}
    alias $wsNS unixNslookup
    variable nslookup $unixNslookup

    # Variables
    variable debug 0
    variable restrVars [list [info vars ${wsNS}::*]]
}

# CODEINFO BEGIN ######################################################

# info proc cgiAdminMail> Procedure to mail a message to the WebSubmit
#     Administrator.
# arg proc> Name of procedure or context in which proc was invoked. 
# arg subject> Subject line of message.
# arg message> Message to send to the WebSubmit Administrator.
# return> None

proc webSubmit::cgi::cgiAdminMail {proc message} {
    variable wsNS
    alias $wsNS adminMailAddr userName

    cgi_mail_start $adminMailAddr
    cgi_mail_add "Subject:  Problem in $proc"
    cgi_mail_add
    cgi_mail_add "$message"
    cgi_mail_add
    cgi_mail_add "User $userName was affected."
    cgi_mail_end
}


# CODEINFO BEGIN ######################################################

# info proc cgiBomb> Procedure to handle bombs gracefully and send output 
#     through CGI

# arg proc> Name of procedure in which bomb occurred
# arg string> String to return in bomb error message
# return> None

proc webSubmit::cgi::cgiBomb {proc string} {
    # Log Error Message
    global remUser remHost
    foreach var [list remUser remHost] {
	if ![info exists $var] {
	    set $var ""
	}
    }

    if {$remUser != "" && $remHost != ""} {
	set info "$remUser@$remHost"
    } else {
	set info $webSubmit::auth::wsID
    }
    regsub -all {<[bB][rR]>} $string "\n  " newString
    regsub -all {<[^>]+>} $newString "" newString
    regsub "\[ \t\n\]+\$" $newString "" newString
    set separator "--------------------------------------------------------------------------"
    set errMsg "ERROR - $proc ($info): $newString \n$separator"
    writeLog [file join $webSubmit::wsLogDir errors] $errMsg

    # HTML Error Message
    cgi_http_head
    cgi_html {
	cgi_body vlink=rosybrown bgcolor=antiquewhite {
	    cgi_br
	    cgi_h3 "[cgi_bold [cgi_font color=firebrick "WebSubmit ERROR (Procedure: $proc)"]]"
	    cgi_hr
	    cgi_p "$string"
	}
    }


    exit
}

# CODEINFO BEGIN ######################################################
# info proc cgiWarn> Procedure to generate warning messages
# arg proc> Name of procedure in which warning was issued
# arg string> String to return in warning message
# return> None

proc webSubmit::cgi::cgiWarn {proc string} {
    cgi_hr
    cgi_h3 "[cgi_bold [cgi_font color=blue "WebSubmit WARNING (Procedure: $proc)"]]"
    cgi_p "[cgi_nbspace][cgi_nbspace][cgi_bold $string]"
    cgi_hr
}


# CODEINFO BEGIN ######################################################

# info proc safeImport> Procedure to prevent manipulation of variables
#     outside the allowed bounds (for example, environment variables).
#     The procedure constructs a list of valid variables and then
#     imports them into the calling scope.

# arg reqVarList> List of variables that *must* be in import list
# return> List of valid imported variables

proc webSubmit::cgi::safeImport {{reqVarList ""}} {
    variable wsNS
    variable restrVars

    # Construct list of valid imported variables
    set importList ""
    set unImportList ""
    if ![catch {uplevel cgi_import_list} importableList] {
	set allSafe 1
	foreach arg $importableList {
	    set thisSafe 1
	    # Disallow modification of webSubmit variables
	    foreach var $restrVars {

		# Note:  Changed *$arg* to $arg, as the former formulation
		# precludes all CGI variables that are any substring of
		# a variable in the webSubmit namespace, and I cannot think
		# of a way in which that would be dangerous without help
		# from our code.  JEK

		if [string match $arg $var] {
		    set thisSafe 0
		    lappend unImportList $arg
		    break 
		}
	    }
	    
	    # Disallow modification of various other variables
	    #  env - Environment variables
	    #  :: - Namespace names
	    switch -glob -- $arg {

		env(*) {
		    # Environment variables
		    set thisSafe 0
		    lappend unImportList $arg
		}
 
		*clientDN* {
		    set thisSafe 0
		    lappend unImportList $arg
		}
		*issuerDN* {
		    set thisSafe 0
		    lappend unImportList $arg
		}

		*::* {
		    # Namespace variables
		    set thisSafe 0
		    lappend unImportList $arg
		}

		default {
		    if {$thisSafe} {
			lappend importList $arg
		    }
		}
	    }
	    if {!$thisSafe} {
		set allSafe 0
	    }
	}


	# Import edited list of variables, after verifying that all required
	# variables are in the edited list, and no unsafe variables are being
	# attempted

	if {!$allSafe} {
	    # Report attempt to import unsafe variables
	    cgiBomb safeImport "Variables supplied for import ($importList)\
                  include some unsafe ones ($unImportList)."
	} elseif {($reqVarList == "") || [reqList $importList $reqVarList]} { 
	    # If no required variables missing from importList, import 
	    # variables
	    foreach var $importList {
		if [catch {uplevel "cgi_import $var"}] {
		    uplevel "set $var \"\""
		}
	    }
	} else {
	    # Import list does not contain required elements
	    cgiBomb safeImport "Variables imported from form ($importList) do\
                  not match those required ($reqVarList)."

	}
    }
    return $importList
}

# CODEINFO BEGIN ######################################################

# info proc authHeader> Procedure to generate authorization header
# arg > None
# return> None

proc webSubmit::cgi::authHeader {} {
    # List user and ws ID
    cgi_h5 "[cgi_font color=darkmagenta "WebSubmit User ID:"] $::webSubmit::wsID [cgi_nl]\
                    [cgi_font color=darkmagenta "WebSubmit User Name:"] $::webSubmit::userName[cgi_nl]\
                    [cgi_font color=darkmagenta "Certificate Issuer:"] $::webSubmit::auth::issuerDN(CN)"
    cgi_br
}


# CODEINFO BEGIN ######################################################

# info proc sessionManager> Procedure to place session manager data
#    into a module or form

# arg moduleName> Name of current module (used for namedSessList)
# arg hostName> Name of current host 
# return> None
# loc-sclr loadedSessName> Name of session currently loaded
# loc-sclr namedSessList> List of named sessions for this user

proc webSubmit::cgi::sessionManager {moduleName hostName} {
    # Get list of user named sessions and name of current session
    set namedSessList [webSubmit::forms::getSessionNames $moduleName $hostName]

    # Aliases
    alias webSubmit::forms::session loadedSessName systemDefName
    
    # Set loaded session name if it exists in the calling scope
    if [uplevel "info exists loadedSessName"] {
	upvar loadedSessName importedSessName
	if {$importedSessName != ""} {
	    set loadedSessName $importedSessName
	}
    }

    # Basic form elements
#     cgi_export moduleName=$moduleName
#     cgi_export hostName=$hostName
    cgi_table border=2 cellpadding=4 {
	cgi_table_row {
	    cgi_table_data bgcolor=darkslateblue colspan=2 {
		puts "[cgi_font size=+1 color=white [cgi_bold "Session Manager"]]\
                       [cgi_nbspace][cgi_nbspace][cgi_nbspace][cgi_nbspace][cgi_nbspace][cgi_nbspace]\
                       [cgi_font color=white "Currently Loaded Session:"]\
                       [cgi_font color=yellow "$loadedSessName"]"
	    }
	}
	cgi_table_row {
	    cgi_table_data nowrap {
		puts "[cgi_font color=darkmagenta [cgi_bold "User Session Library:"]]"
		cgi_select sessName {
		    if {$namedSessList == ""} {
			cgi_option "None"
		    } else {
			foreach sess $namedSessList {
			    cgi_option $sess selected_if_equal=$loadedSessName
			}
		    }
		}
		puts "[cgi_nbspace][cgi_nbspace][cgi_nbspace][cgi_nbspace]"
	    }

	    cgi_table_data nowrap {
		puts "[cgi_font color=darkmagenta [cgi_bold "New Session Name:"]]"
		if {($loadedSessName != "$systemDefName") && ($loadedSessName != "None")} {
		    set newSessName $loadedSessName
		} else {
		    set newSessName ""
		}
		cgi_text newSessName=$newSessName size=20 maxlength=40
		puts "[cgi_nbspace][cgi_nbspace]"
		cgi_submit_button "sessionAction=Save"
	    }
	}

	cgi_table_row {
	    cgi_table_data nowrap colspan=2 {
		puts "[cgi_font color=darkmagenta [cgi_bold "Action:"]]"
		cgi_submit_button "sessionAction=Load Library Session"
		puts "[cgi_nbspace][cgi_nbspace]"
		cgi_submit_button "sessionAction=Load System Defaults"
		puts "[cgi_nbspace][cgi_nbspace]"
		cgi_submit_button "sessionAction=Delete Selected Session"
		puts "[cgi_nbspace][cgi_nbspace][cgi_nbspace][cgi_nbspace]"
		cgi_submit_button "sessionAction=Save As Default"
	    }
	}
    }
} 


# CODEINFO BEGIN ######################################################

# info proc cgiSetup> Procedure to import a group of variables and get
#     session data for a module

# This routine has been replaced by cgiImport and should be deleted when
# the conversion process is complete.  JEK

# arg moduleName> Name of the module to process
# arg moduleVarList> List of variables in the module
# arg reqVarList> List of variables that *must* be in import list
# return>

proc webSubmit::cgi::cgiSetup {moduleName moduleVarList {reqVarList ""}} {
    # Get cgi input
    cgi_input

    # Initialize module variables
    uplevel {initVars $moduleVarList}

    # Import variables
    if {$reqVarList == ""} {
	set importVarList [uplevel safeImport]
    } else {
	set importVarList [uplevel "safeImport [list $reqVarList]"]
    }

    # Retrieve session data if necessary
    if {$importVarList == ""} {
	session load "$moduleName"
	uplevel getSessionData
    }

    return
}

# CODEINFO BEGIN ######################################################

# info proc cgiImport> Procedure to import a group of variables and insure
#      that a required subset are present.

# arg moduleVarList> List of variables in the module
# arg reqVarList> List of variables that *must* be in import list
# return>

proc webSubmit::cgi::cgiImport {moduleVarList {reqVarList ""}} {
    # Get cgi input
    cgi_input

# Debugging strings that can be appended to cgi_input.
# It only reads them if run at the command line.

# "hostName=danube.nist.gov&title=foo&class=Small&job_type=serial&min_processors=1&max_processors=1"
# "mode=BASIC&moduleName=llJob&hostName=danube.nist.gov&bqs=LLS&title=foo"

    # Initialize module variables
    uplevel "initVars [list $moduleVarList]"

    # Import variables
    if {$reqVarList == ""} {
	set importVarList [uplevel safeImport]
    } else {
	set importVarList [uplevel "safeImport [list $reqVarList]"]
    }

    return $importVarList
}

# CODEINFO BEGIN ######################################################
# info proc wsHelpButton> Procedure to generate a Help button 
# arg helpURL> URL for relevant help document
# return> CGI code to generate link button

proc webSubmit::cgi::wsHelpButton {helpURL} {
    set helpGif "${webSubmit::imgHead}/button_help.gif"
    cgi_imglink help $helpGif alt=Help
    return "[cgi_url [cgi_imglink help] $helpURL target=help]"
}

# CODEINFO BEGIN ######################################################

# info proc queryString> Procedure to construct a &-joined query string for 
#   inclusion in a URL.

# arg args> an alternating list of export variables names and values to be 
#     assigned to them
# return> the query string

proc webSubmit::cgi::queryString {args} {
    variable debug

    if {[expr [llength $args] % 2] != 0} {
	return -code error "In queryString, list of variables and their values is incomplete.  It has an uneven number of elements."
    }

    set equationList ""
    foreach {var value} $args {
	if {"$value" != ""} {
	    set equation [cgi_cgi_set $var $value]
	    lappend equationList $equation
	}
    }
    set qs [join $equationList "&"]
    
    if {$debug} {
	puts "queryString is $qs"
    }

    return $qs
}



# CODEINFO BEGIN #######################################################
# info proc inputVerify> Procedure to verify CGI input based on a series
#     of criteria.  Each variable is tested for one or more criteria,
#     and a list of error messages is generated if any variables fail.
#     Data for variables and the tests to be performed are stored in an
#     input array with the following structure
#
#       INPUT(varList)           : List of variables to be checked
#       INPUT($var)              : List of tests to be performed for variable $var
#       INPUT($var,$test,$arg1)  : Argument $arg1 to test $test for variable $var
#         .
#         .
#         .
#       INPUT($var,$test,$argn)  : Argument $argn to test $test for variable $var

# arg arrName> Name of the array containing data about variables and tests
# return> None (bombs if any variables do not meet requirements)

proc webSubmit::cgi::inputVerify {arrName} {
    upvar $arrName INPUT
    variable nslookup
    set ERROR() {}

    # Loop over input variable list
    foreach var $INPUT(varList) {
	if [info exists INPUT($var)] {
	    # Loop over all tests for the selected variable
	    set value [uplevel "set $var"]
	    if {$value != ""} {
		set VALS($var) $value
		foreach test $INPUT($var) {

		    # STRING TESTS
		    if {$test == "noWhite"} {
			# Test for white space 
			set MSG(noWhite) "String should not contain internal white space"
			if [regexp "\[ \t\]" $value] {
			    set ERROR($var,noWhite) 1
			}
		    }  elseif {$test == "regexpMatch"} {
			# Test for match with regular expression
			#  ARGUMENTS: pattern
			set MSG(regexpMatch) "String does not match required pattern"
			if [info exists INPUT($var,regexpMatch,pattern)] {
			    set cmd [list regexp $INPUT($var,regexpMatch,pattern)]
			    if ![eval $cmd \"$value\"] {
				set ERROR($var,regexpMatch) 1
			    }
			}
		    } elseif {$test == "hasChars"} {
			# Variable contains alphanumeric characters
			set MSG(hasChars) "Variable must contain alphanumeric characters"
			if ![regexp {[0-9a-zA-Z]+} $value] {
			    set ERROR($var,hasChars) 1
			}
		    } elseif {$test == "isInteger"} {
			# Variable is an integer
			set MSG(isInteger) "Variable should be an integer"
			if ![regexp {^(-|\+)?[0-9]+$} $value] {
			    set ERROR($var,isInteger) 1
			}
		    } elseif {$test == "isPositiveInt"} {
			# Variable is a positive integer
			set MSG(isPositiveInt) "Variable should be a positive integer"
			if ![regexp {^\+?[1-9][0-9]*$} $value] {
			    set ERROR($var,isPositiveInt) 1
			}
		    } elseif {$test == "isNegativeInt"} {
			# Variable is a negative integer
			set MSG(isNegativeInt) "Variable should be a negative integer"
			if ![regexp {^-[0-9]+$} $value] {
			    set ERROR($var,isNegativeInt) 1
			}
		    } elseif {$test == "isDecimal"} {
			# Variable is a decimal number
			set MSG(isDecimal) "Variable should be a decimal number"
			if ![regexp {^(-|\+)?[0-9]*\.[0-9]*$} $value] {
			    set ERROR($var,isDecimal) 1
			}
		    } elseif {$test == "isValidHost"} {
			# Determine if remote host is a valid Internet host
			set MSG(isValidHost) "Host is invalid"
			if ![catch {exec $nslookup $value} result] {
			    if ![regexp ".*Name: *$value.*\nAddress\[es\]*: *\[0-9\]+\.\[0-9\]+\.\[0-9\]+\.\[0-9\]+.*" $result] {
				set ERROR($var,isValidHost) 1
			    }
			} else {
			    set ERROR($var,isValidHost) 1
			}
		    }

		    # Tests involving remote hosts
		    if [info exists INPUT($var,$test,remHost)] {
			set remHost $INPUT($var,$test,remHost)
			if {$test == "remFileExist"} {
			    # Determine if remote file or directory exists
			    #  ARGUMENT: remUser
			    set MSG(remFileExist) "Specified remote file does not exist"
			    if [info exists INPUT($var,$test,remUser)] {
				set remUser $INPUT($var,$test,remUser)
				if ![webSubmit::network::remFileExist $remUser $remHost $value] {
				    set ERROR($var,remFileExist) 1
				}
			    }
			} elseif {$test == "isFile"} {
			    # Determine if specified remote file is a regular file
			    #  ARGUMENT: remUser
			    set MSG(isFile) "Specified remote file is not a regular file"
			    if [info exists INPUT($var,$test,remUser)] {
				set remUser $INPUT($var,$test,remUser)
				set type [webSubmit::network::remFileType $remUser $remHost $value]

				if {$type != "file" && $type != "text"} {
				    set ERROR($var,isFile) 1
				}
			    }
			} elseif {$test == "isDirectory"} {
			    # Determine if specified remote file is a directory
			    #  ARGUMENT: remUser
			    set MSG(isDirectory) "Specified remote file is not a directory"
			    if [info exists INPUT($var,$test,remUser)] {
				set remUser $INPUT($var,$test,remUser)
				set type [webSubmit::network::remFileType $remUser $remHost $value]
				if {$type != "directory"} {
				    set ERROR($var,isDirectory) 1
				}
			    }	
			} elseif {$test == "isExecutable"} {
			    # Determine if specified remote file is executable
			    #  ARGUMENT: remUser
			    set MSG(isExecutable) "Specified remote file is not an executable or shell script"
			    if [info exists INPUT($var,$test,remUser)] {
				set remUser $INPUT($var,$test,remUser)
				set type [webSubmit::network::remFileType $remUser $remHost $value]

				if {$type != "executable" && $type != "script"} {
				    set ERROR($var,isExecutable) 1
				}
			    }	
			}
		    }
		}
	    }
	}
    }

    # Determine error output string if any
    set errorOutput ""
    foreach var $INPUT(varList) {
	foreach err [array names ERROR $var,*] {
	    regexp {.*,([^,]+)$} $err match errType
	    append errorOutput "<B>Variable</B>: <TT>$var</TT><BR><B>Value</B>: \"<TT>$VALS($var)</TT>\"\
		    <BR><B>Error</B>: <B><FONT COLOR=blue>$MSG($errType)</FONT></B><BR><BR>"
	}
    }
    
    # Generate CGI output related to error information
    if {$errorOutput != ""} {
	# Errors were generated for some input variables
	cgiBomb inputVerify "The following errors were generated during processing of\
		CGI input:<BR><BR>$errorOutput"
    }
    return 0
}

# CODEINFO BEGIN ##############################################################
# info proc helpSetup> Procedure to setup help links and javascript commands
#     for a simple help facility.  Links are defined in terms of form element
#     names, and are meant to be used as the descriptive headers in the row
#     corresponding to the form elements.  Each link executes javascript
#     code to display help text in a specified target frame or window.

# arg helpArrName> Name of array containing help information (headers + text)
# return> None

proc webSubmit::cgi::helpSetup {helpArrName} {
    upvar $helpArrName HELP
    variable wsNS
    alias $wsNS javaScriptHead

    set varNameList ""
    if [info exists HELP(elementList)] {
	# Load javascript functions and classes
	cgi_javascript SRC="$javaScriptHead/FormHelp.js" {}
	cgi_javascript SRC="$javaScriptHead/showHelp.js" {}
	
	# Javascript for typing help text into footer frame
	cgi_javascript {
	    foreach elt $HELP(elementList) {
		if ![info exists HELP($elt,header)] {
		    set HELP($elt,header) "id: $elt"
		}
		
		if ![info exists HELP($elt,text)] {
		    set HELP($elt,text) "No help available..."
		}

		# Clean up element name so that it is a valid identifier in Javascript
		#  Make sure we don't create any duplicates
		set varName "_$elt"
		regsub -all {[^a-zA-Z0-9_]} $varName "_" varName
		if {[lsearch -exact $varNameList $varName] != -1} {
		    set varName "_$varName"
		}
		lappend varNameList $varName
		set ELTMAP($elt) $varName

		puts "// Help setup for element $elt ($varName)"
		puts "var $varName = new FormHelp(\"$HELP($elt,header)\",\"$HELP($elt,text)\");"
		puts ""
	    }
	}
	puts ""
	
	# Set up link data
	foreach elt $HELP(elementList) {
	    cgi_link $elt $HELP($elt,header) javascript:showHelp($ELTMAP($elt)) "onMouseOver=status='Show help';return false;"
	}
    } else {
	return -code error "Could not set up help links - no element list provided"
    }
}

# CODEINFO BEGIN ###########################################
# info proc alert> Procedure to raise an alert window with
#     a small message

proc webSubmit::cgi::alert {msg} {
    cgi_javascript {
	puts "alert(\"$msg\");"
    }
    return
}
