# WebSubmit 2.0 wsFormElements.tcl @(#)wsFormElements.tcl	1.16 SCCS/s.wsFormElements.tcl  99/07/12 
 
# AUTHORS #############################################################
# WebSubmit 2.0:
#     Authors: John Koontz, Ryan McCormack, Judith Devaney
#     Original Development Work: Robert Lipman, John Koontz, Katherine
#         Pagoaga

# CODEINFO BEGIN ######################################################
# info>  WebSubmit script that controls construction of forms,
#     based on moduleName and formMode, supplied via query string,
#     or environment variable, set by the launching script 
#     (master.cgi). 

# CODEINFO END ########################################################
  

# AUTHORS #############################################################

# WebSubmit 2.0:
#  Authors: John Koontz, Ryan McCormack, Judith Devaney
#  Original Development Work: Robert Lipman, John Koontz, Katherine
#      Pagoaga

# Copyright Information ###############################################

#   This software was developed at the National Institute of Standards
#   and Technology by employees of the Federal Government in the course
#   of their official duties.  Pursuant to title 17 Section 105 of the
#   United States Code this software is not subject to copyright
#   protection and is in the public domain.

# CODEINFO BEGIN ######################################################

# info package wsFormElements> The elements that comprise a WebSubmit 
#     page.
 
# descriptions???

# CODEINFO END ########################################################

# Package Declarations ################################################
package provide wsFormElements 1.0

# Namespace Definition ################################################
namespace eval webSubmit::formElements {
    # Namespace definitions
    variable thisNS [namespace current]
    variable upNS [namespace parent]
    variable wsNS [namespace parent]
    variable formsNS ${wsNS}::forms

    # Namespace imports
    catch {namespace import ${wsNS}::util::*}
    catch {namespace import ${wsNS}::database::*}

    cgi_imglink helpButton "http://www.nist.gov/images/help-button.gif" \
        alt="Help Button"

    namespace export feActionButton
    namespace export feBlankCells
    namespace export feBlankMargin
    namespace export feComment
    namespace export feCommentList
    namespace export feColumnizedSeparator
    namespace export feGaussianVersions
    namespace export feHelpButton
    namespace export feInstructions
    namespace export feLLJobClassSelector
    namespace export feMarginLabel
    namespace export fePrompt
    namespace export fePulldownLabelList 
    namespace export fePulldownNumberList
    namespace export feSeparator
    namespace export feSubtitleBar
    namespace export feTextEntry
    namespace export feTitleBar
    namespace export feWebSubMake

    # Variable definitions
    variable debug 0
    alias $formsNS marginBgColor marginFgColor 
    alias $formsNS bodyBgColor bodyFgColor
    alias $formsNS titleBgColor titleFgColor
    alias ${wsNS}::database paramConcatenator


    if {$debug} {
	puts "TRACE(wsFormElements.tcl):  Creating ::webSubmit::formElements"
    }
}

# Procedure Definitions ###############################################

# CODEINFO BEGIN ######################################################
# info proc feGaussianVersions> Generate a pulldown list of Gaussian
#     package versions.

# arg-sclr instanceName> A unique name associated with this instance
#     of (call to) the proc
# arg-list optionalList> The list of options for the pulldown list.
# arg-sclr option> The initial index in the optionList.  The user
#     adjusts this with the pulldown widget.
# return> Nothing.

proc webSubmit::formElements::feGaussianVersions {instanceName
    optionList {option 0}} {
    variable wsNS
    alias ${wsNS}::database paramConcatenator

    set optionName [joinKey [list $instanceName option] $paramConcatenator]

    set minOptionIndex 0
    set maxOptionIndex [expr [llength $optionList] - 1]
    if {[expr $option < $minOptionIndex] || [expr $option > $maxOptionIndex]} {
        bomb webSubmit::formElements::feGaussianVersions\
            "Pulldown option index out of range."
    }
    cgi_select $optionName {
        for {set optionIndex 0}\
            {$optionIndex <= [expr [llength $optionList] - 1]}\
            {incr optionIndex} {
                cgi_option [lindex $optionList $optionIndex]\
                    value=$optionIndex\
                    selected_if_equal=$option
        }
    }
}


# CODEINFO BEGIN ######################################################

# info proc feLLJobClassSelector> Give a list of the LL job classes,
#     accompanied by a radiobutton selector and pulldown selectors
#     for the number of processors.

# arg-sclr instanceName> A unique name associated with this instance
#     of (call to) the proc
# arg-sclr class> The selected class's name.  
# arg-list minProc> Existing min_processor value
# arg-list maxProc> Existing max_processor value
# arg-sclr colSpan> The number of cells in the form table to span
#     with this form element.
# return> Nothing.

proc webSubmit::formElements::feLLJobClassSelector {instanceName \
    {class 0} {minProc 0} {maxProc 0} {colSpan 1}} {
    variable wsNS
    alias ${wsNS}::loadleveler CLASS classNameList

    # Provide default class selection
    if {$class == 0} {
	set class [lindex $classNameList 0]
    }
    set selClassName [joinKey [list $instanceName class]]
    set minProcName [joinKey [list $instanceName minList]]
    set maxProcName [joinKey [list $instanceName maxList]]

    cgi_table_data nowrap colspan=$colSpan valign=top {
	# Issue data about classes, accompanied by radiobuttons.

	cgi_table width=100% cellpadding=0 cellspacing=0 {
	    cgi_table_row {
		cgi_table_head rowspan=2 {
		    put [cgi_font size=-1 color=darkmagenta "SELECT"]
		}
		cgi_table_head rowspan=2 {
		    put [cgi_font size=-1 "Class"]
		}
		cgi_table_head colspan=2 {
		    put [cgi_font size=-1 "Processors"]
		}
		cgi_table_head rowspan=2 {
		    put [cgi_font size=-1 "Priority"]
		}
		cgi_table_head {
		    put [cgi_font size=-1 "Wall-Clock Limit"]
		}
	    }
	    cgi_table_row {
		cgi_table_head {
		    put [cgi_font size=-1 "Min"]
		}
		cgi_table_head {
		    put [cgi_font size=-1 "Max"]
		}
		cgi_table_head {
		    put [cgi_font size=-1 "(hh:mm:ss)"]
		}
	    }

	    foreach name $classNameList {
		cgi_table_row {
		    cgi_table_data nowrap align=center {
			cgi_radio_button $selClassName=$name\
			    checked_if_equal=$class
		    }
		    cgi_table_data align=center {
			put $name
		    }
		    cgi_table_data align=center {
			if {"$name" == "$class"} {
			    set localMinProc $minProc
			} else {
			    set localMinProc 1
			}
			
			cgi_select $minProcName {
			    for {set i 1} {$i <= $CLASS($name,max_processors)} {incr i} {
				cgi_option $i value=$i selected_if_equal=localMinProc
			    }
			}
		    }
		    
		    cgi_table_data align=center {
			if {"$name" == "$class"} {
			    set localMaxProc $maxProc
			} else {
			    set localMaxProc $localMinProc
			}
			
			cgi_select $maxProcName {
			    for {set i 1} {$i <= $CLASS($name,max_processors)} {incr i} {
				cgi_option $i value=$i selected_if_equal=localMaxProc
			    }
			}
		    }

		    cgi_table_data align=center {
			put $CLASS($name,priority)
		    }
		    regexp {([^,]+).*} $CLASS($name,wall_clock_limit) match limit
		    cgi_table_data align=center {
			put $limit
		    }
		}
	    }
	}
    }
}

