#!/itl/links/generic/bin/tclsh8.0
# WebSubmit 2.0 wsLSF.tcl  @(#)wsLSF.tcl	1.13 SCCS/s.wsLSF.tcl  99/05/19  

# AUTHORS #############################################################

#  Authors: John Koontz, Ryan McCormack, Judith Devaney
#  Original Development Work: Robert Lipman, John Koontz, Katherine
#      Pagoaga

# Copyright Information ###############################################

# This software was developed at the National Institute of Standards
# and Technology by employees of the Federal Government in the course
# of their official duties.  Pursuant to title 17 Section 105 of the
# United States Code this software is not subject to copyright
# protection and is in the public domain.

# CODEINFO BEGIN ######################################################

# info script wsLSF.tcl> Support utilities for LSF batch processing.

# CODEINFO END ########################################################

# Package definitions
package provide wsLSF 1.0

namespace eval webSubmit::lsf {
    # Namespace variables
    variable thisNS [namespace current]
    variable wsNS [namespace parent]
    variable upNS [namespace parent]
    variable debug 0

    # Procedures to be imported
    namespace import ${wsNS}::network::*
    namespace import ${wsNS}::database::*
    namespace import ${wsNS}::util::*

    # Procedures to be exported
    namespace export submitLSFMPIjob
    namespace export submitLSFjob
    namespace export getQueueInfo
    namespace export getBjobsInfo
    namespace export checkLimits
    namespace export submitG94Job

    # Variable aliases
    alias $wsNS wsRootDir

    # Host-specific path information
    variable PATHS() {}
    ;;set rapidBinDir /usr/local/lsf/bin
    ;;set PATHS(rapid.nist.gov,bsub) [file join "$rapidBinDir" bsub]
    ;;set PATHS(rapid.nist.gov,bqueues) [file join "$rapidBinDir" bqueues]
    ;;set PATHS(rapid.nist.gov,bjobs) [file join "$rapidBinDir" bjobs]
    ;;set PATHS(rapid.nist.gov,bkill) [file join "$rapidBinDir" bkill]
    ;;set PATHS(rapid.nist.gov,bpeek) [file join "$rapidBinDir" bpeek]
    ;;set PATHS(rapid.nist.gov,bstop) [file join "$rapidBinDir" bstop]
    ;;set PATHS(rapid.nist.gov,bresume) [file join "$rapidBinDir" bresume]
    ;;set PATHS(rapid.nist.gov,jmsum) "/usr/local/bin/jmsum"
    ;;set PATHS(rapid.nist.gov,cleanall) "/usr/local/bin/CleanAll"
    ;;set PATHS(rapid.nist.gov,cleansome) "/usr/local/bin/CleanSome"
    ;;set PATHS(rapid.nist.gov,mkall) "/usr/local/bin/MkTmpUsrAll"
    ;;set PATHS(rapid.nist.gov,cleanhost) "/usr/local/bin/CleanHost"
    ;;set PATHS(rapid.nist.gov,showall) "/usr/local/bin/ShowAll"

    # Host-specific configuration information
    variable CONFIG() {}
    ;;set CONFIG(hostList) [list rapid.nist.gov]
    ;;set CONFIG(rapid.nist.gov,maxProcessors) 16

    # General BSUB information
    variable bsubOptList [list q m n R J b t i o e u f E c W F M D S C k w L P G]
    variable bsubFlagList [list x r N B I Ip Is K]
    foreach flag $bsubFlagList {
	set BSUBFLAGS(-$flag) "off"
    }
    foreach opt $bsubOptList {
	set BSUBOPTS(-$opt) ""
    }
    variable shell "/bin/csh"
    variable workDir ""
    variable jobScript ""
    variable jobFileName ""
    variable inputFileList ""
    variable outputFileList ""
    variable workDir ""
    variable SchemaName ""
    variable SchemaPathName ""
    variable SchemaContents ""
    variable nodeString ""
    variable nodeList ""
    variable ExecutableName ""
    variable executableFileString ""

    # Queue information
    variable BQUEUES() {}
    variable queueList
    variable queueHeaderList
    set BQUEUES(QUEUE_NAME,text) "Queue"
    set BQUEUES(STATUS,text) "Status"
    set BQUEUES(PRIO,text) "Priority"
    set BQUEUES(MAX,text) "Max CPUs"
    set BQUEUES(CPULIMIT,text) "CPU Time Limit"
    set BQUEUES(RUN,text) "Running Jobs"
    set BQUEUES(PEND,text) "Pending Jobs"
    set BQUEUES(SUSP,text) "Suspended Jobs"
    set BQUEUES(JL/U,text) "User Job Slots"
    set BQUEUES(JL/P,text) "Processor Job Slots"    
    set BQUEUES(JL/H,text) "Host Job Slots"
    set BQUEUES(NJOBS,text) "Job Slots Used"
    set BQUEUES(QUEUE_NAME,show) 1
    set BQUEUES(STATUS,show) 1
    set BQUEUES(PRIO,show) 0
    set BQUEUES(MAX,show) 1
    set BQUEUES(CPULIMIT,show) 1
    set BQUEUES(RUN,show) 0
    set BQUEUES(PEND,show) 0
    set BQUEUES(SUSP,show) 0
    set BQUEUES(JL/U,show) 0
    set BQUEUES(JL/P,show) 0
    set BQUEUES(JL/H,show) 0
    set BQUEUES(NJOBS,show) 0

    # Job Information
    variable BJOBS() {}
    variable BJOBSINFO() {}
    variable jobHeaderList [list JOBID USER STAT QUEUE FROM_HOST EXEC_HOST JOB_NAME SUBMIT_TIME]
    set BJOBSINFO(JOBID,text) "Job ID"
    set BJOBSINFO(USER,text) "User"
    set BJOBSINFO(STAT,text) "Status"
    set BJOBSINFO(QUEUE,text) "Queue"
    set BJOBSINFO(FROM_HOST,text) "Submission Host"
    set BJOBSINFO(EXEC_HOST,text) "Host(s)"
    set BJOBSINFO(JOB_NAME,text) "Job Name"
    set BJOBSINFO(SUBMIT_TIME,text) "Submitted"
    set BJOBSINFO(JOBID,show) 1
    set BJOBSINFO(USER,show) 0
    set BJOBSINFO(STAT,show) 1
    set BJOBSINFO(QUEUE,show) 1
    set BJOBSINFO(FROM_HOST,show) 0
    set BJOBSINFO(EXEC_HOST,show) 1
    set BJOBSINFO(JOB_NAME,show) 1
    set BJOBSINFO(SUBMIT_TIME,show) 1
    set BJOBSINFO(STAT,PEND) "Pending"
    set BJOBSINFO(STAT,PSUSP) "Suspended while pending"
    set BJOBSINFO(STAT,RUN) "Running"
    set BJOBSINFO(STAT,USUSP) "Suspended during execution"
    set BJOBSINFO(STAT,DONE) "Done"
    set BJOBSINFO(STAT,EXIT) "Aborted"
    set BJOBSINFO(STAT,UNKWN) "Unknown"
    set BJOBSINFO(STAT,ZOMBI) "Zombie"
    

    # Gaussian 94 Information
    variable G94() {}
}


# CODEINFO BEGIN ############################################################
# info proc getQueueInfo> Procedure to retrieve information about LSF queues
# arg remHost> Host for which data is needed
# return> None

proc webSubmit::lsf::getQueueInfo {remHost} {
    variable PATHS
    variable BQUEUES
    variable queueList
    variable queueHeaderList
    variable debug
    variable CONFIG

    # Determine valid login on remote host
    webSubmit::auth::createAuthDb
    set login ""
    foreach user $webSubmit::auth::dbKeyList {
	if [existDbVal webSubmit::auth $user $remHost] {
	    set login [getDbVal webSubmit::auth $user $remHost]
	    if {$login != "" && $login != "*"} {
		break
	    }
	}
    }

    # Get coarse-grained queue data
    if {$debug} {
	puts "Getting queue data: (Login = $login, RemHost = $remHost)"
    }

    if [catch {sshCmdExec $login $remHost $PATHS($remHost,bqueues)} allQData] {
	cgiBomb getQueueInfo "Unable to obtain queue information for $remHost\n$allQData"
    }

    set dataList [split $allQData \n]
    set queueHeaderList [compSplit [lindex $dataList 0]]
    set tmpQueueList [lreplace $dataList 0 0]
    foreach qData $tmpQueueList {
	set qDataList [compSplit $qData]
	set qName [lindex $qDataList 0]
	lappend queueList $qName
	for {set i 0} {$i < [llength $queueHeaderList]} {incr i} {
	    set header [lindex $queueHeaderList $i]
	    set data [lindex $qDataList $i]
	    set BQUEUES($qName,$header) $data
	}
    }

    # Parse data for queues to determine CPU time limit
    if [catch {sshCmdExec $login $remHost "$PATHS($remHost,bqueues) -l"} allQData] {
	cgiBomb getQueueInfo "Unable to obtain queue information for $remHost\n$allQData"
    }

    # Get CPU Limit and fix processors
    set qDataList [split $allQData \n]
    lappend queueHeaderList CPULIMIT
    foreach queue $queueList {
	# Processors
	if {$BQUEUES($queue,MAX) == "-"} {
	    set BQUEUES($queue,MAX) $CONFIG($remHost,maxProcessors)
	}

	# CPU Limit
	set BQUEUES($queue,CPULIMIT) "Unknown"
	set startIdx [lsearch -regexp $qDataList "QUEUE: *$queue"]

	for {set i $startIdx} {$i < [llength $qDataList]} {incr i} {
	    set line [lindex $qDataList $i]
	    if [string match *RUNLIMIT* $line] {
		set timeList [compSplit [lindex $qDataList [expr $i + 1]]]
		set BQUEUES($queue,CPULIMIT) "[lrange $timeList 0 1]"
		break
	    }
	}
    }
	
    if {$debug} {
	puts "Queue Header List: $queueHeaderList"
	foreach queue $queueList {
	    puts "\n  QUEUE: $queue"
	    foreach header $queueHeaderList {
		puts "    $header: $BQUEUES($queue,$header)"
	    }
	}
    }
    return
}


# CODEINFO BEGIN ############################################################
# info proc writeLSFjob> Procedure to write out an LSF job file
# arg fid> File descriptor for job control file
# return> None
proc webSubmit::lsf::writeLSFjob {fid} {
    variable bsubFlagList
    variable bsubOptList
    variable BSUBFLAGS
    variable BSUBOPTS
    variable jobScript
    variable shell
    
    # Shell for script interpretation
    if {$shell != ""} {
	puts $fid "#!$shell"
    }

    # Write out BSUB flags
    foreach flag $bsubFlagList {
	if {$BSUBFLAGS(-$flag) != "off"} {
	    puts $fid "# BSUB -$flag"
	}
    }

    # Write out BSUB options
    foreach opt $bsubOptList {
	if {$opt != "f"} {
	    if {$BSUBOPTS(-$opt) != ""} {
		puts $fid "# BSUB -$opt $BSUBOPTS(-$opt)"
	    }
	} else {
	    if {$BSUBOPTS(-f) != ""} {
		foreach file $BSUBOPTS(-f) {
		    puts $fid "# BSUB -f $file"
		}
	    }
	}
    }

    # Write out job script
    if {$jobScript != ""} {
	puts $fid "$jobScript"
    }

    return
}


# CODEINFO BEGIN ############################################################
# info proc submitLSFjob> Procedure to submit an LSF job

proc webSubmit::lsf::submitLSFjob {remHost remHome remUser} {
    variable jobFileName
    variable PATHS
    variable workDir

    set pid [pid]
    # Resolve job file name
    if {$jobFileName != ""} {
	set jobFileName [mkPathAbs $jobFileName $remHome]
    } else {
	set jobFileName "job_$pid"
	cgiWarn submitLSFjob "No name provided for job control script - set to <B>$jobFileName</B>"
    }

    # Write job control script to temporary file
    set tmpFile [file join /tmp tmp.lsfjob.$pid]
    if [catch {open $tmpFile w} fid] {
	cgiBomb submitLSFjob "Could not write temporary job control script"
    }
    writeLSFjob $fid
    close $fid

    # Copy job control file to remote host - Overwrite if necessary
    if [catch {webSubmit::network::scpFile $tmpFile put 1 $remUser $remHost $jobFileName} err] {
	cgiBomb submitLSFjob "Could not transfer LSF job file ($err)"
    }

    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Job file written:"]] [cgi_sample $jobFileName]"
    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Remote host:"]] [cgi_sample $remHost]"
    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Remote user:"]] [cgi_sample $remUser]"
    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Job File Submitted:"]]"
    if [catch {webSubmit::util::execCgi "cat $tmpFile"} result] {
	cgiWarn submitLSFjob "$result"
    } else {
	file delete -force $tmpFile
	cgi_preformatted {
	    foreach line [split $result \n] {
		puts [cgi_quote_html $line]
	    }
	}
    }

    #  Submit job file
    #    LSF Error Trapping:
    #  LSF occasionally returns harmless diagnostic output on stderr, hence
    #  don't attempt to trap errors, and just return output
    cgi_hr
    set bsubCmd $PATHS($remHost,bsub)
    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Submitting Job:"]] [cgi_bold "$bsubCmd < $jobFileName"]"
    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Working Directory:"]]  [cgi_bold "$workDir"]"
    catch {webSubmit::network::sshCmdExec $remUser $remHost "cd $workDir; $bsubCmd < $jobFileName"} result

    # Extract job Id
    set jid "Unknown"
    set lineList [split $result \n]
    foreach line $lineList {
	if [regexp {^Job *<([0-9]+)>} $line match tmp] {
	    set jid $tmp
	}
    }
    
    cgi_p "[cgi_font color=darkmagenta [cgi_bold "LSF Job ID:"]][cgi_nbspace][cgi_font color=blue [cgi_bold [cgi_sample "$jid"]]]"
    cgi_br

    cgi_preformatted {
	foreach line [split $result \n] {
	    puts [cgi_quote_html $line]
	}
    }
    return 0
}




# CODEINFO BEGIN ############################################################
# info proc writeSchema > Procedure to write out a LAM MPI schema file
# arg fid> File descriptor for schema file
# return> None
proc webSubmit::lsf::writeSchema {fid} {
    variable SchemaContents

        puts $fid "$SchemaContents"
   
    return
}






# CODEINFO BEGIN ############################################################
# info proc writeLSFMPIjob> Procedure to write out an LSF MPI job file
# arg fid> File descriptor for job control file
# return> None
proc webSubmit::lsf::writeLSFMPIjob {fid} {
    variable bsubFlagList
    variable bsubOptList
    variable BSUBFLAGS
    variable BSUBOPTS
    variable jobScript
    variable shell
    variable SchemaName
    variable inputFileList
    variable outputFileList

    # Shell for script interpretation
        puts $fid "#!/bin/bash"
    

    # Write out BSUB flags
    foreach flag $bsubFlagList {
        if {$BSUBFLAGS(-$flag) != "off"} {
            puts $fid "# BSUB -$flag"
        }
    }

    # Write out BSUB options
    foreach opt $bsubOptList {
        if {$opt != "f"} {
            if {$BSUBOPTS(-$opt) != ""} {
                puts $fid "# BSUB -$opt $BSUBOPTS(-$opt)"
            }
        } else {
            if {$BSUBOPTS(-f) != ""} {
                foreach file $BSUBOPTS(-f) {
                    puts $fid "# BSUB -f $file"
                }
            }
        }
    }

    


set var {
user=`whoami`
xinitialdir="/tmp/${user}"
finaldir="/wrk/${user}"
#
#
#

######  To copy files to /tmp/${user}, set cpfiles to the list of
######  files to be copied.
#}
puts $fid $var

puts $fid "cpfiles1=\"$inputFileList\""
puts $fid "cpfiles2=\"$outputFileList\""
puts $fid "program=\"$SchemaName\""



set var {
######  Get the running shell name without the path
#
export CMD="${0##*/}"
#
######  ... and for convenience:
#
export PROGRAM=${xinitialdir}/$program
export HOST=`hostname`
HOST=${HOST%%.*}
#
echo "Running $CMD on node $HOST"
echo "PROGRAM NAME =$program"
#
#
###### Copy executables and input file (if any) to /tmp/${user} on all the
###### allocated nodes
#
lsgrun -p -m "$LSB_HOSTS" mkdir $xinitialdir
for i in ${cpfiles1}
do
 case ${i} in
     /*)
        cfp=$i;;
     ~*)
        cfp=$i;;
     *)
        cfp=${LS_SUBCWD}/${i};;
 esac
 bcfp=`basename $cfp`
 lsgrun -p -m "$LSB_HOSTS" rcp rapid:${cfp} $xinitialdir/$bcfp
done
#
######  Start the program.
#
if [ -f $PROGRAM ]
then
  echo "$CMD: Starting user program $PROGRAM"
  cd ${xinitialdir}
 #
 #setup mpi hosts file
  LAMBHOST=.$LSB_JOBID
  echo $LSB_HOSTS

  for word in $LSB_HOSTS
do
  echo $word >> $LAMBHOST
done

 # boot lam
  /usr/local/lam/bin/lamboot $LAMBHOST

 # run mpi job
  /usr/local/lam/bin/mpirun -v -w -O -c2c -D $PROGRAM

 #Kill lam daemons and clean up hostfile
  /usr/local/lam/bin/wipe -v $LAMBHOST
  rm -f $LAMBHOST
else
  echo "$CMD: error $PROGRAM not found"
  exit
fi
#
####### Copy output files back to machine to finaldir (/wrk/user).
#

for o in ${cpfiles2}
do
  i=0
  for lsfhost in $LSB_HOSTS
  do
    echo copying ${lsfhost}:$xinitialdir/$o back to rapid:${finaldir}/${o}.$i.${lsfhost}.${LSB_JOBID}
    lsrcp ${lsfhost}:$xinitialdir/${o} rapid:${finaldir}/${o}.$i.${lsfhost}.${LSB_JOBID} 1>/dev/null 2>/dev/null
    i=`expr $i + 1`
  done
done

}
puts $fid "$var"




    return
}




# CODEINFO BEGIN ############################################################
# info proc submitLSFMPIjob> Procedure to submit an LSFMPI job

proc webSubmit::lsf::submitLSFMPIjob {remHost remHome remUser} {
    variable jobFileName
    variable PATHS
    variable workDir
    variable SchemaName
    variable SchemaPathName

    set pid [pid]
    # Resolve Schema file name
    if {$SchemaName != ""} {
        set SchemaPathName [mkPathAbs $SchemaName $workDir]
    } else {
        cgiBomb submitLSFMPIjob "No name provided for Schema File Name"
    }

    # Write SchemaContents to temporary file
    set tmpsFile [file join /tmp tmps.lsfjob.$pid]
    if [catch {open $tmpsFile w} fid] {
        cgiBomb submitLSFMPIjob "Could not write temporary Schema File"
    }
    writeSchema $fid
    close $fid

    # Copy job control file to remote host - Overwrite if necessary
    if [catch {webSubmit::network::scpFile $tmpsFile put 1 $remUser $remHost $SchemaPathName} err] {
        cgiBomb submitLSFjob "Could not transfer MPI Schema file ($err)"
    }
    # Set permission on schema file name on remote host
    if [catch {webSubmit::network::sshCmdExec $remUser $remHost "cd $workDir; chmod 0644 $SchemaName"} err] {
        cgiBomb submitLSFjob "Could not change permissions on Schema file ($err)"
    }




    # Resolve job file name
    if {$jobFileName != ""} {
        set jobFileName [mkPathAbs $jobFileName $workDir]
    } else {
        set jobFileName "job_$pid"
        cgiWarn submitLSFMPIjob "No name provided for job control script - set to <B>$jobFileName</B>"
    }

    # Write job control script to temporary file
    set tmpFile [file join /tmp tmp.lsfjob.$pid]
    if [catch {open $tmpFile w} fid] {
        cgiBomb submitLSFMPIjob "Could not write temporary job control script"
    }
    writeLSFMPIjob $fid
    close $fid

    # Copy job control file to remote host - Overwrite if necessary
    if [catch {webSubmit::network::scpFile $tmpFile put 1 $remUser $remHost $jobFileName} err] {
        cgiBomb submitLSFMPIjob "Could not transfer LSF MPI job file ($err)"
    }




    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Schema file written:"]] [cgi_sample $SchemaName]"
    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Job file written:"]] [cgi_sample $jobFileName]"
    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Remote host:"]] [cgi_sample $remHost]"
    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Remote user:"]] [cgi_sample $remUser]"
    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Job File Submitted:"]]"
    if [catch {webSubmit::util::execCgi "cat $tmpFile"} result] {
        cgiWarn submitLSFMPIjob "$result"
    } else {
        file delete -force $tmpFile
        cgi_preformatted {
            foreach line [split $result \n] {
                puts [cgi_quote_html $line]
            }
        }
    }

    #  Submit job file
    #    LSF Error Trapping:
    #  LSF occasionally returns harmless diagnostic output on stderr, hence
    #  don't attempt to trap errors, and just return output
    cgi_hr
    set bsubCmd $PATHS($remHost,bsub)
    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Submitting Job:"]] [cgi_bold "$bsubCmd < $jobFileName"]"
    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Working Directory:"]]  [cgi_bold "$workDir"]"
    catch {webSubmit::network::sshCmdExec $remUser $remHost "cd $workDir; $bsubCmd < $jobFileName"} result


    # Extract job Id
    set jid "Unknown"
    set lineList [split $result \n]
    foreach line $lineList {
        if [regexp {^Job *<([0-9]+)>} $line match tmp] {
            set jid $tmp
        }
    }

    cgi_p "[cgi_font color=darkmagenta [cgi_bold "LSF Job ID:"]][cgi_nbspace][cgi_font color=blue [cgi_bold [cgi_sample "$jid"]]]"
    cgi_br

    cgi_preformatted {
        foreach line [split $result \n] {
            puts [cgi_quote_html $line]
        }
    }
    return 0
}








# CODEINFO BEGIN #################################################################
# info proc getBjobsInfo> Procedure to parse and store the output from bjobs.
#
#     Current Output format:
#     ----------------------
#     JOBID  USER   STAT   QUEUE    FROM_HOST    EXEC_HOST   JOB_NAME   SUBMIT_TIME
#     2358   ryanm  PEND   Small8   rapid                    test       Aug 31 21:27
#
#     Procedure has to parse the data based on header column locations in order
# to differentiate meaningful white-space.

# arg flag> bjobs flag used to generate data
# return> None
proc webSubmit::lsf::getBjobsInfo {remUser remHost {flag ""}} {
    variable BJOBS
    variable PATHS

    # Retrieve bjobs data
    if [catch {sshCmdExec $remUser $remHost "$PATHS($remHost,bjobs) $flag"} data] {
	if ![string match "*No unfinished job found*" $data] {
	    cgiBomb getBjobsInfo "Could not retrieve job data on $remHost ($data)"
	}
    }

    # Parse BJOBS information based on flag passed
    set BJOBSINFO(jobIdList) ""
    set jobList ""
    if {$flag == ""} {
	set dataList [split $data \n]
	if {[lsearch -exact $dataList "No unfinished job found"] == -1} {
	    set header [lindex $dataList 0]
	    set headerList [compSplit $header]
	    set jobList ""

	    # Bogus method for extracting data needed to deal with white space
	    #  in multicolumn output
	    foreach hdr $headerList {
		set POS($hdr,start) [string first $hdr $header]
	    }

	    for {set i 0} {$i < [expr [llength $headerList] - 1]} {incr i} {
		set this [lindex $headerList $i]
		set next [lindex $headerList [expr $i + 1]]
		set POS($this,end) [expr $POS($next,start) - 1]
	    }

	    for {set i 1} {$i < [llength $dataList]} {incr i} {
		set jobData [lindex $dataList $i]
		foreach hdr $headerList {
		    if [info exists POS($hdr,end)] {
			set tmp [string range $jobData $POS($hdr,start) $POS($hdr,end)]
		    } else {
			set tmp [string range $jobData $POS($hdr,start) end]
		    }
		    set tmp [string trim $tmp]

		    # Use jobId as index for job array
		    if {$hdr == "JOBID"} {
			set jobId $tmp
			lappend jobList $jobId
			set BJOBS($jobId,JOBID) $tmp
		    } else {
			set BJOBS($jobId,$hdr) $tmp
		    }
		}
	    }
	}
    }

    # Identify all jobs owned by $remUser
    set BJOBS(jobIdList) ""
    set BJOBS(completeJobList) $jobList
    foreach job $jobList {
	if {$BJOBS($job,USER) == "$remUser"} {
	    lappend BJOBS(jobIdList) $job
	}
    }
    return
}


# CODEINFO BEGIN ##################################################################
# info proc checkLimits> Procedure to check requested resources against queue
#     limits for a given LSF host.  If the request exceeds limits, a warning is
#     raised.

# arg remHost> LSF Host
# arg resource> Resource being requested
# arg request> Value of resource being requested
# arg units> Optional units for the request
# return> Warning if request exceeds queue limits

proc webSubmit::lsf::checkLimits {remHost resource request {units ""}} {
    variable CONFIG

    # Verify that limits are available for host and that a finite request has been made
    if {[lsearch -exact $CONFIG(hostList) $remHost] == -1} {
	cgiWarn checkLimits "Resource requests cannot be verified for host $remHost"
	return
    } elseif {$request == ""} {
	return
    }

    # If resource is maxCPUTime, convert request to seconds for comparison
    if {$resource == "maxCPUTime"} {
	if [regexp {([0-9]+):([0-9][0-9]):([0-9][0-9]).*} $request match hh mm ss] {
	    set request [expr ($hh * 3600) + ($mm * 60) + $ss]
	} elseif [regexp {([0-9]?[0-9]):([0-9][0-9]).*} $request match mm ss] {
	    set request [expr ($mm * 60) + $ss]
	}
    }

    # Check request against limits for resource
    if {$units == ""} {
	if [info exists CONFIG($remHost,$resource)] {
	    if [expr $request > $CONFIG($remHost,$resource)] {
		cgiWarn checkLimits "Request exceeds $resource (Max = $CONFIG($remHost,$resource), Req = $request)"
	    }
	} else {
	    cgiWarn checkLimits "No limits available for resource $resource on host $remHost"
	}
    } else {
	if [info exists CONFIG($remHost,$resource,$units)] {
	    if [expr $request > $CONFIG($remHost,$resource,$units)] {
		cgiWarn checkLimits "Request exceeds $resource (Max = $CONFIG($remHost,$resource,$units) $units,\
			Req = $request $units)"
	    }
	} else {
	    cgiWarn checkLimits "No limits available for resource $resource ($units) on host $remHost"
	}
    }
    return
}


# CODEINFO BEGIN ##########################################################
# info proc submitG94Job> Procedure to write and submit a Gaussian 94 LSF
#     job

# arg remDir> Remote working directory
# arg remHost> LSF host
# arg remUser> Remote user
# arg clobberG94> Overwrite flag for G94 file
# arg tmpG94File> Name of local Gaussian 94 input file
# arg remG94File> Name of remote G94 input file
# return> None
proc webSubmit::lsf::submitG94Job {remDir remHost remUser clobberG94 tmpG94File remG94File} {
    variable BSUBOPTS
    variable G94
    variable PATHS
    variable jobScript
    variable debug
    
    if {$debug} {
	puts "Submitting G94 Job..."
	puts "  Remote directory: $remDir"
	puts "  Remote Host: $remHost"
	puts "  Remote User: $remUser"
	puts "  Clobber? $clobberG94"
	puts "  Temp G94 File: $tmpG94File ([file exists $tmpG94File])"
	puts "  Remote G94 File: $remG94File"
    }

    # Compare number of processors specified (if any) with
    #   System processor limits
    checkG94Proc $remHost $tmpG94File

    # Set up Output and Error filenames
    if {$G94(OutputType) == "UD" && $BSUBOPTS(-o) != ""} {
	set f1 [file tail $BSUBOPTS(-o)]
	set BSUBOPTS(-o) [file join $remDir $f1.out]
	set BSUBOPTS(-e)  [file join $remDir $f1.err]
    } elseif {$G94(OutputType) == "IF"} {
	set f1 [file tail $remG94File]
	set BSUBOPTS(-o) [file join $remDir $f1.out]
	set BSUBOPTS(-e)  [file join $remDir $f1.err]
    } else {
	set BSUBOPTS(-o) [file join $remDir $BSUBOPTS(-r).out]
	set BSUBOPTS(-e)  [file join $remDir $BSUBOPTS(-r).err]
    }
    set jobScript "$PATHS($remHost,g94) < $remG94File"

    # Create job file names
    set locJobFile "/tmp/$BSUBOPTS(-r)_[pid]"
    set remJobFile [file join $remDir [file tail $locJobFile]]

    # Debugging
    if {$debug} {
	puts "Temp G94 File: $tmpG94File"
	puts "Remote G94 File: $remG94File"
	puts "Local job file: $locJobFile"
	puts "Remote job file: $remJobFile"
	puts "Output File: $BSUBOPTS(-o)"
	puts "Error File: $BSUBOPTS(-e)"
	puts "Job script: $jobScript"
    }

    # Write Job File
    if [catch {open "$locJobFile" w 0600} fid] {
	cgiBomb submitG94Job "Could not open Gaussian 94 job file ($fid)"
    } else {
	#  write job file
	writeLSFjob $fid
	close $fid 
	cgi_p "[cgi_font color=darkmagenta [cgi_bold "LSF Job File:"]]"
	set txt [execCgi "cat $locJobFile"]
	cgi_preformatted {
	    foreach line [split $txt \n] {
		puts "[cgi_sample $line]"
	    }
	}
	cgi_br
    }
    
    #  Upload Gaussian 94 file and LSF file, then submit job
    if [catch {
	webSubmit::network::scpFile $tmpG94File put $clobberG94 $remUser $remHost $remG94File
	webSubmit::network::scpFile $locJobFile put 1 $remUser $remHost $remJobFile} result] {
	    cgiBomb submitG94Job "Problem uploading input files ($result)"
    } else {
	# Delete Local job file
	file delete $locJobFile

	# Delete local Gaussian directives file
	file delete $tmpG94File
	
	# Submit job
	set subCmd $PATHS($remHost,bsub)
	if [catch {webSubmit::network::sshCmdExec $remUser $remHost "$subCmd $remJobFile"} result] {
	    cgiBomb wsLSF.tcl "Could not submit Gaussian 94 Job ($result)"
	} else {
	    # Extract job Id
	    set idLine [lindex [split $result \n] 1]
	    regexp { *Request * *<([0-9]+).*} $idLine match jid
	    cgi_p "[cgi_font color=darkmagenta [cgi_bold "LSF Job ID:"]][cgi_font color=blue [cgi_bold [cgi_sample "$jid"]]]"
	    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Job file:"]][cgi_sample $remJobFile]"
	    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Output file:"]][cgi_sample $BSUBOPTS(-o)]"
	    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Error file:"]] [cgi_sample $BSUBOPTS(-e)]"
	    cgi_br
	    
	    cgi_preformatted {
		foreach line [split $result \n] {
		    puts [cgi_quote_html $line]
		}
	    }
	} 
    }
}


# CODEINFO BEGIN ####################################################
# info proc checkG94Proc> Procedure to construct a consistent number
#     of processors for a Gaussian input file.  Number of processors 
#     is checked against current limits and redefines BSUBOPTS(-l,mpp_p)
#     if inconsistent.  If the input file is specified as the location
#     for the number of processors, but no specification is given,
#     then the file is left untouched.

# arg remHost> Host on which calculation performed
# arg g94File> Gaussian 94 input file name
# return> None
proc webSubmit::lsf::checkG94Proc {remHost g94File} {
    variable G94
    variable CONFIG
    variable BSUBOPTS
    variable debug

    if {$debug} {
	puts "Verifying G94 Processors:"
	puts "  Remote Host: $remHost (maxProcessors = $CONFIG($remHost,maxProcessors))"
	puts "  G94 File: $g94File"
    }

    # String used by Gaussian 94 to define the number of processors
    set pstr "\%NProc="
    
    # Determine if number of processors specified in input file
    set reset 0
    if [file exists $g94File] {
	if [catch {tclGrep $pstr $g94File} procLine] {
	    # No processors specified
	    if {$G94(NProcType) == "IF"} {
		# Allow Gaussian to determine default number of processors
		return
	    } else {
		if {$BSUBOPTS(-n) == ""} {
		    # Fix at maximum number for the current class
		    set BSUBOPTS(-n) $CONFIG($remHost,maxProcessors)
		}
	    }
	} else {
	    if {$G94(NProcType) == "IF"} {
		# Extract Nproc from input file
		regexp "^${pstr}\(.*\)" $procLine match BSUBOPTS(-n)
	    }
	}
    } else {
	cgiBomb checkG94Proc "Specified Gaussian input file \"$g94File\" does not exist"
    }

    if {$debug} {
	puts "   Number of processors specified: $BSUBOPTS(-n)"
    }

    # Determine if processors in allowed range for current class
    if {$BSUBOPTS(-n) < 1 || $BSUBOPTS(-n) > $CONFIG($remHost,maxProcessors)} {
	cgiWarn checkG94Proc "Error in specification of number of processors.  Reset\
		to maximum for $remHost ($CONFIG($remHost,maxProcessors))"
	set BSUBOPTS(-n) $CONFIG($remHost,maxProcessors)
	set reset 1
    }

    if {$reset || $G94(NProcType) == "UD"} {
	# Open Gaussian 94 input file and determine number of processors
	if [catch {open "$g94File" r} fid] {
	    cgiBomb checkG94Proc "Could not open Gaussian input file ($fid)"
	} else {
	    # Read input file
	    set g94DataList [split [read $fid] \n]
	    set idx [lsearch -regexp $g94DataList $pstr]
	    if {$idx != -1} {
		# Input file already contains %Nproc --> Replace
		set g94DataList [lreplace $g94DataList $idx $idx "${pstr}$BSUBOPTS(-n)"]
	    } else {
		# Input file does not specify %Nproc --> Insert at top of file
		if {$G94(NProcType) == "UD"} {
		    set g94DataList [linsert $g94DataList 0 "${pstr}$BSUBOPTS(-n)"]
		}
	    }
	    close $fid

	    # Rewrite G94 input file with correct number of processors
	    set fid [open $g94File w]
	    foreach line $g94DataList {
		puts $fid $line
	    }
	    close $fid
	}
    }
}












