#!/itl/links/generic/bin/tclsh8.0
# WebSubmit 2.0 name @(#)wsLoadLeveler.tcl	1.30 SCCS/s.wsLoadLeveler.tcl  99/07/12 

# AUTHORS #############################################################

#  Authors: John Koontz, Ryan McCormack, Judith Devaney
#  Original Development Work: Robert Lipman, John Koontz, Katherine
#      Pagoaga

# Copyright Information ###############################################

# This software was developed at the National Institute of Standards
# and Technology by employees of the Federal Government in the course
# of their official duties.  Pursuant to title 17 Section 105 of the
# United States Code this software is not subject to copyright
# protection and is in the public domain.

# CODEINFO BEGIN ######################################################

# info script wsLSF.tcl> Support utilities for LSF batch processing.

# CODEINFO END ########################################################

# Package definitions
package provide wsLoadLeveler 1.0

namespace eval webSubmit::loadleveler {
    # Namespace variables
    variable thisNS [namespace current]
    variable wsNS [namespace parent]
    variable upNS [namespace parent]

    # Procedures to be imported
    namespace import ${wsNS}::database::*
    namespace import ${wsNS}::util::*

    # Procedures to be exported
    namespace export getClassInfo
    namespace export submitLLJob
    namespace export submitGaussianJob
    namespace export createKeywordDb
    namespace export getValidNumProc
    namespace export buildPoeScript
    namespace export buildLLBoolean

    # Variable aliases
    alias $wsNS wsLibDir

    # Host-specific path information
    variable PATHS() {}
    
    ;;set PATHS(danube.nist.gov,poe) "/usr/bin/poe"
    ;;set PATHS(danube.nist.gov,g94) "/usr/local/apps/WebSubmit/gauss/gauss"
    ;;set PATHS(danube.nist.gov,g98) "/usr/local/apps/WebSubmit/gauss/gauss.new"
    ;;set PATHS(danube.nist.gov,adminFileName) "/danube/support/loadl/LoadL_admin"
    ;;set PATHS(danube.nist.gov,localAdminFileName) [file join $wsLibDir LL danube.nist.gov LoadL_admin]
    ;;set PATHS(danube.nist.gov,llsubmit) "/usr/lpp/LoadL/nfs/bin/llsubmit"
    ;;set PATHS(danube.nist.gov,llcancel) "/usr/lpp/LoadL/nfs/bin/llcancel"
    ;;set PATHS(danube.nist.gov,llclass) "/usr/lpp/LoadL/nfs/bin/llclass"
    ;;set PATHS(danube.nist.gov,llq) "/usr/lpp/LoadL/nfs/bin/llq"
    ;;set PATHS(danube.nist.gov,llstatus) "/usr/lpp/LoadL/nfs/bin/llstatus"
    ;;set PATHS(danube.nist.gov,llhold) "/usr/lpp/LoadL/nfs/bin/llhold"
    ;;set PATHS(danube.nist.gov,llprio) "/usr/lpp/LoadL/nfs/bin/llprio"
    ;;set PATHS(danube.nist.gov,jmsum) "/usr/local/bin/jmsum"

    ;;set PATHS(sci-apps.nist.gov,poe) "undefined"
    ;;set PATHS(sci-apps.nist.gov,g94) "/usr/local/apps/g94/g94"
    ;;set PATHS(sci-apps.nist.gov,g98) "/usr/local/apps/g95/g95"
    ;;set PATHS(sci-apps.nist.gov,adminFileName) "/sci-apps/support/loadl/LoadL_admin"
    ;;set PATHS(sci-apps.nist.gov,localAdminFileName) [file join $wsLibDir LL sci-apps.nist.gov LoadL_admin]
    ;;set PATHS(sci-apps.nist.gov,llsubmit) "/sci-apps/support/loadl/bin/llsubmit"
    ;;set PATHS(sci-apps.nist.gov,llcancel) "/sci-apps/support/loadl/bin/llcancel"
    ;;set PATHS(sci-apps.nist.gov,llclass) "/sci-apps/support/loadl/bin/llclass"
    ;;set PATHS(sci-apps.nist.gov,llq) "/sci-apps/support/loadl/bin/llq"
    ;;set PATHS(sci-apps.nist.gov,llstatus) "/sci-apps/support/loadl/bin/llstatus"
    ;;set PATHS(sci-apps.nist.gov,llhold) "/sci-apps/support/loadl/bin/llhold"
    ;;set PATHS(sci-apps.nist.gov,llprio) "/sci-apps/support/loadl/bin/llprio"
    ;;set PATHS(sci-apps.nist.gov,jmsum) "/usr/local/bin/jmsum"

    # General Variables
    variable debug 0
    variable CLASS() {}
    variable classNameList
    # Attributes of classes found in class stanzas
    variable classTokenList [list admin class_comment exclude_groups\
	    exclude_users include_groups include_users master_node_requirement\
	    maxjobs max_processors nice NQS_class NQS_submit NQS_query\
	    priority core_limit cpu_limit data_limit file_limit job_cpu_limit\
	    rss_limit stack_limit wall_clock_limit]

    # Variables for job control file parsing
    variable scriptList
    variable commentList
    variable jobKeywordList
    variable JOBCONTROL() {}
    set JOBCONTROL(comment) {^\# *\[^@\]*}
    set JOBCONTROL(commentvars) "match"
    set JOBCONTROL(key) {^\# *@ *(\[^= \]+)( *= *(.*)|(\[ \t\]*$))}
    set JOBCONTROL(keyvars) "match name tmp value"

    # Load Leveler Information
    variable LLPREF() {}
    foreach var [list Memory Adapter Feature] {
	set LLPREF($var) ""
    }
    variable LLREQ() {}
    foreach var [list Memory Adapter Feature] {
	set LLREQ($var) ""
    }
    variable jobScript ""

    # Gaussian Information
    variable Gaussian() {}
    set Gaussian(JobNames) [list "Input File" "User-defined, enter in blank"]

    # POE and Local Node Information
    variable scratchRoot "/tmp"
    variable inputFileList ""
    variable outputFileList ""

    # Keyword Database
    variable DB
    variable dbKeyList
    variable attrList
    variable dbFile [file join $wsLibDir LL keyword]
}


# CODEINFO BEGIN ######################################################
# info proc createKeywordDb> Procedure to parse and store database related 
#     to load leveler keyword information.  Also creates variables that 
#     correspond to all Load Leveler keywords

# return> Status
proc webSubmit::loadleveler::createKeywordDb {} {
    variable thisNS 
    variable dbKeyList
    variable dbFile

    # Read Database Information
    buildDb $dbFile $thisNS

    # Set up keywords and their values
    foreach keyword $dbKeyList {
	variable $keyword
	catch {unset $keyword}
	set $keyword [getDbVal $thisNS $keyword value]
    }

    return 0
}


# CODEINFO BEGIN ###########################################################
# info proc getClassInfo> Procedure to retrieve and parse Load Leveler admin
#     file and retrieve data about class definitions on the system.  A local
#     copy of this data is updated once per day and serialized to hopefully
#     remove the need to constantly grab and parse this file.

# arg remHost> Remote Load Leveler Host
# return> None

proc webSubmit::loadleveler::getClassInfo {remHost} {
    variable CLASS
    variable classNameList
    variable classTokenList
    variable PATHS
    variable thisNS

    # Look for local version of class data and load
    set remFile $PATHS($remHost,adminFileName)
    set localFile $PATHS($remHost,localAdminFileName)
    set localSerial "$localFile.tcl"
    set getData 0
    set serialExist 0
    set localExist 0
    if [file exists $localFile] {
	set localExist 1
	if [file exists $localSerial] {
	    set serialTime [file mtime $localSerial]
	    set localTime [file mtime $localFile]
	    if {[expr $serialTime - $localTime] > 86400} {
		# Serialized file out of date wrt localFile
		set getData 1
	    } else {
		if {$serialTime > $localTime} {
		    # Serial file newer than local file, but no more than one day
		    set serialExist 1
		}
	    }
	}
    } else {
	# Ensure existence of directory for localFile
        set localFileDir [file dirname $localFile]
        if ![file isdirectory $localFileDir] {
	    tclMkdir 775 $localFileDir
	}
	set getData 1
    }

    # Determine valid login on remote host
    webSubmit::auth::createAuthDb
    set remUser [webSubmit::auth::authTrans $remHost]
    if {$remUser == ""} {
	cgiBomb getClassInfo "Current user is not authorized to access $remHost"
    }

    set parseData 0
    if {$getData} {
	# Retrieve remote file
	if [catch {webSubmit::network::getRemFile $remUser $remHost $remFile} adminData] {
	    cgiBomb getClassInfo "Could not retrieve remote Load Leveler admin file:\n$adminData"
	}
	set parseData 1

	# Write remote file to local file
	if [catch {open $localFile w} fid] {
	    cgiBomb getClassInfo "Could not write local copy of class data file:\n$fid"
	} else {
	    foreach line $adminData {
		puts $fid $line
	    }
	    close $fid
	}
    } else {
	if {!($serialExist)} {
	    # Load data from localFile
	    if [catch {tclCat $localFile} adminData] {
		cgiBomb getClassInfo "Could not retrieve local Load Leveler admin file:\n$adminData"
	    } else {
		set adminData [split $adminData \n]
		set parseData 1
	    }
	} else {
	    # Load Serialized version - Data need not be parsed
	    loadSerial $localSerial
	}
    }

    # Parse admin data if necessary
    if {$parseData} {
	set classData 0
	set classNameList ""
	foreach line $adminData {
	    if [regexp "^\[ \t\]*#.*" $line match] {
		# Comment - ignore
	    } else {
		if {$classData} {
		    if ![regexp "^\[ \t\]*$" $line match] {
			# Class data continues (non-blank line encountered)
			regsub "\[ \t\]*# .*" $line "" line
			lappend CLASS($className) $line
		    } else {
			# Blank line encountered:  end of data for this class
			set classData 0
		    }
		} else {
		    if [regexp "^\[ \t\]*\(\[^:\]+\):\[ \t\]+type *= *class" $line match className] {
			set classData 1
			lappend classNameList $className
			catch {unset CLASS(className)}
			
			# Remove comments from line and add to class definition
			regsub "\[ \t\]*# .*" $line "" line
			lappend CLASS($className) $line
		    }
		}
	    }
	}

	# Parse class data
	foreach class $classNameList {
	    # Join all class data to avoid problems with entries that span multiple lines
	    #  in the admin file
	    set data [join $CLASS($class) " "]
	    
	    foreach token $classTokenList {
		if [regexp "$token\[ \t\]*=\[ \t\]*(\[^=\]+)=" $data match tmp] {
		    # Found token - Strip off trailing data corresponding to next token
		    set value [lreplace $tmp end end]
		    set CLASS($class,$token) $value
		} elseif [regexp "$token\[ \t\]*=\[ \t\]*(\[^=\]+)" $data match tmp] {
		    set CLASS($class,$token) $tmp
		}
	    }
	}

	# Remove default class from list for clarity
	set idx [lsearch -exact $classNameList default]
	if {$idx != -1} {
	    set classNameList [lreplace $classNameList $idx $idx]
	}
	
        # Propagate default class properties to other classes were needed

        foreach class $classNameList {
	    foreach elmt [array names CLASS default,*] {
		set attribute [lindex [split $elmt ","] 1]
		if {[llength [array names CLASS $class,$attribute]] == 0} {
		    set CLASS($class,$attribute) $CLASS(default,$attribute)
		}
	    }
	}

        # Ensure there's a wall_clock_limit

	foreach class $classNameList {
	    if {[llength [array names CLASS $class,wall_clock_limit]] == 0} {
		set CLASS($class,wall_clock_limit) "none"
	    }
	}
	
	# Sort classes according to max_processors
	set classNameList [lsort -command compClass $classNameList]
    }

    # Serialize data if necessary
    if {!($serialExist)} {
	serialize $thisNS [list CLASS classNameList] $localSerial
    }
}


# CODEINFO BEGIN ######################################################

# info proc parseJobData> Procedure to retrieve and parse a job control
#     file on a remote system.

# arg jobData> Job data to be parsed
# return> Status
# loc-arr KEYWORDS> Local array to hold directive values
# loc-arr JOBCONTROL> Array containing regeular expression data to parse
#     job control file
# loc-list scriptList> Local list for control script
# loc-list commentList> Local list containing comments from script

proc webSubmit::loadleveler::parseJobData {jobData} {
    # Variable definitions
    foreach var [list thisNS JOBCONTROL jobKeywordList commentList scriptList debug] {
	variable $var
    }

    # Construct regular expressions that act to extract data from control
    #  file.  These must be carefully constructed to avoid difficulties
    #  with the Tcl interpreter and metacharacters
    foreach key [list key keyvars comment commentvars] {
	if ![info exists JOBCONTROL($key)] {
	    cgiBomb parseJobData "Insufficient job control format data."
	}
    }
    set keyCmd "regexp \"$JOBCONTROL(key)\" \$line $JOBCONTROL(keyvars)"
    set commCmd "regexp \"$JOBCONTROL(comment)\" \$line $JOBCONTROL(commentvars)"
    if {$debug} {
	puts "Key match regexp: $keyCmd"
	puts "Comm match regexp: $commCmd"
    }

    # Parse job file: Break into keywords,comments and script commands
    catch {unset jobKeywordList}
    foreach line $jobData {
	if [eval $keyCmd] {
	    # Job Control Keywords
	    variable $name
	    set $name $value
	    lappend jobKeywordList $name
	} elseif [eval $commCmd] {
	    # Comment line
	    lappend commentList $line
	} elseif [regexp "^\[ \t\]*$" $line match] {
	    # Blank line - ignore
	} else {
	    # Script statement
	    lappend scriptList $line
	}
    }

    # Debugging
    if {$debug} {
	puts "\nComment block:"
	puts "--------------"
	foreach comment $commentList {
	    puts $comment
	}

	puts "\nKeywords:"
	puts "-----------"
	foreach keyword $jobKeywordList {
	    puts "Key: \"$keyword\""
	    puts "Value: \"[set $keyword]\"\n"
	}

	puts "\nScript:"
	puts "-------"
	foreach line $scriptList {
	    puts $line
	}
    }
    return 0
}

# CODEINFO BEGIN ###############################################################
# info proc getValidClass> Determine if the current class is consistent with the
#     specified number of processors.  If the class is not valid, then search
#     for the lowest valid class and return it.

# arg testClass> Class to test
# arg numProc> Number of processors
# return> Valid class or error
proc webSubmit::loadleveler::getValidClass {testClass numProc} {
    variable CLASSES
    variable classNameList

    set validClassList ""
    foreach class $classNameList {
	if {$numProc <= $CLASS($class,max_processors)} {
	    lappend validClassList $class
	}    
    }

    # Check input class against valid class list
    if {$validClassList == ""} {
	# No valid classes
	cgiBomb getValidClass "Number of processors exceeds maximum allowed"
    } else {
	if {[lsearch -exact $validClassList $testClass] == -1} {
	    # Specified class is not valid - return lowest valid class
	    return [lindex $validClassList 0]
	} else {
	    # Input class is valid
	    return $testClass
	}
    }
}


# CODEINFO BEGIN ######################################################
# info proc compClass> Procedure to sort classes based on max_processors
# arg class1> Name of first class
# arg class2> Name of second class
# return> Class with smaller number of max_processors

proc webSubmit::loadleveler::compClass {class1 class2} {
    variable CLASS

    set np1 $CLASS($class1,max_processors)
    set np2 $CLASS($class2,max_processors)
    if {$np1 < $np2} {
	return -1
    } elseif {$np1 > $np2} {
	return 1
    } else {
	return 0
    }
}


################################################################################
# info proc getValidNumProc> Check number of processors, reset based on class
# arg class> Class to consider
# return> Status (1 = Reset, 0 = Valid)
proc webSubmit::loadleveler::getValidNumProc {class} {
    variable CLASS
    variable thisNS
    variable min_processors
    variable max_processors
    
    # Get max_processors for specified class
    if [info exists CLASS($class,max_processors)] {
	set maxProc $CLASS($class,max_processors)
    } else {
	cgiBomb getValidNumProc "Could not retrieve max_processors for class $class"
    }

    set reset 0
    if {$min_processors != "" || $max_processors != ""} {
	if {$min_processors == ""} {set min_processors 1}
	if {$max_processors == ""} {set max_processors $min_processors}
	
	if {$max_processors > $maxProc} {
	    set max_processors $maxProc
	    set reset 1
	}

	if {$min_processors > $maxProc} {
	    set min_processors $maxProc
	    set reset 1
	}
	
	if {$min_processors > $max_processors} {
	    set min_processors $max_processors
	    set reset 1
	}
    }
    return $reset
}


# CODEINFO BEGIN ###################################################
#  info proc writeLine> Write formatted LoadLeveler command line
# arg fid> File descriptor to write
# arg arg1> Load Leveler keyword
# arg arg2> Keyword value
# return> None
proc webSubmit::loadleveler::writeLine {fid arg1 {arg2 ""}} {

   set fmt1 "# @ %-15s"
   set fmt2 "# @ %-15s = %-s"
   
   if {$arg2 != ""} {
      set str [format $fmt2 $arg1 $arg2]
   } else {
      set str [format $fmt1 $arg1]
   }
   puts $fid $str
}


# CODEINFO BEGIN #############################################################
# info proc writeJobFile> Procedure to write out a Load Leveler job control script
# arg fid> File descriptor to write
# return> list of error messages (if any)
proc webSubmit::loadleveler::writeJobFile {fid} {
    variable dbKeyList
    variable thisNS
    variable jobScript
    set errMsgList ""
    foreach key $dbKeyList {
	variable $key
    }

    # Remove keywords that are treated specially
    set specialKeys [list shell queue executable parallel_path]
    foreach key $specialKeys {
	set idx [lsearch -exact $dbKeyList $key]
	set dbKeyList [lreplace $dbKeyList $idx $idx]
    }

    # Shell header
    if {$shell != "" && $jobScript != ""} {
	puts $fid "#!$shell"
    }

    # Comment
    puts $fid "#"
    puts $fid "# LoadLeveler job command file generated by WebSubmit"
    puts $fid "# For more info, see: http://www.nist.gov/itl/div895/websubmit"
    puts $fid "#"

    # Executable
    #   - Ignore this value if a job shell script has been specified
    #   - Raise an error if no executable or shell script given
    if {$executable != ""} {
	writeLine $fid executable $executable
    } else {
	if {$jobScript == ""} {
	    cgiWarn writeJobFile "No executable or job shell script specified"
	}
    }
    
    # Parallel Path
    if {$parallel_path != "" && $job_type == "pvm3"} {
	writeLine $fid parallel_path  $parallel_path
    }

    ###############################
    # General Load Leveler Keywords
    ###############################
    foreach keyword $dbKeyList {
	variable $keyword
	set value [set $keyword]
	if {$value != ""} {
	    writeLine $fid $keyword $value
	}
    }

    # Required - queue command
    writeLine $fid queue
    puts $fid "#"

    # Additional script commands in job control file
    if {$jobScript != ""} {
	puts $fid $jobScript
    }

    return
}


# CODEINFO BEGIN ###############################################################
# info proc submitLLJob> Procedure to submit a Load Leveler job
# arg remHost> Remote load leveler host system
# arg remHome> Home directory on remote host
# arg remUser> Remote user login name
# return> None

proc webSubmit::loadleveler::submitLLJob {remHost remHome remUser} {
    variable CLASSES
    variable dbKeyList
    variable PATHS
    variable debug
    foreach key $dbKeyList {
	variable $key
    }

    # Set up initial working directory
    if {$initialdir == ""} {
	set initialdir $remHome
    }

    # Set filename for job command file
    if {$job_name != ""} {
	set jobFileName ${job_name}_[pid].job
    } elseif {$executable != ""} {
	set jobFileName [file tail $executable]_[pid].job
    } else {
	set jobFileName loadleveler_[pid].job
    }
    set jname [file join /tmp $jobFileName]

    # Write temporary job file
    if [catch {open "$jname" w 0644} fid] {
	webSubmit::util::bomb llutil.tcl "Could not create temporary job file"
    } else {
	writeJobFile $fid
	close $fid    
    }

    set remFile [file join $initialdir $jobFileName]
    # Copy file to remote host - Overwrite if necessary
    if [catch {webSubmit::network::scpFile $jname put 1 $remUser $remHost $remFile} err] {
	cgiBomb submitLLJob "Could not transfer LL job file ($err)"
    }

    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Job file written:"]] [cgi_sample $remFile]"
    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Remote host:"]] [cgi_sample $remHost]"
    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Remote user:"]] [cgi_sample $remUser]"
    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Job File Submitted:"]]"
    if [catch {webSubmit::util::tclCat $jname} result] {
	cgiWarn submitLLJob "$result"
    } else {
	file delete -force $jname
	cgi_preformatted {
	    foreach line [split $result \n] {
		puts [cgi_quote_html $line]
	    }
	}
    }
    
    #  Submit job file
    cgi_hr
    set subCmd $PATHS($remHost,llsubmit)
    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Submitting Job:"]] [cgi_bold "llsubmit $remFile"]"
    if [catch {webSubmit::network::sshCmdExec $remUser $remHost "$subCmd $remFile"} result] {
 	cgiBomb submitLLJob "Could not submit Load Leveler Job ($result)"
    } else {
	# Extract job Id
	if {$debug} {
	    cgi_p "[cgi_font color=red [cgi_bold "Result was \"$result\""]]"
	}
	regexp {[^.]+\.([0-9]+).*} $result match jid
	if [info exists jid] {
	    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Load Leveler Job ID:"]]\
		[cgi_font color=blue [cgi_bold [cgi_sample "$jid"]]]"
	    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Load Leveler Job Name:"]]\
		[cgi_font color=blue [cgi_bold [cgi_sample "$job_name"]]]"
	    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Output file:"]][cgi_sample $output]"
	    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Error file:"]] [cgi_sample $error]"
	} else {
	    cgiBomb submitLLJob "Could not submit Load Leveler Job ($result)"
	}
	cgi_br

 	cgi_preformatted {
 	    foreach line [split $result \n] {
 		puts [cgi_quote_html $line]
 	    }
	}
    }
    return 1
}

# CODEINFO BEGIN ####################################################
# info proc checkGaussianProc> Procedure to construct a consistent number
#     of processors for a Gaussian input file.  Number of processors 
#     is checked against current class and redfined to max_processors
#     if inconsistent.  If the input file is specified as the location
#     for the number of processors, but no specification is given,
#     then the file is left untouched.

# arg gaussianFile> Gaussian input file name
# return> None
proc webSubmit::loadleveler::checkGaussianProc {gaussianFile} {
    variable Gaussian
    variable CLASS
    variable class
    variable debug

    # String used by Gaussian to define the number of processors
    set pstr "\%NProc="
    
    if {$debug} {
	puts "Checking Gaussian processors: $gaussianFile"
	puts "  Processor specification: $Gaussian(NProcType)"
	puts "  Number of processors: $Gaussian(Nproc)"
    }

    # Determine if number of processors specified in input file
    set reset 0
    if [file exists $gaussianFile] {
	if [catch {tclGrep $pstr $gaussianFile} procLine] {
	    # No processors specified
	    if {$Gaussian(NProcType) == "IF"} {
		# Allow Gaussian to determine default number of processors
		return
	    } else {
		if ![info exists Gaussian(Nproc)] {
		    # Fix at maximum number for the current class
		    set Gaussian(Nproc) $CLASS($class,max_processors)
		}
	    }
	} else {
	    if {$Gaussian(NProcType) == "IF"} {
		# Extract Nproc from input file
		regexp "^${pstr}\(.*\)" $procLine match Gaussian(Nproc)
	    }
	}
    } else {
	cgiBomb checkGaussianProc "Specified Gaussian input file does not exist"
    }

    if {$debug} {
	puts "  Modified number of processors: $Gaussian(Nproc)"
    }

    # Determine if processors in allowed range for current class
    if {$Gaussian(Nproc) < 1 || $Gaussian(Nproc) > $CLASS($class,max_processors)} {
	cgiWarn checkGaussianProc "Error determining number of processors.  Reset\
		to maximum for class $class ($CLASS($class,max_processors))"
	set Gaussian(Nproc) $CLASS($class,max_processors)
	set reset 1
    }

    if {$reset || $Gaussian(NProcType) == "UD"} {
	# Open Gaussian input file and determine number of processors
	if [catch {open "$gaussianFile" r} fid] {
	    cgiBomb checkGaussianProc "Could not open Gaussian input file ($fid)"
	} else {
	    # Read input file
	    set gaussianDataList [split [read $fid] \n]
	    set idx [lsearch -regexp $gaussianDataList $pstr]
	    if {$idx != -1} {
		# Input file already contains %Nproc --> Replace
		set gaussianDataList [lreplace $gaussianDataList $idx $idx "${pstr}$Gaussian(Nproc)"]
	    } else {
		# Input file does not specify %Nproc --> Insert at top of file
		if {$Gaussian(NProcType) == "UD"} {
		    set gaussianDataList [linsert $gaussianDataList 0 "${pstr}$Gaussian(Nproc)"]
		}
	    }
	    close $fid

	    # Rewrite Gaussian input file with correct number of processors
	    set fid [open $gaussianFile w]
	    foreach line $gaussianDataList {
		puts $fid $line
	    }
	    close $fid
	}
    }
}

# CODEINFO BEGIN ##########################################################
# info proc submitGaussianJob> Procedure to write and submit a Gaussian Load
#     Leveler job

# arg remHost> Load Leveler host
# arg remUser> Remote user
# arg version> Index of Gaussian version
#         0 => Gaussian94
#         1 => Gaussian98
# arg clobberGaussian> Overwrite flag for Gaussian file
# arg tmpGaussianFile> Name of local Gaussian input file
# arg remGaussianFile> Name of remote Gaussian input file
# return> None
proc webSubmit::loadleveler::submitGaussianJob {remHost remUser version clobberGaussian tmpGaussianFile remGaussianFile} {
    variable LLREQ
    variable Gaussian
    variable PATHS
    variable dbKeyList
    variable jobScript
    foreach keyword $dbKeyList {
	variable $keyword
    }
    
    # Compare number of processors specified (if any) with class
    checkGaussianProc $tmpGaussianFile

    # Set up Load Leveler Variables
    if {$Gaussian(OutputType) == "UD" && $Gaussian(Output) != ""} {
	set f1 [file tail $Gaussian(Output)]
	set output "$(initialdir)/$f1\_$(jobid).out"
	set error  "$(initialdir)/$f1\_$(jobid).err"
    } elseif {$Gaussian(OutputType) == "IF"} {
	set f1 [file tail $remGaussianFile]
	set output "$(initialdir)/$f1\_$(jobid).out"
	set error  "$(initialdir)/$f1\_$(jobid).err"
    } else {
	set output "$(initialdir)/$(jobid).out"
	set error  "$(initialdir)/$(jobid).err"
    }
    
    set shell "/bin/csh"
    set LLREQ(Adapter) hps_user
    set requirements [buildLLBoolean LLREQ]
    set max_processors $Gaussian(Nproc)
    set min_processors $Gaussian(Nproc)
    if {$max_processors > 1} {
	set job_type parallel
    }
    set notification always

# Use version information to select proper version of Gaussian

    switch $version {

	0 {
	    # Version 0 = Gaussian94
	    set jobScript "$PATHS($remHost,g94) < [file tail $remGaussianFile]"
	}

	1 {
	    # Version 1 = Gaussian98
	    set jobScript "$PATHS($remHost,g98) < [file tail $remGaussianFile]"
	}

	default {
	    cgiBomb submitGaussianJob "Unknown version of Gaussian with index $version."
	}
    }

    # Create job file names
    set locJobFile "/tmp/${job_name}_[pid]"
    set remJobFile [file join $initialdir [file tail $locJobFile]]

    # Debugging
#     puts "Temp Gaussian File: $tmpGaussianFile"
#     puts "Remote Gaussian File: $remGaussianFile"
#     puts "Local job file: $locJobFile"
#     puts "Remote job file: $remJobFile"
#     foreach keyword $dbKeyList {
# 	set value [set $keyword]
# 	if {$value != ""} {
# 	    puts "$keyword = $value"
# 	}
#     }
#     puts "Job script: $jobScript"

    # Write Job File
    if [catch {open "$locJobFile" w 0600} fid] {
	cgiBomb submitGaussianJob "Could not open Gaussian job file ($fid)"
    } else {
	#  write job file
	writeJobFile $fid
	close $fid 
	cgi_p "[cgi_font color=darkmagenta [cgi_bold "Load Leveler Job File:"]]"
	set txt [tclCat $locJobFile]
	cgi_preformatted {
	    foreach line [split $txt \n] {
		puts "[cgi_sample $line]"
	    }
	}
	cgi_br
    }
    
    #  Upload Gaussian file and LL file, then submit job
    if [catch {
	webSubmit::network::scpFile $tmpGaussianFile put $clobberGaussian $remUser $remHost $remGaussianFile
	webSubmit::network::scpFile $locJobFile put 1 $remUser $remHost $remJobFile} result] {
	    cgiBomb submitGaussianJob "Problem uploading input files ($result)"
    } else {
	# Delete Local job file
	file delete $locJobFile

	# Delete local Gaussian directives file
	file delete $tmpGaussianFile
	
	# Submit job
	set subCmd $PATHS($remHost,llsubmit)
	if [catch {webSubmit::network::sshCmdExec $remUser $remHost "$subCmd $remJobFile"} result] {
	    cgiBomb wsLoadLeveler.tcl "Could not submit Gaussian Job ($result)"
	} else {
	    # Extract job Id
	    regexp {[^.]+\.([0-9]+).*} $result match jid
	    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Load Leveler Job ID:"]][cgi_sample "$jid"]"
	    foreach var [list output error] {
		regsub {\$\(initialdir\)} [set $var] "$initialdir" $var
		regsub {\$\(jobid\)} [set $var] "$jid" $var
		regsub {\$\(job_name\)} [set $var] "$job_name" $var
	    }
	    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Output file:"]][cgi_sample $output]"
	    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Error file:"]] [cgi_sample $error]"
	    cgi_br
	    
	    cgi_preformatted {
		foreach line [split $result \n] {
		    puts [cgi_quote_html $line]
		}
	    }
	} 
    }
}


# CODEINFO BEGIN #######################################################################
# info proc buildPoeScript> Procedure to build up a script for POE that copies input 
#     files to local nodes, executes program, and then returns output files to a 
#     specified host

# arg remUser> Remote user login name
# arg remHost> Remote host
# arg userExec> User Executable
# arg userArgs> Arguments to user executable
# arg stdin> Standard input
# arg cpOut> Flag to determine whether output is copied to return host
# arg returnDir> Directory in which output copied
# return> None
proc webSubmit::loadleveler::buildPoeScript {remUser remHost userExec userArgs stdin cpOut returnDir} {
    foreach var [list shell initialdir PATHS scratchRoot jobScript inputFileList outputFileList] {
	variable $var
    }
    set poeBin $PATHS($remHost,poe)
    set returnHost $remHost

    # Set LL shell to ksh for poe script interpretation
    set shell "/bin/ksh"

    # Make sure standard input is in the input file list
    if {$stdin != ""} {
	set inDir [file dirname $stdin]
	set name [file tail $stdin]
	if {$inDir == "$initialdir"} {
	    if {[lsearch -exact $inputFileList $name] == -1 && [lsearch -exact $inputFileList $stdin] == -1} {
		lappend inputFileList $name
	    }
	} else {
	    if {[lsearch -exact $inputFileList $stdin] == -1} {
		lappend inputFileList $stdin
	    }
	}
    }

    # Create job script
    catch {unset jobScript}
    append jobScript "user=$remUser\n"
    append jobScript "workDir=\"[file join $scratchRoot $remUser]\"\n"

    append jobScript "\n# Files to copy to and from local nodes\n"
    append jobScript "inFiles=\"$inputFileList\"\n"
    append jobScript "outFiles=\"$outputFileList\"\n"

    append jobScript "\n# Program to execute\n"
    append jobScript "program=\"$userExec\"\n"
    set progName [file tail $userExec]
    append jobScript "\n# Variables to export\n"
    append jobScript {export CMD="${0##*/}"}
    append jobScript "\n"
    append jobScript "export PROGRAM=\"\$\{workDir\}/$progName\"\n"
    append jobScript "export HOST=`hostname`\n"
    append jobScript {HOST=${HOST%%.*}}
    append jobScript "\n"
    append jobScript {echo "Running $CMD on node $HOST"}
    append jobScript "\n"
    append jobScript {echo "PROGRAM NAME = $PROGRAM"}
    append jobScript "\n"
    append jobScript "\n# Copy executable and input files (if any) to all allocated nodes\n"
    append jobScript {for poehost in $LOADL_PROCESSOR_LIST}
    append jobScript "\n"
    append jobScript "do\n"
    append jobScript {  rsh ${poehost} mkdir $workDir > /dev/null 2>&1}
    append jobScript "\n"
    append jobScript {  rcp ${inFiles} ${poehost}:$workDir}
    append jobScript "\n"
    append jobScript {  echo "rcp ${inFiles} ${poehost}:$workDir"}
    append jobScript "\n"
    append jobScript {  rcp ${program} ${poehost}:$workDir}
    append jobScript "\n"
    append jobScript {  echo "rcp ${program} ${poehost}:$workDir"}
    append jobScript "\n"
    append jobScript "done\n"

    append jobScript "\n# Execute the program\n"
    append jobScript {if [ -x $PROGRAM ]}
    append jobScript "\n"
    append jobScript "then\n"
    append jobScript {  echo "$CMD: Starting user program $PROGRAM"}
    append jobScript "\n"
    append jobScript {  cd ${workDir}}
    append jobScript "\n"
    set progName [file tail $userExec]
    if {$userArgs != ""} {
	if {$stdin != ""} {
	    # Strip path from standard input file
	    set stdin [file tail $stdin]
	    append jobScript "  \($poeBin [file join $scratchRoot $remUser $progName] $userArgs < $stdin\)\n"
	} else {
	    append jobScript "  \($poeBin [file join $scratchRoot $remUser $progName] $userArgs\)\n"
	} 
    } else {
	append jobScript "  \($poeBin [file join $scratchRoot $remUser $progName]\)\n"
    }
    append jobScript "else\n"
    append jobScript {  print -u2 "$CMD: error - $PROGRAM not found"}
    append jobScript "\n"
    append jobScript "fi\n"

    if {$cpOut} {
	append jobScript "\n# Copy output files back to $returnHost\n"
	append jobScript {for poehost in $LOADL_PROCESSOR_LIST}
	append jobScript "\n"
	append jobScript "do\n"
	append jobScript {  tmpFiles=`rsh ${poehost} "cd $workDir;ls ${outFiles}" 2>/dev/null`}
	append jobScript "\n"
	append jobScript {  for outfile in ${tmpFiles}}
	append jobScript "\n"
	append jobScript "    do\n"
	append jobScript "       echo rcp \$\{poehost\}:\$workDir/\$\{outfile\} $returnHost:[file join $returnDir \$\{outfile\}.\$\{poehost\}]\n"
	append jobScript "       rcp \$\{poehost\}:\$workDir/\$\{outfile\} $returnHost:[file join $returnDir \$\{outfile\}.\$\{poehost\}]\n"
	append jobScript "    done\n"
	append jobScript "done\n"

	append jobScript "\n#Clean up tmp directory on local nodes\n"
	append jobScript {for poehost in $LOADL_PROCESSOR_LIST}
	append jobScript "\n"
	append jobScript "do\n"
	append jobScript "  rsh \$\{poehost\} \"cd \$workDir; rm -f \$\{inFiles\} $progName\"\n"
	append jobScript "  echo \"rsh \$\{poehost\} \(cd \$workDir; rm -f \$\{inFiles\} $progName\)\"\n"
	append jobScript "\n"
	append jobScript {  rsh ${poehost} "cd $workDir; rm -f ${outFiles}"}
	append jobScript "\n"
	append jobScript {  echo "rsh ${poehost} (cd $workDir; rm -f ${outFiles})"}
	append jobScript "\n"
	append jobScript "done\n"
	append jobScript "\nexit 0"
    }
}


# CODEINFO BEGIN ########################################################
# info proc buildLLBoolean> Procedure to build up Load Leveler requirements
#     and preferences Boolean strings

# arg arrName> Name of array containing Boolean data
# return> Boolean string expression used in Load Leveler

proc webSubmit::loadleveler::buildLLBoolean {arrName} {
    variable $arrName
    upvar 0 $arrName LLARR

    set boolString ""
    foreach var [array names LLARR] {
	if {$LLARR($var) != ""} {
	    if {$var == "Memory"} {
		set str "($var == $LLARR($var)) && "
	    } else {
		set str "($var == \"$LLARR($var)\") && "
	    }
	    append boolString $str
	}
    }
    return [string trimright $boolString "& "]
}












































































