#!/itl/links/generic/bin/tclsh8.0
# WebSubmit 2.0 wsNQS.tcl @(#)wsNQS.tcl	1.23 SCCS/s.wsNQS.tcl  99/07/22 

# AUTHORS #############################################################

#  Authors: John Koontz, Ryan McCormack, Judith Devaney
#  Original Development Work: Robert Lipman, John Koontz, Katherine
#      Pagoaga

# Copyright Information ###############################################

# This software was developed at the National Institute of Standards
# and Technology by employees of the Federal Government in the course
# of their official duties.  Pursuant to title 17 Section 105 of the
# United States Code this software is not subject to copyright
# protection and is in the public domain.

# CODEINFO BEGIN ######################################################

# info script wsNQS.tcl>  Support utilities for NQS batch processing.

# CODEINFO END ########################################################

# Package definitions
package provide wsNQS 1.0

namespace eval webSubmit::nqs {
    # Namespace variables
    variable thisNS [namespace current]
    variable wsNS [namespace parent]
    variable upNS [namespace parent]
    variable debug 0

    # Procedures to be imported
    namespace import ${wsNS}::database::*
    namespace import ${wsNS}::util::*

    # Procedures to be exported
    namespace export submitNQSjob
    namespace export parseQstatInfo
    namespace export checkLimits
    namespace export submitGaussianJob

    # Variable aliases
    alias $wsNS wsRootDir

    # Host-specific path information
    variable PATHS() {}

    ;;set PATHS(amur.nist.gov,qdel) "/usr/craysoft/nqe/bin/qdel"
    ;;set PATHS(amur.nist.gov,qstat) "/usr/craysoft/nqe/bin/qstat"
    ;;set PATHS(amur.nist.gov,qsub) "/usr/craysoft/nqe/bin/qsub"
    ;;set PATHS(amur.nist.gov,g94) "/usr/local/apps/g94/g94/g94"
    ;;set PATHS(amur.nist.gov,g98) "/usr/local/apps/g98/g98/g98"

    ;;set PATHS(arno.nist.gov,qdel) "/usr/craysoft/nqe/bin/qdel"
    ;;set PATHS(arno.nist.gov,qstat) "/usr/craysoft/nqe/bin/qstat"
    ;;set PATHS(arno.nist.gov,qsub) "/usr/craysoft/nqe/bin/qsub"
    ;;set PATHS(arno.nist.gov,g94) "/usr/local/apps/g94/g94/g94"
    ;;set PATHS(arno.nist.gov,g98) "/usr/local/apps/g98/g98/g98"

    ;;set PATHS(neva.nist.gov,qdel) "/usr/craysoft/nqe/bin/qdel"
    ;;set PATHS(neva.nist.gov,qstat) "/usr/craysoft/nqe/bin/qstat"
    ;;set PATHS(neva.nist.gov,qsub) "/usr/craysoft/nqe/bin/qsub"
    ;;set PATHS(neva.nist.gov,g94) "/usr/local/apps/g94/g94/g94"
    ;;set PATHS(neva.nist.gov,g98) "/usr/local/apps/g98/g98/g98"

    ;;set PATHS(tay.nist.gov,qdel) "/usr/craysoft/nqe/bin/qdel"
    ;;set PATHS(tay.nist.gov,qstat) "/usr/craysoft/nqe/bin/qstat"
    ;;set PATHS(tay.nist.gov,qsub) "/usr/craysoft/nqe/bin/qsub"
    ;;set PATHS(tay.nist.gov,g94) "/usr/local/apps/g94/g94/g94"
    ;;set PATHS(tay.nist.gov,g98) "/usr/local/apps/g98/g98/g98"

    # Host-specific configuration information
    variable CONFIG() {}
    ;;set CONFIG(hostList) [list amur.nist.gov arno.nist.gov neva.nist.gov tay.nist.gov]

    ;;set CONFIG(amur.nist.gov,maxProcessors) 12
    ;;set CONFIG(amur.nist.gov,maxMemory,Gb) 12
    ;;set CONFIG(amur.nist.gov,maxCPUTime) 432000

    ;;set CONFIG(arno.nist.gov,maxProcessors) 4
    ;;set CONFIG(arno.nist.gov,maxMemory,Gb) 4
    ;;set CONFIG(arno.nist.gov,maxCPUTime) 180060

    ;;set CONFIG(neva.nist.gov,maxProcessors) 12
    ;;set CONFIG(neva.nist.gov,maxMemory,Gb) 12
    ;;set CONFIG(neva.nist.gov,maxCPUTime) 432000

    ;;set CONFIG(tay.nist.gov,maxProcessors) 12
    ;;set CONFIG(tay.nist.gov,maxMemory,Gb) 12
    ;;set CONFIG(tay.nist.gov,maxCPUTime) 432000

    foreach host $CONFIG(hostList) {
	foreach {s1 s2} {Mb Gb Kb Mb b Kb} {
	    set CONFIG($host,maxMemory,$s1) [expr $CONFIG($host,maxMemory,$s2) * 1024]
	}
	foreach {s1 s2} {Gw Gb Mw Mb Kw Kb w b} {
	    set CONFIG($host,maxMemory,$s1) [expr $CONFIG($host,maxMemory,$s2) / 8]
	}
    }

    # General QSUB information
    variable qsubOptList [list a A C e j J l,mpp_p la lc ld lf lF lm lM ln\
	    lQ ls lt lT lw mu o p q r s u X]
    variable qsubFlagList [list eo ke kj ko mb me mr mt nc nr re Rf ro x z]
    foreach flag $qsubFlagList {
	set QSUBFLAGS(-$flag) "off"
    }
    foreach opt $qsubOptList {
	set QSUBOPTS(-$opt) ""
    }
    variable workDir
    variable jobScript
    variable jobFileName

    # General QSTAT information
    variable QSTATINFO() {}
    variable QSTAT() {}
    variable majorStatList [list A C D E H Q P R S U W]
    variable minorStatList [list ce cg cm cq cr cu du ge gg gm gq gr gt gu lm md mp\
	    nu op qe qg qm qq qr qs qu rj sh td us xs xw "??"]
    set QSTAT(-a,header) [list IDENTIFIER NAME USER LOCATION/QUEUE JID PRTY REQMEM REQTIM ST]
    set QSTAT(IDENTIFIER,text) "Request Identifier"
    set QSTAT(NAME,text) "Job Name"
    set QSTAT(USER,text) "User Name"
    set QSTAT(LOCATION/QUEUE,text) "Queue"
    set QSTAT(JID,text) "Job Identifier"
    set QSTAT(PRTY,text) "Priority"
    set QSTAT(REQMEM,text) "Requested Memory (kB)"
    set QSTAT(REQTIM,text) "Requested CPU Time (sec)"
    set QSTAT(ST,text) "Status"
    set QSTAT(IDENTIFIER,show) 0
    set QSTAT(NAME,show) 1
    set QSTAT(USER,show) 0
    set QSTAT(LOCATION/QUEUE,show) 1
    set QSTAT(JID,show) 0
    set QSTAT(PRTY,show) 1
    set QSTAT(REQMEM,show) 1
    set QSTAT(REQTIM,show) 1
    set QSTAT(ST,show) 1
    set QSTAT(ST,major,A) "Arriving"
    set QSTAT(ST,major,C) "Checkpointed"
    set QSTAT(ST,major,D) "Departing"
    set QSTAT(ST,major,E) "Exiting"
    set QSTAT(ST,major,H) "Held"
    set QSTAT(ST,major,Q) "Queued"
    set QSTAT(ST,major,P) "Preempted"
    set QSTAT(ST,major,R) "Running"
    set QSTAT(ST,major,S) "Suspended"
    set QSTAT(ST,major,U) "Unknown"
    set QSTAT(ST,major,W) "Waiting"
    set QSTAT(ST,minor,ce) "Complex MPP processing limit reached"
    set QSTAT(ST,minor,cg) "Complex group run limit reached"
    set QSTAT(ST,minor,cm) "Complex memory limit reached"
    set QSTAT(ST,minor,cq) "Complex quickfile limit reached"
    set QSTAT(ST,minor,cr) "Complex run limit reached"
    set QSTAT(ST,minor,cu) "Complex user run limit reached"
    set QSTAT(ST,minor,du) "Destination queue unavailable"
    set QSTAT(ST,minor,ge) "Global MPP processing limit reached"
    set QSTAT(ST,minor,gg) "Global group run limit reached"
    set QSTAT(ST,minor,gm) "Global memory limit reached"
    set QSTAT(ST,minor,gq) "Global quickfile limit reached"
    set QSTAT(ST,minor,gr) "Global run limit reached"
    set QSTAT(ST,minor,gt) "Global tape drive limit reached"
    set QSTAT(ST,minor,gu) "Global user run limit reached"
    set QSTAT(ST,minor,lm) "License unavailable for request"
    set QSTAT(ST,minor,md) "Cray T3D device not accessible"
    set QSTAT(ST,minor,mp) "Insufficient MPP processing elements"
    set QSTAT(ST,minor,nu) "Network Load Balancer unavailable"
    set QSTAT(ST,minor,op) "Major status set through operator action"
    set QSTAT(ST,minor,qe) "Queue MPP processing element limit reached"
    set QSTAT(ST,minor,qg) "Queue group run limit reached"
    set QSTAT(ST,minor,qm) "Queue memory limit reached"
    set QSTAT(ST,minor,qq) "Queue quickfile limit reached"
    set QSTAT(ST,minor,qr) "Queue run limit reached"
    set QSTAT(ST,minor,qs) "Requested queue is stopped"
    set QSTAT(ST,minor,qu) "Queue user run limit reached"
    set QSTAT(ST,minor,rj) "Request rejected by Unified Resource Manager"
    set QSTAT(ST,minor,sh) "System shut down"
    set QSTAT(ST,minor,td) "UNICOS tape daemon unavailable"
    set QSTAT(ST,minor,us) "Request in Unified Resource manager scheduling pool"
    set QSTAT(ST,minor,xs) "Request sent to Miser scheduler"
    set QSTAT(ST,minor,xw) "Request waiting to be scheduled by Miser scheduler"
    set QSTAT(ST,minor,??) "Unknown"

    # Gaussian Information
    variable Gaussian() {}
}

# CODEINFO BEGIN ############################################################
# info proc writeNQSjob> Procedure to write out an NQS job file
# arg fid> File descriptor for job control file
# return> None
proc webSubmit::nqs::writeNQSjob {fid} {
    variable qsubFlagList
    variable qsubOptList
    variable QSUBFLAGS
    variable QSUBOPTS
    variable jobScript

    # Write out qsub flags
    foreach flag $qsubFlagList {
	if {$QSUBFLAGS(-$flag) != "off"} {
	    puts $fid "# QSUB -$flag"
	}
    }

    # Write out qsub options
    foreach opt $qsubOptList {
	if {$QSUBOPTS(-$opt) != ""} {
	    if [regexp {([^,]+),(.*)} $opt match opt1 opt2] {
		puts $fid "# QSUB -$opt1 ${opt2}=$QSUBOPTS(-$opt)"
	    } else {
		puts $fid "# QSUB -$opt $QSUBOPTS(-$opt)"
	    }
	}
    }

    # Write out job script
    if {$jobScript != ""} {
	puts $fid "$jobScript"
    }

    return
}


# CODEINFO BEGIN ############################################################
# info proc submitNQSjob> Procedure to submit an NQS job

proc webSubmit::nqs::submitNQSjob {remHost remHome remUser} {
    variable jobFileName
    variable workDir
    variable PATHS

    set pid [pid]
    # Resolve job file name
    #  i)  None given: jobFileName --> $workDir/job.NQS.$pid
    #  ii) Provided:   jobFileName --> $workDir/$jobFileName.NQS (avoids overwriting executable
    #                    if user specifies a jobFileName equivalent to the executable
    if {$jobFileName == ""} {
	set jobFileName [file join $workDir "job.NQS.$pid"]
	cgiWarn submitNQSjob "No name provided for job control script - set to <B>$jobFileName</B>"
    } else {
	if {[file extension $jobFileName] != ".NQS"} {
	    set jobFileName [mkPathAbs "$jobFileName.NQS" $workDir]
	} else {
	    set jobFileName [mkPathAbs $jobFileName $workDir]
	}
    }

    # Write job control script to temporary file
    set tmpFile [file join /tmp tmp.nqsjob.$pid]
    if [catch {open $tmpFile w} fid] {
	cgiBomb submitNQSjob "Could not write temporary job control script"
    }
    writeNQSjob $fid
    close $fid

    # Copy job control file to remote host - Overwrite if necessary
    if [catch {webSubmit::network::scpFile $tmpFile put 1 $remUser $remHost $jobFileName} err] {
	cgiBomb submitNQSjob "Could not transfer NQS job file ($err)"
    }

    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Job file written:"]] [cgi_sample $jobFileName]"
    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Remote host:"]] [cgi_sample $remHost]"
    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Remote user:"]] [cgi_sample $remUser]"
    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Job File Submitted:"]]"
    if [catch {webSubmit::util::tclCat $tmpFile} result] {
	cgiWarn submitNQSjob "$result"
    } else {
	file delete -force $tmpFile
	cgi_preformatted {
	    foreach line [split $result \n] {
		puts [cgi_quote_html $line]
	    }
	}
    }

    #  Submit job file
    cgi_hr
    set qsubCmd $PATHS($remHost,qsub)
    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Submitting Job:"]] [cgi_bold "$qsubCmd $jobFileName"]"
    if [catch {webSubmit::network::sshCmdExec $remUser $remHost "$qsubCmd $jobFileName"} result] {
 	cgiBomb submitNQSJob "Could not submit NQS Batch Job ($result)"
    } else {
	# Extract job Id
	set idLine [lindex [split $result \n] 1]
	regexp { *Request * *<([0-9]+).*} $idLine match jid
	cgi_p "[cgi_font color=darkmagenta [cgi_bold "NQS Job ID:"]][cgi_font color=blue [cgi_bold [cgi_sample "$jid"]]]"
	cgi_br

 	cgi_preformatted {
 	    foreach line [split $result \n] {
 		puts [cgi_quote_html $line]
 	    }
	}
    }
    return 0
}


# CODEINFO BEGIN #################################################################
# info proc parseQstatInfo> Procedure to parse and store the output from qstat
# arg infoList> Data from qstat to be parsed (already split)
# arg flag> qstat flag used to generate data
# return> None
proc webSubmit::nqs::parseQstatInfo {infoList flag} {
    variable QSTATINFO
    variable QSTAT

    # Parse qstat information based on flag passed
    set QSTATINFO(jobIdList) ""
    if {$flag == "-a"} {
	# Summary job information for user
	set headerLength [llength $QSTAT(-a,header)]
	foreach line $infoList {
	    if [regexp {^[0-9]} $line] {
		# User job data
		set dataList [compSplit $line]
		set dataLength [llength $dataList]

		# Extract sequence_number for request
		regexp {^([0-9]+)\..*} [lindex $dataList 0] match jobId
		lappend QSTATINFO(jobIdList) $jobId
		set QSTATINFO($jobId,[lindex $QSTAT(-a,header) 0]) [lindex $dataList 0]
		
		# Match fields with header
		if {$dataLength == $headerLength} {
		    for {set i 1} {$i < $dataLength} {incr i} {
			set field [lindex $QSTAT(-a,header) $i]
			set QSTATINFO($jobId,$field) [lindex $dataList $i]
		    }
		} else {
		    # Assume JID field is empty - Kludge
		    set QSTATINFO($jobId,JID) ""
		    set jidIdx [lsearch -exact $QSTAT(-a,header) JID]
		    for {set i 1} {$i < $headerLength} {incr i} {
			if {$i < $jidIdx} {
			    set field [lindex $QSTAT(-a,header) $i]
			    set QSTATINFO($jobId,$field) [lindex $dataList $i]
			} elseif {$i > $jidIdx} {
			    set field [lindex $QSTAT(-a,header) $i]
			    set QSTATINFO($jobId,$field) [lindex $dataList [expr $i - 1]]
			}
		    }
		}
	    }
	}
    }
    return
}


# CODEINFO BEGIN ##################################################################
# info proc checkLimits> Procedure to check requested resources against queue
#     limits for a given NQS host.  If the request exceeds limits, a warning is
#     raised.

# arg remHost> NQS Host
# arg resource> Resource being requested
# arg request> Value of resource being requested
# arg units> Optional units for the request
# return> Warning if request exceeds queue limits

proc webSubmit::nqs::checkLimits {remHost resource request {units ""}} {
    variable CONFIG

    # Verify that limits are available for host and that a finite request has been made
    if {[lsearch -exact $CONFIG(hostList) $remHost] == -1} {
	cgiWarn checkLimits "Resource requests cannot be verified for host $remHost"
	return
    } elseif {$request == ""} {
	return
    }

    # If resource is maxCPUTime, convert request to seconds for comparison
    if {$resource == "maxCPUTime"} {
	if [regexp {([0-9]+):([0-9][0-9]):([0-9][0-9]).*} $request match hh mm ss] {
	    set request [expr ($hh * 3600) + ($mm * 60) + $ss]
	} elseif [regexp {([0-9]?[0-9]):([0-9][0-9]).*} $request match mm ss] {
	    set request [expr ($mm * 60) + $ss]
	}
    }

    # Check request against limits for resource
    if {$units == ""} {
	if [info exists CONFIG($remHost,$resource)] {
	    if [expr $request > $CONFIG($remHost,$resource)] {
		cgiWarn checkLimits "Request exceeds $resource (Max = $CONFIG($remHost,$resource), Req = $request)"
	    }
	} else {
	    cgiWarn checkLimits "No limits available for resource $resource on host $remHost"
	}
    } else {
	if [info exists CONFIG($remHost,$resource,$units)] {
	    if [expr $request > $CONFIG($remHost,$resource,$units)] {
		cgiWarn checkLimits "Request exceeds $resource (Max = $CONFIG($remHost,$resource,$units) $units,\
			Req = $request $units)"
	    }
	} else {
	    cgiWarn checkLimits "No limits available for resource $resource ($units) on host $remHost"
	}
    }
    return
}


# CODEINFO BEGIN ##########################################################
# info proc submitGaussianJob> Procedure to write and submit a Gaussian NQS
#     job

# arg remDir> Remote working directory
# arg remHost> NQS host
# arg remUser> Remote user
# arg version> Index of Gaussian version
#         0 => Gaussian94
#         1 => Gaussian98
# arg clobberGaussian> Overwrite flag for Gaussian file
# arg tmpGaussianFile> Name of local Gaussian input file
# arg remGaussianFile> Name of remote Gaussian input file
# return> None
proc webSubmit::nqs::submitGaussianJob {remDir remHost remUser version clobberGaussian tmpGaussianFile remGaussianFile} {
    variable QSUBOPTS
    variable Gaussian
    variable PATHS
    variable jobScript
    variable debug
    
    if {$debug} {
	puts "Submitting Gaussian Job..."
	puts "  Remote directory: $remDir"
	puts "  Remote Host: $remHost"
	puts "  Remote User: $remUser"
	puts "  Clobber? $clobberGaussian"
	puts "  Temp Gaussian File: $tmpGaussianFile ([file exists $tmpGaussianFile])"
	puts "  Remote Gaussian File: $remGaussianFile"
    }

    # Compare number of processors specified (if any) with
    #   System processor limits
    checkGaussianProc $remHost $tmpGaussianFile

    # Set up Output and Error filenames
    if {$Gaussian(OutputType) == "UD" && $QSUBOPTS(-o) != ""} {
	set f1 [file tail $QSUBOPTS(-o)]
	set QSUBOPTS(-o) [file join $remDir $f1.out]
	set QSUBOPTS(-e)  [file join $remDir $f1.err]
    } elseif {$Gaussian(OutputType) == "IF"} {
	set f1 [file tail $remGaussianFile]
	set QSUBOPTS(-o) [file join $remDir $f1.out]
	set QSUBOPTS(-e)  [file join $remDir $f1.err]
    } else {
	set QSUBOPTS(-o) [file join $remDir $QSUBOPTS(-r).out]
	set QSUBOPTS(-e)  [file join $remDir $QSUBOPTS(-r).err]
    }

# Use version information to select proper version of Gaussian

    switch $version {

	0 {
	    # Version 0 = Gaussian94
	    set jobScript "$PATHS($remHost,g94) < $remGaussianFile"
	}

	1 {
	    # Version 1 = Gaussian98
	    set jobScript "$PATHS($remHost,g98) < $remGaussianFile"
	}

	default {
	    cgiBomb submitGaussianJob "Unknown version of Gaussian with index $version."
	}
    }

    # Create job file names
    set locJobFile "/tmp/$QSUBOPTS(-r)_[pid]"
    set remJobFile [file join $remDir [file tail $locJobFile]]

    # Debugging
    if {$debug} {
	puts "Temp Gaussian File: $tmpGaussianFile"
	puts "Remote Gaussian File: $remGaussianFile"
	puts "Local job file: $locJobFile"
	puts "Remote job file: $remJobFile"
	puts "Output File: $QSUBOPTS(-o)"
	puts "Error File: $QSUBOPTS(-e)"
	puts "Job script: $jobScript"
    }

    # Write Job File
    if [catch {open "$locJobFile" w 0600} fid] {
	cgiBomb submitGaussianJob "Could not open Gaussian job file ($fid)"
    } else {
	#  write job file
	writeNQSjob $fid
	close $fid 
	cgi_p "[cgi_font color=darkmagenta [cgi_bold "NQS Job File:"]]"
	set txt [tclCat $locJobFile]
	cgi_preformatted {
	    foreach line [split $txt \n] {
		puts "[cgi_sample $line]"
	    }
	}
	cgi_br
    }
    
    #  Upload Gaussian file and NQS file, then submit job
    if [catch {
	webSubmit::network::scpFile $tmpGaussianFile put $clobberGaussian $remUser $remHost $remGaussianFile
	webSubmit::network::scpFile $locJobFile put 1 $remUser $remHost $remJobFile} result] {
	    cgiBomb submitGaussianJob "Problem uploading input files ($result)"
    } else {
	# Delete Local job file
	file delete $locJobFile

	# Delete local Gaussian directives file
	file delete $tmpGaussianFile
	
	# Submit job
	set subCmd $PATHS($remHost,qsub)
	if [catch {webSubmit::network::sshCmdExec $remUser $remHost "$subCmd $remJobFile"} result] {
	    cgiBomb wsNQS.tcl "Could not submit Gaussian Job ($result)"
	} else {
	    # Extract job Id
	    set idLine [lindex [split $result \n] 1]
	    regexp { *Request * *<([0-9]+).*} $idLine match jid
	    cgi_p "[cgi_font color=darkmagenta [cgi_bold "NQS Job ID:"]][cgi_font color=blue [cgi_bold [cgi_sample "$jid"]]]"
	    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Job file:"]][cgi_sample $remJobFile]"
	    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Output file:"]][cgi_sample $QSUBOPTS(-o)]"
	    cgi_p "[cgi_font color=darkmagenta [cgi_bold "Error file:"]] [cgi_sample $QSUBOPTS(-e)]"
	    cgi_br
	    
	    cgi_preformatted {
		foreach line [split $result \n] {
		    puts [cgi_quote_html $line]
		}
	    }
	} 
    }
}


# CODEINFO BEGIN ####################################################
# info proc checkGaussianProc> Procedure to construct a consistent number
#     of processors for a Gaussian input file.  Number of processors 
#     is checked against current limits and redefines QSUBOPTS(-l,mpp_p)
#     if inconsistent.  If the input file is specified as the location
#     for the number of processors, but no specification is given,
#     then the file is left untouched.

# arg remHost> Host on which calculation performed
# arg gaussianFile> Gaussian input file name
# return> None
proc webSubmit::nqs::checkGaussianProc {remHost gaussianFile} {
    variable Gaussian
    variable CONFIG
    variable QSUBOPTS
    variable debug

    if {$debug} {
	puts "Verifying Gaussian Processors:"
	puts "  Remote Host: $remHost (maxProcessors = $CONFIG($remHost,maxProcessors))"
	puts "  Gaussian File: $gaussianFile"
    }

    # String used by Gaussian to define the number of processors
    set pstr "\%NProc="
    
    # Determine if number of processors specified in input file
    set reset 0
    if [file exists $gaussianFile] {
	if [catch {tclGrep $pstr $gaussianFile} procLine] {
	    # No processors specified
	    if {$Gaussian(NProcType) == "IF"} {
		# Allow Gaussian to determine default number of processors
		return
	    } else {
		if {$QSUBOPTS(-l,mpp_p) == ""} {
		    # Fix at maximum number for the current class
		    set QSUBOPTS(-l,mpp_p) $CONFIG($remHost,maxProcessors)
		}
	    }
	} else {
	    if {$Gaussian(NProcType) == "IF"} {
		# Extract Nproc from input file
		regexp "^${pstr}\(.*\)" $procLine match QSUBOPTS(-l,mpp_p)
	    }
	}
    } else {
	cgiBomb checkGaussianProc "Specified Gaussian input file \"$gaussianFile\" does not exist"
    }

    if {$debug} {
	puts "   Number of processors specified: $QSUBOPTS(-l,mpp_p)"
    }

    # Determine if processors in allowed range for current class
    if {$QSUBOPTS(-l,mpp_p) < 1 || $QSUBOPTS(-l,mpp_p) > $CONFIG($remHost,maxProcessors)} {
	cgiWarn checkGaussianProc "Number of processors specified exceeds the\
		maximum.  Reset to maximum for $remHost ($CONFIG($remHost,maxProcessors))"
	set QSUBOPTS(-l,mpp_p) $CONFIG($remHost,maxProcessors)
	set reset 1
    }

    if {$reset || $Gaussian(NProcType) == "UD"} {
	# Open Gaussian input file and determine number of processors
	if [catch {open "$gaussianFile" r} fid] {
	    cgiBomb checkGaussianProc "Could not open Gaussian input file ($fid)"
	} else {
	    # Read input file
	    set gaussianDataList [split [read $fid] \n]
	    set idx [lsearch -regexp $gaussianDataList $pstr]
	    if {$idx != -1} {
		# Input file already contains %Nproc --> Replace
		set gaussianDataList [lreplace $gaussianDataList $idx $idx "${pstr}$QSUBOPTS(-l,mpp_p)"]
	    } else {
		# Input file does not specify %Nproc --> Insert at top of file
		if {$Gaussian(NProcType) == "UD"} {
		    set gaussianDataList [linsert $gaussianDataList 0 "${pstr}$QSUBOPTS(-l,mpp_p)"]
		}
	    }
	    close $fid

	    # Rewrite Gaussian input file with correct number of processors
	    set fid [open $gaussianFile w]
	    foreach line $gaussianDataList {
		puts $fid $line
	    }
	    close $fid
	}
    }
}
























































































